spool disparadores.log

/*********************************************************************
TRIGGERS DE ESTADISTICAS
**********************************************************************/

/*******************************************************************************
NOMBRE: TRG_DISPOSITIVO_INSERT
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_DISPOSITIVOS_INSERT AFTER INSERT ON DISPOSITIVOS 
FOR EACH ROW 
DECLARE 

EXISTE VARCHAR2(1);

-- Descripcin del tipo de dispositivo
NOMBRE_TIPO TIPO_DISPOSITIVO.DESCRIPCION%TYPE;

BEGIN
/*******************************************************************************
PROPSITO: Cuando se da de alta un nuevo dispositivo, actualiza la tabla 
Estadsticas_Ao, incrementando en 1 el nmero de dispositivos Si el dispositivo
es un portatil, actualiza la tabla Portatiles_sin_averia, incrementado en 1 el
nmero de porttiles y el nmero de porttiles sin avera.
********************************************************************************/

-- Se obtiene la descripcin del tipo de dispositivo
SELECT DESCRIPCION
INTO NOMBRE_TIPO
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = :NEW.TIPO;

BEGIN

-- Si hay un registro en la tabla Estadisticas_Ao para el ao de la fecha de compra del dispositivo
SELECT 'S'
INTO EXISTE
FROM ESTADISTICAS_AO
WHERE AO = TO_CHAR(:NEW.FECHA_COMPRA, 'YYYY');

-- Se incrementa en 1 el nmero de dispositivos de la tabla Estadisticas_Ao para el ao de compra
UPDATE ESTADISTICAS_AO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS + 1
WHERE AO = TO_CHAR(:NEW.FECHA_COMPRA, 'YYYY');

EXCEPTION 

WHEN NO_DATA_FOUND THEN

-- Si no hay un registro para ese ao en la tabla Estadisticas_Ao
-- se inserta el ao, el nmero de dispositivos a 1 y el importe de los pedidos y el nmero
-- de averas a 0.
INSERT INTO ESTADISTICAS_AO VALUES (TO_CHAR(:NEW.FECHA_COMPRA, 'YYYY'),1,0,0);

END;

BEGIN
-- Se mira si existe algn registro en la tabla Portatiles_Sin_Averia
SELECT 'S'
INTO EXISTE
FROM PORTATILES_SIN_AVERIA
WHERE ROWNUM = 1;

-- Si el tipo del dispositivo es un portatil
IF (NOMBRE_TIPO = 'PORTATIL') THEN

-- Se incrementa el nmero de porttiles y el nmro de porttiles sin avera de la
-- tabla Portatiles_Sin_Averia
UPDATE PORTATILES_SIN_AVERIA
SET NUM_PORTATILES = NUM_PORTATILES + 1,
PORTATILES_SIN_AVERIA = PORTATILES_SIN_AVERIA + 1,
POR_SIN_AVERIA = ((PORTATILES_SIN_AVERIA + 1) /(NUM_PORTATILES +1))*100;

END IF;

EXCEPTION 
WHEN NO_DATA_FOUND THEN
-- Si no hay datos en la tabla Portatiles_Sin_Averia y el dispositivo es un portatil
-- se inserta en la tabla el nmero de portatiles y portatiles sin avera igual a 1
-- y el porcentaje igual a 100
IF (NOMBRE_TIPO = 'PORTATIL') THEN
INSERT INTO PORTATILES_SIN_AVERIA (NUM_PORTATILES, PORTATILES_SIN_AVERIA, POR_SIN_AVERIA)
VALUES (1, 1, 100);


END IF;
END;
END;

/

/*****************************************************************************
NOMBRE: TRG_DEPARTAMENTO_INSERT
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_DEPARTAMENTO_INSERT AFTER INSERT ON DEPARTAMENTOS 
FOR EACH ROW 
BEGIN

/*****************************************************************************
PROPSITO: Cuando se da de alta un nuevo departamento, inserta en la tabla 
Dispositivos_Activos_Departamento, el departamento con cada uno de los tipos de 
dispositivo activos existentes en la tabla Tipo_Dispositivo, con nmero de 
dispositivos igual a 0. Se inserta en la tabla Inventario_Departamento el
identificador y el nombre del departamento con nmero de tipos de dispositivos y
valor del inventario igual a 0.
********************************************************************************/

-- Se declara un cursor que contendr los tipos de dispositivos. Cada vez que se inserte
-- un departamento, se insertarn en la tabla Disp_Activos_Departamento tantos registros
-- como tipos de dispositivos haya, poniendo el nmero de dispositivos a 0.
DECLARE 
    CURSOR CURSORTIPODISP IS
    select ID_TIPO_DISPOSITIVO, DESCRIPCION 
    from TIPO_DISPOSITIVO
    WHERE FECHA_BAJA IS NULL;
    V_ID_TIPO_DISPOSITIVO TIPO_DISPOSITIVO.ID_TIPO_DISPOSITIVO%TYPE;
    V_DESCRIPCION TIPO_DISPOSITIVO.DESCRIPCION%TYPE;
  BEGIN
    OPEN CURSORTIPODISP;
    FETCH CURSORTIPODISP INTO V_ID_TIPO_DISPOSITIVO, V_DESCRIPCION;
    WHILE CURSORTIPODISP%FOUND
    LOOP
    INSERT INTO DISP_ACTIVOS_DEPARTAMENTO (ID_DEPARTAMENTO,ID_TIPO_DISPOSITIVO,NOMBRE_DEPARTAMENTO,
    DESCRIPCION_TIPO, NUM_DISPOSITIVOS)
    VALUES (:NEW.ID_DEPARTAMENTO,V_ID_TIPO_DISPOSITIVO,:NEW.NOMBRE_DEPARTAMENTO, V_DESCRIPCION, 0);
 FETCH CURSORTIPODISP INTO V_ID_TIPO_DISPOSITIVO, V_DESCRIPCION;
END LOOP;	
    CLOSE CURSORTIPODISP;
  
END;

-- Se insertan en la tabla inventario_departamento, el identificador del departamento,
-- el nombre del departamento y el nmero de tipos de dispositivos y el valor del inventario a 0
INSERT INTO INVENTARIO_DEPARTAMENTO(ID_DEPARTAMENTO, NOMBRE_DEPARTAMENTO, NUM_TIPOS_DISPOSITIVOS, VALOR_INVENTARIO)
VALUES (:NEW.ID_DEPARTAMENTO, :NEW.NOMBRE_DEPARTAMENTO, 0, 0);

END;

/

/*****************************************************************************
NOMBRE: TRG_DEPARTAMENTO_DELETE
********************************************************************************/
CREATE OR REPLACE
TRIGGER TRG_DEPARTAMENTO_DELETE AFTER UPDATE OF FECHA_BAJA ON DEPARTAMENTOS 
FOR EACH ROW 
BEGIN

/*****************************************************************************
PROPSITO: Se eliminan de la tabla Dispositivos_Activos_Departamento todas las instancias en las que el departamento sea el borrado. Se elimina de la tabla Inventario_Departamento el registro correspondiente al departamento borrado.
********************************************************************************/


DELETE DISP_ACTIVOS_DEPARTAMENTO
WHERE ID_DEPARTAMENTO = :OLD.ID_DEPARTAMENTO;

DELETE INVENTARIO_DEPARTAMENTO
WHERE ID_DEPARTAMENTO = :OLD.ID_DEPARTAMENTO;


END;

/

/*****************************************************************************
NOMBRE: TRG_PROVEEDOR_INSERT
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_PROVEEDOR_INSERT AFTER INSERT ON PROVEEDORES 
FOR EACH ROW 
DECLARE

EXISTE VARCHAR2(1);
-- Periodo de entrega del proveedor
PERIODO NUMBER;

BEGIN

/*****************************************************************************
PROPSITO: cuando se da de alta un proveedor, se comprueba si existen datos en
la tabla Proveedor_Mas_Rapido. Si no hay datos introduce los datos del proveedor. 
Si tiene datos, se comprueba si el periodo de entrega del proveedor que se va a 
dar de alta es menor que el de la tabla. Si es as, se borran los datos de la
tabla y se insertan los datos del nuevo proveedor.Si el periodo de entrega es
igual al que va a insertarse, se insertan en la tabla los nuevos datos. 
********************************************************************************/


-- Mira si hay datos en la tabla Proveedor_Mas_Rapido
SELECT 'S'
INTO EXISTE
FROM PROVEEDOR_MAS_RAPIDO
WHERE ROWNUM = 1;

-- Se obtiene el periodo de entrega del proveedor que est en la tabla Proveedor_Mas_Rapido
SELECT DISTINCT(PERIODO_ENTREGA)
INTO PERIODO
FROM PROVEEDOR_MAS_RAPIDO;

-- Si el periodo que hay en la tabla es mayor que el periodo de entrega del proveedor
IF (PERIODO > :NEW.PERIODO_ENTREGA)
THEN

-- Se borra la tabla Proveedor_Mas_Rapido
DELETE PROVEEDOR_MAS_RAPIDO;

-- Se inserta el nuevo proveedor en la tabla Proveedor_Mas_Rapido
INSERT INTO PROVEEDOR_MAS_RAPIDO (ID_PROVEEDOR, NOMBRE, PERIODO_ENTREGA)
VALUES (:NEW.ID_PROVEEDOR, :NEW.NOMBRE, :NEW.PERIODO_ENTREGA);

END IF;

-- Si el periodo de entrega del proveedor es igual que el de la tabla
IF (PERIODO = :NEW.PERIODO_ENTREGA) THEN

-- Se insertan los datos del proveedor en la tabla Proveedor_Mas_Rapido
INSERT INTO PROVEEDOR_MAS_RAPIDO (ID_PROVEEDOR, NOMBRE, PERIODO_ENTREGA)
VALUES (:NEW.ID_PROVEEDOR, :NEW.NOMBRE,:NEW.PERIODO_ENTREGA);

END IF;

-- Si no hay datos en la tabla Proveedor_Mas_Rapido
EXCEPTION
WHEN NO_DATA_FOUND THEN 

-- Insertamos los datos del proveedor en la tabla Proveedor_Mas_Rapido
INSERT INTO PROVEEDOR_MAS_RAPIDO (ID_PROVEEDOR, NOMBRE, PERIODO_ENTREGA)
VALUES (:NEW.ID_PROVEEDOR, :NEW.NOMBRE,:NEW.PERIODO_ENTREGA);


END;


/

/*****************************************************************************
NOMBRE: TRG_PROVEEDOR_DELETE
********************************************************************************/
CREATE OR REPLACE
TRIGGER TRG_PROVEEDOR_DELETE AFTER UPDATE OF FECHA_BAJA ON PROVEEDORES 
FOR EACH ROW 
DECLARE

EXISTE VARCHAR2(1);

-- Periodo de entrega (en dias) del proveedor
PERIODO  PROVEEDORES.PERIODO_ENTREGA%TYPE;


BEGIN

/*****************************************************************************
PROPSITO: Cuando se da de baja un proveedor, se comprueba si est en la tabla 
Proveedor_Mas_Rapido. Si es as, se borra y se inserta en esta tabla el proveedor 
ms rpido de los que han quedado en la tabla.
********************************************************************************/


-- Mira si el proveedor que se va a dar de baja es el proveedor ms rpido
SELECT 'S'
INTO EXISTE
FROM PROVEEDOR_MAS_RAPIDO
WHERE ID_PROVEEDOR = :OLD.ID_PROVEEDOR
AND ROWNUM = 1;

-- Si es as, se borra la tabla Proveedor_Mas_Rapido
DELETE PROVEEDOR_MAS_RAPIDO;

-- Se insertan los nuevos datos del proveedor mas rpido
INSERT INTO PROVEEDOR_MAS_RAPIDO 
SELECT ID_PROVEEDOR, NOMBRE, PERIODO_ENTREGA
FROM PROVEEDORES_AUX
WHERE PERIODO_ENTREGA IN 
(SELECT MIN(PERIODO_ENTREGA) FROM PROVEEDORES_AUX 
WHERE FECHA_BAJA IS NULL)
AND FECHA_BAJA IS NULL;

EXCEPTION
-- Si no es el proveedor ms rpido, no hacemos nada
WHEN NO_DATA_FOUND THEN NULL;

END;

/

/*****************************************************************************
NOMBRE: TRG_PROV_UPD_PERIODO
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_PROV_UPD_PERIODO AFTER UPDATE OF PERIODO_ENTREGA ON PROVEEDORES 
FOR EACH ROW 
DECLARE

-- Cuenta si el proveedor est en la tabla Proveedor_Mas_Rapido
CUENTA NUMBER;
-- Periodo de entrega de la tabla Proveedor_Mas_Rapido
PERIODO NUMBER;


BEGIN  

/*****************************************************************************
PROPSITO: Si se modifica el campo periodo_entrega de la tabla proveedores, se 
comprueba si ese proveedor es el ms rpido. Si es as, se borra la tabla 
Proveedor_Mas_Rapido y se insertan los datos del que tenga el menor periodo de 
entrega. Si no es el ms rpido, se comprueba si el nuevo periodo de entrega es 
menor que el que hay en la tabla Proveedor_Mas_Rapido. Si lo es, borra la tabla 
e inserta los datos del proveedor que se ha actualizado. Si es igual, inserta 
los datos del proveedor actualizado.
********************************************************************************/


-- Mira si el proveedor que se ha modificado est en la tabla Proveedor_Mas_Rapido

SELECT COUNT(*) 
INTO CUENTA
FROM PROVEEDOR_MAS_RAPIDO
WHERE ID_PROVEEDOR = :NEW.ID_PROVEEDOR;

-- Si el proveedor est en la tabla
IF (CUENTA > 0) THEN

-- Se borra la tabla Proveedor_Mas_Rapido
DELETE PROVEEDOR_MAS_RAPIDO;

-- Se inserta en la tabla el nuevo proveedor con el periodo de entrega ms corto
INSERT INTO PROVEEDOR_MAS_RAPIDO 
SELECT ID_PROVEEDOR, NOMBRE, PERIODO_ENTREGA
FROM PROVEEDORES_AUX
WHERE PERIODO_ENTREGA IN 
(SELECT MIN(PERIODO_ENTREGA) FROM PROVEEDORES_AUX 
WHERE FECHA_BAJA IS NULL)
AND FECHA_BAJA IS NULL;

ELSE

-- Mira si el nuevo periodo de entrega es mayor que el de la tabla

SELECT DISTINCT (PERIODO_ENTREGA) 
INTO PERIODO
FROM PROVEEDOR_MAS_RAPIDO;

-- Si el periodo de entrega es igual que el de la tabla
IF (:NEW.PERIODO_ENTREGA = PERIODO) THEN

-- Inserta en la tabla Proveedor_Mas_Rapido los datos del proveedor que se modifica
INSERT INTO PROVEEDOR_MAS_RAPIDO (ID_PROVEEDOR, NOMBRE, PERIODO_ENTREGA)
VALUES (:NEW.ID_PROVEEDOR, :NEW.NOMBRE, :NEW.PERIODO_ENTREGA);

END IF;

-- Si el periodo de entrega modificado es menor que el de la tabla 
IF (:NEW.PERIODO_ENTREGA < PERIODO ) THEN

BEGIN

-- Se borra la tabla Proveedor_Mas_Rapido
DELETE FROM PROVEEDOR_MAS_RAPIDO;

-- Se insertan los datos del proveeor en la tabla Proveedor_Mas_Rapido
INSERT INTO PROVEEDOR_MAS_RAPIDO (ID_PROVEEDOR, NOMBRE, PERIODO_ENTREGA)
VALUES (:NEW.ID_PROVEEDOR, :NEW.NOMBRE, :NEW.PERIODO_ENTREGA);

END;
END IF;
END IF;


END;

/

/*****************************************************************************
NOMBRE: TRG_TIPO_DISPOSITIVO_INSERT
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_TIPO_DISPOSITIVO_INSERT AFTER INSERT ON TIPO_DISPOSITIVO 
FOR EACH ROW 
BEGIN

/*****************************************************************************
PROPSITO: Despus de cada insercin en la tabla Tipo_Dispositivo, se insertan
en la tabla Dispositivos_Activos_Departamento todos los departamentos de la
empresa para el tipo de dispositivo creado. Se inserta en la tabla 
Unidades_Dispositivo_Almacen, el tipo de dispositivo creado, su descripcin y el
nmero de unidades que se encuentran en el almacn igual a 0. 
********************************************************************************/

-- Declaramos un cursor con los datos de los departamentos de la empresa
-- Cuando se da de alta un tipo de dispositivo, se inserta en la tabla
-- Disp_Activos_Departamento un registro para el tipo de dispositivo por cada departamento
DECLARE 
    CURSOR CURSORDEPART IS
    select ID_DEPARTAMENTO, NOMBRE_DEPARTAMENTO 
    from DEPARTAMENTOS
    WHERE FECHA_BAJA IS NULL;
    V_ID_DEPARTAMENTO VARCHAR2(3);
    V_NOMBRE_DEPARTAMENTO VARCHAR2(40);
  BEGIN
    OPEN CURSORDEPART;
    FETCH CURSORDEPART INTO V_ID_DEPARTAMENTO, V_NOMBRE_DEPARTAMENTO;
    WHILE CURSORDEPART%FOUND
    LOOP
    INSERT INTO DISP_ACTIVOS_DEPARTAMENTO (ID_DEPARTAMENTO,ID_TIPO_DISPOSITIVO,NOMBRE_DEPARTAMENTO,
    DESCRIPCION_TIPO, NUM_DISPOSITIVOS)
    VALUES (V_ID_DEPARTAMENTO,:NEW.ID_TIPO_DISPOSITIVO,V_NOMBRE_DEPARTAMENTO, :NEW.DESCRIPCION, 0);
    FETCH CURSORDEPART INTO V_ID_DEPARTAMENTO, V_NOMBRE_DEPARTAMENTO;
    END LOOP;
    CLOSE CURSORDEPART;
  
END;

-- Se insertan en la tabla Unidades_Dispositivo_Almacen, el tipo de dispositivo, la descripcin
-- y el nmero de unidades igual a 0
INSERT INTO UNIDADES_DISPOSITIVO_ALMACEN(ID_TIPO_DISPOSITIVO, DESCRIPCION, NUM_UNIDADES) 
VALUES (:NEW.ID_TIPO_DISPOSITIVO, :NEW.DESCRIPCION, 0);

END;

/

/*****************************************************************************
NOMBRE: TRG_TIPO_DISPOSITIVO_DELETE
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_TIPO_DISPOSITIVO_DELETE AFTER UPDATE OF FECHA_BAJA ON TIPO_DISPOSITIVO 
FOR EACH ROW 
BEGIN

/*****************************************************************************
PROPSITO: Cuando se da de baja un tipo de dispositivo, se eliminan de la tabla 
Dispositivos_Activos_Departamento y de la tabla Unidades_Dispositivo_Almacen 
todos los registros cuyo tipo de dispositivo es el borrado.
********************************************************************************/


DELETE FROM DISP_ACTIVOS_DEPARTAMENTO
WHERE ID_TIPO_DISPOSITIVO = :OLD.ID_TIPO_DISPOSITIVO;

DELETE FROM UNIDADES_DISPOSITIVO_ALMACEN
WHERE ID_TIPO_DISPOSITIVO = :OLD.ID_TIPO_DISPOSITIVO;


END;

/

/*****************************************************************************
NOMBRE: TRG_USUARIO_INSERT
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_USUARIO_INSERT AFTER INSERT ON USUARIOS 
FOR EACH ROW 
DECLARE 

EXISTE VARCHAR2(1);

BEGIN

/*****************************************************************************
PROPSITO: cuando se da de alta un usuario, inserta en la tabla Usuarios_Averias
el identificador del usuario y el nombre con nmero de averas igual a 0. Si la 
tabla Usuarios_Dispositivos est vaca, inserta los datos total usuarios y 
usuarios sin dispositivos igual a 1 y usuarios con mas de 5 dispositivos a 0. Si
la tabla no est vaca, incrementa en 1 tanto el nmero de usuarios como el 
nmero de usuarios sin dispositivos.
********************************************************************************/

-- Inserta el usuario en la tabla Usuarios_Averias, con nmro de averias igual a 0
INSERT INTO USUARIOS_AVERIAS (ID_USUARIO, NOMBRE_USUARIO, NUM_AVERIAS)
VALUES (:NEW.ID_USUARIO, :NEW.NOMBRE, 0);

-- Mira si hay datos en la tabla Usuarios_Dispositivos
SELECT 'S'
INTO EXISTE
FROM USUARIOS_DISPOSITIVOS
WHERE ROWNUM = 1;

-- Si es as, se incrementa el total de usuarios y los usuarios sin dispositivos
UPDATE USUARIOS_DISPOSITIVOS
SET TOTAL_USUARIOS = TOTAL_USUARIOS + 1,
USUARIOS_SIN_DISPOSITIVOS = USUARIOS_SIN_DISPOSITIVOS + 1,
POR_SIN_DISPOSITIVOS = ((USUARIOS_SIN_DISPOSITIVOS + 1) / (TOTAL_USUARIOS + 1))*100;

-- Si no hay datos en la tabla
EXCEPTION

WHEN NO_DATA_FOUND THEN

-- Se insertan en la tabla Usuarios_Dispositivos el total de usuarios igual a 1, 
-- los usuarios sin dispositivo igual a 1 y los usuarios con mas de 5 dispositivos
-- igual a 0.
INSERT INTO USUARIOS_DISPOSITIVOS (TOTAL_USUARIOS, USUARIOS_SIN_DISPOSITIVOS, 
POR_SIN_DISPOSITIVOS, USUARIOS_MAS5_DISPOSITIVOS) 
VALUES (1, 1, 100, 0);

END;

/

/*****************************************************************************
NOMBRE: TRG_USUARIO_DELETE
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_USUARIO_DELETE AFTER UPDATE OF FECHA_BAJA ON USUARIOS 
FOR EACH ROW 
DECLARE
EXISTE  VARCHAR2(1);

BEGIN

/*****************************************************************************
PROPSITO: cuando se da de baja un usuario, borra de la tabla Usuarios_Averias 
el registro correspondiente al usuario borrado.Si el usuario se encuentra en la
tabla Usuarios_Top10_Averias, borra la tabla y vuelve a recalcularla. Decrementa 
en 1 el total de usuarios de la tabla Usuarios_Dispositivos. Si el usuario no 
tiene asignado todava ningn dispositivo, decrementa el campo usuarios sin 
dispositivos de la tabla Usuarios_Dispositivos.
********************************************************************************/

-- Se borra el usuario de la tabla Usuarios_Averias
DELETE FROM USUARIOS_AVERIAS 
WHERE ID_USUARIO = :OLD.ID_USUARIO;

-- Mira si el usuario est en la tabla usuarios_top10_averias

BEGIN

SELECT 'S'
INTO EXISTE
FROM USUARIOS_TOP10_AVERIAS
WHERE ID_USUARIO = :OLD.ID_USUARIO
AND ROWNUM = 1;

-- Si est, se borra la tabla y se vuelve a recalcular

DELETE USUARIOS_TOP10_AVERIAS;

INSERT INTO USUARIOS_TOP10_AVERIAS 
SELECT ID_USUARIO, NOMBRE_USUARIO, NUM_AVERIAS,
DENSE_RANK()
OVER (ORDER BY NUM_AVERIAS DESC) DRNK
FROM USUARIOS_AVERIAS
ORDER BY NUM_AVERIAS DESC;

EXCEPTION 
WHEN NO_DATA_FOUND THEN NULL;

END;

-- Se decrementa el total de usuarios de la tabla Usuarios_Dispositivos
UPDATE USUARIOS_DISPOSITIVOS
SET TOTAL_USUARIOS = TOTAL_USUARIOS -1,
POR_SIN_DISPOSITIVOS = (USUARIOS_SIN_DISPOSITIVOS / (TOTAL_USUARIOS - 1))* 100;

BEGIN

-- Se mira si existe algn dispositivo que est asignado al usuario que se va a dar de baja
SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS
WHERE ID_USUARIO = :OLD.ID_USUARIO;

-- Si es as, se pone a null el id_usuario en todos los dispositivos que estn asignados al usuario
UPDATE DISPOSITIVOS
SET ID_USUARIO = NULL
WHERE ID_DISPOSITIVO IN
(SELECT ID_DISPOSITIVO FROM DISPOSITIVOS 
WHERE ID_USUARIO = :OLD.ID_USUARIO);

-- Si el usuario no tena ningn dispositivo asignado
EXCEPTION
WHEN NO_DATA_FOUND THEN

-- Se decrementan los usuarios sin dispositivos de la tabla Usuarios_Dispositivos
UPDATE USUARIOS_DISPOSITIVOS
SET USUARIOS_SIN_DISPOSITIVOS = USUARIOS_SIN_DISPOSITIVOS -1,
POR_SIN_DISPOSITIVOS = ((USUARIOS_SIN_DISPOSITIVOS - 1) / TOTAL_USUARIOS)* 100;

END;

END;

/

/*********************************************************************************
NOMBRE: TRG_PEDIDO_INSERT
*********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_PEDIDO_INSERT AFTER INSERT ON PEDIDOS 
FOR EACH ROW 
DECLARE 

EXISTE VARCHAR2(1);

BEGIN

/*********************************************************************************
PROPSITO: Siempre que el pedido est ligado a una orden de inversin, si el ao
del pedido no existe en la tabla Estadisticas_ao, inserta el ao y pone el
nmero de dispositivos a 0, el importe del pedido con el importe del pedido y el
nmero de averas a 0. Si el ao ya existe, suma el importe del pedido al campo
importe_pedido. Si es un pedido que no est ligado a una orden de inversin,
inserta el ao con el nmero de dispositivos, el importe del pedido y el nmero
de averas a 0.
*********************************************************************************/

-- Mira si hay algn registro en la tabla Estadisticas_Ao para el ao del pedido
SELECT 'S'
INTO EXISTE
FROM ESTADISTICAS_AO
WHERE AO = TO_CHAR(:NEW.FECHA_PEDIDO,'YYYY')
AND ROWNUM = 1;

-- Si el pedido est ligado a una orden de inversin
IF :NEW.ORDEN_INVERSION IS NOT NULL 
THEN
-- Se incrementa el importe pedidos de la tabla Estadisticas_Ao con el importe del pedido
-- que se va a dar de alta para el ao del pedido
UPDATE ESTADISTICAS_AO
SET IMPORTE_PEDIDOS = IMPORTE_PEDIDOS + :NEW.IMPORTE_PEDIDO
WHERE AO = TO_CHAR(:NEW.FECHA_PEDIDO,'YYYY');
END IF;
-- Si no hay registros en la tabla Estadisticas_Ao
EXCEPTION
WHEN NO_DATA_FOUND THEN

-- Si el pedido est ligado a una orden de inversin
IF :NEW.ORDEN_INVERSION IS NOT NULL
THEN

-- Se inserta en la tabla Estadisticas_Ao, el ao, el nmero de dispositivos igual a 0, 
-- el importe del pedido y el nmero de averias igual a 0.
INSERT INTO ESTADISTICAS_AO (AO, NUM_DISPOSITIVOS, IMPORTE_PEDIDOS, NUM_AVERIAS) VALUES
(TO_CHAR(:NEW.FECHA_PEDIDO,'YYYY'),0, :NEW.IMPORTE_PEDIDO, 0);

-- Si el pedido no est ligado a una orden de inversin
ELSE

-- Se inserta en la tabla el ao del pedido y el resto de los campos a 0.
INSERT INTO ESTADISTICAS_AO (AO, NUM_DISPOSITIVOS, IMPORTE_PEDIDOS, NUM_AVERIAS) VALUES
(TO_CHAR(:NEW.FECHA_PEDIDO,'YYYY'),0, 0, 0);
END IF;
END;

/

/*********************************************************************************
NOMBRE: TRG_PEDIDO_DELETE
*********************************************************************************/
CREATE OR REPLACE
TRIGGER TRG_PEDIDO_DELETE AFTER DELETE ON PEDIDOS 
FOR EACH ROW 
DECLARE 

EXISTE VARCHAR2(1);

BEGIN

/*********************************************************************************
PROPSITO: Siempre que el pedido est ligado a una orden de inversin, resta el
importe del pedido al importe de los pedidos para ese ao de la tabla 
Estadisticas_ao.
*********************************************************************************/
-- Si el pedido va ligado a una orden de inversin
IF (:OLD.ORDEN_INVERSION IS NOT NULL)
THEN

-- Se modifica la tabla Estadisticas_Ao, restando el importe del pedido que se va a borrar
-- del importe de los pedidos ligados a una orden de inversin de ese ao
UPDATE ESTADISTICAS_AO
SET IMPORTE_PEDIDOS = IMPORTE_PEDIDOS - :OLD.IMPORTE_PEDIDO
WHERE AO = TO_CHAR(:OLD.FECHA_PEDIDO,'YYYY');
END IF;

END;


/

/*********************************************************************************
NOMBRE: TRG_PEDIDO_UPD_IMPORTE
*********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_PEDIDO_UPD_IMPORTE AFTER UPDATE OF IMPORTE_PEDIDO ON PEDIDOS 
FOR EACH ROW 
BEGIN

/*********************************************************************************
PROPSITO: Siempre que el pedido est ligado a una orden de inversin, resta el
anterior importe del pedido al importe de los pedidos para ese ao de la
tabla Estadisticas_ao.
*********************************************************************************/


-- Si el pedido est asociado a una orden de inversin
IF (:OLD.ORDEN_INVERSION IS NOT NULL)

THEN

-- Se actualiza la tabla Estadisticas_Ao con el nuevo importe del pedido para el
-- ao del pedido.
UPDATE ESTADISTICAS_AO
SET IMPORTE_PEDIDOS = IMPORTE_PEDIDOS - :OLD.IMPORTE_PEDIDO
WHERE AO = TO_CHAR(:NEW.FECHA_PEDIDO,'YYYY');

END IF;

END;

/

/*********************************************************************************
NOMBRE: TRG_PEDIDO_UPD_ORDEN
*********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_PEDIDO_UPD_ORDEN AFTER UPDATE OF ORDEN_INVERSION ON PEDIDOS 
FOR EACH ROW 
DECLARE 

BEGIN

/*********************************************************************************
PROPSITO: Si el pedido no corresponde a una orden de inversin y se modifica el 
campo orden_inversion, actualiza la tabla Estadisticas_Ao, sumando al importe
de los pedidos, de ese ao, el importe del pedido modificado.
*********************************************************************************/

-- Si se modifica el campo orden de inversin de un pedido
IF (:NEW.ORDEN_INVERSION IS NOT NULL)

THEN

-- Se aade el importe del pedido al importe de los pedidos de la tabla Estadisticas_Ao
-- para el ao del pedido.
UPDATE ESTADISTICAS_AO
SET IMPORTE_PEDIDOS = IMPORTE_PEDIDOS + :NEW.IMPORTE_PEDIDO
WHERE AO = TO_CHAR(:NEW.FECHA_PEDIDO,'YYYY');
END IF;


END;

/

/*****************************************************************************
NOMBRE: TRG_MOVIMIENTO_INSERT
********************************************************************************/


CREATE OR REPLACE
TRIGGER TRG_MOVIMIENTO_INSERT AFTER INSERT ON MOVIMIENTOS 
FOR EACH ROW 
DECLARE 

-- Se declaran las variables que van a ser necesarias para el funcionamiento del disparador
EXISTE VARCHAR2(1);
-- Nmero de averas de un dispositivo
AVERIA NUMBER;
-- Descripcin del tipo de movimiento
DESCRIP_MOVI TIPO_MOVIMIENTO.DESCRIPCION%TYPE;
-- Descripcin del tipo de dispositivo
DESCRIP_DISP TIPO_DISPOSITIVO.DESCRIPCION%TYPE;
-- Identificador del tipo de dispositivo
TIPODISP TIPO_DISPOSITIVO.ID_TIPO_DISPOSITIVO%TYPE;
-- Identificador del departamento del usuario actual
DEPART DEPARTAMENTOS.ID_DEPARTAMENTO%TYPE;
-- Identificador del departamento del usuario anterior
DEPART_OLD DEPARTAMENTOS.ID_DEPARTAMENTO%TYPE;
-- Identificador del departamento que tiene mas reasignaciones
DEPART_REASIG DEPTO_MAS_REASIGNACIONES.ID_DEPARTAMENTO%TYPE;
-- Nombre del departamento
NOMBRE_DEPART DEPARTAMENTOS.NOMBRE_DEPARTAMENTO%TYPE;
-- Nmero de tipos de dispositivos del departamento del usuario actual
NUM_DISP_DEPART NUMBER;
-- Nmero de tipos de dispositivos del departamento del usuario anterior
NUM_DISP_DEPART_OLD NUMBER;
-- Nmero de usuarios con ms de 5 dispositivos asignados
NUM_USUARIOS NUMBER;
-- Valor del inventario del departamento del usuario actual
VALOR_INV NUMBER;
-- Valor del inventario del departamento del usuario anterior
VALOR_INV_OLD NUMBER;
-- Nmero de  reasignaciones de la tabla Depto_Mas_Reasignaciones
TOTAL_REASIG NUMBER;
-- Total reasignaciones en la tabla movimientos 
REASIGNACIONES NUMBER;
-- Nmero de dispositivos del usuario actual
NUM_DISP_USUARIO NUMBER;
-- Nmero de dispositivos del usuario anterior
NUM_DISP_USUARIO_OLD NUMBER;
-- Ao de compra del dispositivo
AO_COMPRA VARCHAR2(4);

BEGIN

/*****************************************************************************
PROPSITO: Cuando se da de alta un nuevo movimiento de un dispositivo, 
dependiendo del tipo de movimiento de que se trate, se actualizan diferentes 
tablas del repositorio estadstico.
********************************************************************************/
-- Se obtiene la descripcin del tipo de movimiento
SELECT DESCRIPCION
INTO DESCRIP_MOVI
FROM TIPO_MOVIMIENTO
WHERE ID_TIPO_MOVIMIENTO = :NEW.ID_TIPO_MOVIMIENTO;

-- Se obtiene el tipo de dispositivo para el dispositivo del movimiento
SELECT TIPO
INTO TIPODISP
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;

-- Se obtiene la descripcin del tipo de dispositivo
SELECT DESCRIPCION
INTO DESCRIP_DISP
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = TIPODISP;

-- Si el usuario destino no es nulo
IF (:NEW.ID_USUARIO_DESTINO IS NOT NULL)

THEN
-- Se obtiene el identificador del departamento del usuario destino
SELECT ID_DEPARTAMENTO
INTO DEPART
FROM USUARIOS 
WHERE ID_USUARIO = :NEW.ID_USUARIO_DESTINO;

-- Se obtiene el nombre del departamento del usuario destino
SELECT NOMBRE_DEPARTAMENTO
INTO NOMBRE_DEPART
FROM DEPARTAMENTOS
WHERE ID_DEPARTAMENTO = DEPART;

END IF;

-- Si el movimiento es un alta

IF (DESCRIP_MOVI = 'ALTA') THEN

BEGIN

-- Se incrementan las unidades de dispositivso en el almacn para el tipo de dispositivo
UPDATE UNIDADES_DISPOSITIVO_ALMACEN
SET NUM_UNIDADES = NUM_UNIDADES + 1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP;

-- Se asigna la fecha del movimiento a la fecha del ltimo inventario del dispositivo
UPDATE DISPOSITIVOS
SET FECHA_ULTIMO_INVENTARIO = :NEW.FECHA_MOVIMIENTO
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;

END;

END IF;

-- Si el movimiento corresponde a una baja

IF (DESCRIP_MOVI = 'BAJA') THEN

BEGIN

-- Se actualiza la tabla Dispositivos, rellenando la fecha de baja para el 
--dispositivo que se ha dado de baja

UPDATE DISPOSITIVOS
SET FECHA_BAJA = :NEW.FECHA_MOVIMIENTO
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;

-- Se obtiene el ao de compra del dispositivo, de la tabla dispositivos

SELECT TO_CHAR(FECHA_COMPRA,'YYYY')
INTO AO_COMPRA
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;

-- Se actualiza la tabla Estadisticas_Ao, decrementando en 1 el nmero de dispositivos de ese ao

UPDATE ESTADISTICAS_AO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS -1
WHERE AO = AO_COMPRA;

-- Mira si el dispositivo que se va a dar de baja tiene algn usuario asignado

BEGIN

SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO
AND ID_USUARIO IS NULL
AND ROWNUM = 1;

-- Si no tiene ningn usuario asignado, decrementa en 1 las unidades del tipo de dispositivo en el almacn

UPDATE UNIDADES_DISPOSITIVO_ALMACEN
SET NUM_UNIDADES = NUM_UNIDADES - 1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP;

END;
-- Si el dispositivo es un portatil se actualiza la tabla Portatiles_Sin_Averia

IF (DESCRIP_DISP = 'PORTATIL') THEN

-- Se cuentan las averas que tiene el dispositivo
SELECT COUNT(*)  INTO AVERIA
FROM MOVIMIENTOS_AUX M, DISPOSITIVOS D
WHERE M.ID_DISPOSITIVO = D.ID_DISPOSITIVO
AND M.ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO 
AND ID_TIPO_MOVIMIENTO IN
(SELECT ID_TIPO_MOVIMIENTO FROM TIPO_MOVIMIENTO
WHERE DESCRIPCION='REPARACION DE AVERIAS');


IF (AVERIA >= 1) THEN

-- Se decremena el nmero de porttiles de la tabla Portatiles_Sin_Averia
UPDATE PORTATILES_SIN_AVERIA
SET NUM_PORTATILES = NUM_PORTATILES -1,
POR_SIN_AVERIA = (PORTATILES_SIN_AVERIA  / (NUM_PORTATILES - 1))*100;

ELSE

-- Se decrementa el nmero de porttiles de la tabla Porttiles_Sin_Averia
-- Se decrementa el nmero de porttiles sin avera de la misma tabla
UPDATE PORTATILES_SIN_AVERIA
SET NUM_PORTATILES = NUM_PORTATILES -1,
PORTATILES_SIN_AVERIA = PORTATILES_SIN_AVERIA - 1,
POR_SIN_AVERIA = ((PORTATILES_SIN_AVERIA - 1) / (NUM_PORTATILES - 1))*100;

END IF;
END IF;

END;

END IF;

-- Si el movimiento corresponde a una actualizacin

IF (DESCRIP_MOVI = 'ACTUALIZACION') THEN

BEGIN

-- Se decrementa en 1 el nmero de dispositivos con fecha de inventario mayor de un ao

UPDATE DISPOSITIVOS_FECHA_INVENTARIO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS - 1;

-- Se actualiza la fecha_ultimo_inventario de la tabla dispositivos con la fecha del movimiento

UPDATE DISPOSITIVOS
SET FECHA_ULTIMO_INVENTARIO = :NEW.FECHA_MOVIMIENTO
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;

END;
END IF;

-- Si el movimiento correponde a una avera

IF (DESCRIP_MOVI = 'REPARACION DE AVERIAS') THEN

BEGIN

-- Se incrementa el nmero de averas de ese ao en la tabla Estadisticas_Ao

UPDATE ESTADISTICAS_AO
SET NUM_AVERIAS = NUM_AVERIAS + 1
WHERE AO = TO_CHAR(:NEW.FECHA_MOVIMIENTO, 'YYYY');

--  Se incrementa el nmero de averias del usuario que tiene asignado el dispositivo

UPDATE USUARIOS_AVERIAS
SET NUM_AVERIAS = NUM_AVERIAS +1
WHERE ID_USUARIO = :NEW.ID_USUARIO_DESTINO;

-- Si el dispositivo es un portatil, se decrementa el campo portatiles_sin_averia de 
--la misma tabla

IF (DESCRIP_DISP = 'PORTATIL') THEN

SELECT COUNT(*) INTO AVERIA
FROM MOVIMIENTOS_AUX M, DISPOSITIVOS D
WHERE M.ID_DISPOSITIVO = D.ID_DISPOSITIVO
AND M.ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO 
AND ID_TIPO_MOVIMIENTO IN
(SELECT ID_TIPO_MOVIMIENTO FROM TIPO_MOVIMIENTO
WHERE DESCRIPCION='REPARACION DE AVERIAS');

-- Si es la primera avera que ha tenido el dispositivo
IF (AVERIA = 1) THEN
-- Se decrementa el campo portatiles_sin_averia
UPDATE PORTATILES_SIN_AVERIA
SET PORTATILES_SIN_AVERIA = PORTATILES_SIN_AVERIA - 1,
POR_SIN_AVERIA = ((PORTATILES_SIN_AVERIA - 1) / NUM_PORTATILES)*100;

END IF;
END IF;
-- Calcula el ranking de los 10 usuarios que tienen ms averas y los introduce en la tabla Usuarios_Top10_Averias

DELETE USUARIOS_TOP10_AVERIAS;

INSERT INTO USUARIOS_TOP10_AVERIAS 
SELECT ID_USUARIO, NOMBRE_USUARIO, NUM_AVERIAS,
DENSE_RANK()
OVER (ORDER BY NUM_AVERIAS DESC) DRNK
FROM USUARIOS_AVERIAS
ORDER BY NUM_AVERIAS DESC;

END;

END IF;

-- Si el movimiento corresponde a una asignacin

IF (DESCRIP_MOVI = 'ASIGNACION') THEN

BEGIN

-- Se actualiza el campo id_usuario y el campo fecha instalacin de la tabla dispositivos 

UPDATE DISPOSITIVOS
SET ID_USUARIO = :NEW.ID_USUARIO_DESTINO,
FECHA_INSTALACION = :NEW.FECHA_MOVIMIENTO
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;

-- Se decrementa el nmero de dispositivos en el almacn.

UPDATE UNIDADES_DISPOSITIVO_ALMACEN
SET NUM_UNIDADES = NUM_UNIDADES - 1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP;

-- Si el usuario no tena asignado ningn otro dispositivo
-- Se decrementa el nmero de usuarios sin dispositivos de la tabla Usuarios_Dispositivos

BEGIN

SELECT COUNT(*)
INTO NUM_DISP_USUARIO
FROM DISPOSITIVOS
WHERE ID_USUARIO = :NEW.ID_USUARIO_DESTINO;

IF NUM_DISP_USUARIO = 1 THEN

UPDATE USUARIOS_DISPOSITIVOS
SET USUARIOS_SIN_DISPOSITIVOS = USUARIOS_SIN_DISPOSITIVOS -1,
POR_SIN_DISPOSITIVOS = ((USUARIOS_SIN_DISPOSITIVOS -1) / TOTAL_USUARIOS)*100;

END IF;

END;

-- Se incrementa en 1 el nmero de dispositivos para ese tipo y ese departamento

UPDATE DISP_ACTIVOS_DEPARTAMENTO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS + 1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP
AND ID_DEPARTAMENTO = DEPART;


-- Se cuentan los tipos diferentes de dispositivos que hay en el departamento y se 
-- actualiza la tabla Inventario_Departamento

SELECT  COUNT(DISTINCT (TIPO))
INTO NUM_DISP_DEPART
FROM DISPOSITIVOS
WHERE TIPO IN
(SELECT D.TIPO 
FROM DISPOSITIVOS D, USUARIOS U 
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART
GROUP BY  D.TIPO);

SELECT NVL(SUM(D.PRECIO_COMPRA),0)
INTO VALOR_INV
FROM DISPOSITIVOS D, USUARIOS U
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART
AND D.FECHA_BAJA IS NULL;


UPDATE INVENTARIO_DEPARTAMENTO
SET NUM_TIPOS_DISPOSITIVOS = NUM_DISP_DEPART,
VALOR_INVENTARIO = VALOR_INV
WHERE ID_DEPARTAMENTO = DEPART;

--Se actualiza la tabla Depto_Mas_TiposDisp con el departamento que tiene mas tipos de dispositivos

DELETE DEPTO_MAS_TIPOSDISP;

INSERT INTO DEPTO_MAS_TIPOSDISP
SELECT ID_DEPARTAMENTO, NOMBRE_DEPARTAMENTO, NUM_TIPOS_DISPOSITIVOS
FROM INVENTARIO_DEPARTAMENTO
WHERE NUM_TIPOS_DISPOSITIVOS IN
(SELECT MAX(NUM_TIPOS_DISPOSITIVOS) FROM INVENTARIO_DEPARTAMENTO);


-- Se cuentan los usuarios que tienen asignados mas de 5 dispositivos y se actualiza 
-- la tabla Usuarios_Dispositivos

SELECT COUNT(DISTINCT (ID_USUARIO))
INTO NUM_USUARIOS
FROM DISPOSITIVOS 
WHERE ID_USUARIO IN
(SELECT ID_USUARIO FROM DISPOSITIVOS
GROUP BY ID_USUARIO
HAVING COUNT(ID_USUARIO) > 5);


UPDATE USUARIOS_DISPOSITIVOS
SET USUARIOS_MAS5_DISPOSITIVOS = NUM_USUARIOS;

END;
END IF;

-- Si el movimiento corresponde a una reasignacin

IF (DESCRIP_MOVI = 'REASIGNACION') THEN

BEGIN


SELECT COUNT(*)
INTO NUM_DISP_USUARIO_OLD
FROM DISPOSITIVOS
WHERE ID_USUARIO = :NEW.ID_USUARIO_ORIGEN;

IF (NUM_DISP_USUARIO_OLD = 1) THEN

UPDATE USUARIOS_DISPOSITIVOS
SET USUARIOS_SIN_DISPOSITIVOS = USUARIOS_SIN_DISPOSITIVOS + 1,
POR_SIN_DISPOSITIVOS = ((USUARIOS_SIN_DISPOSITIVOS + 1) / TOTAL_USUARIOS)*100;

END IF;

-- Si el usuario destino no tena asignado ningn otro dispositivo
-- Se decrementa el nmero de usuarios sin dispositivos de la tabla Usuarios_Dispositivos


SELECT COUNT(*)
INTO NUM_DISP_USUARIO
FROM DISPOSITIVOS
WHERE ID_USUARIO = :NEW.ID_USUARIO_DESTINO;

IF NUM_DISP_USUARIO = 0 THEN

UPDATE USUARIOS_DISPOSITIVOS
SET USUARIOS_SIN_DISPOSITIVOS = USUARIOS_SIN_DISPOSITIVOS -1,
POR_SIN_DISPOSITIVOS = ((USUARIOS_SIN_DISPOSITIVOS -1) / TOTAL_USUARIOS)*100;

END IF;

-- Se actualiza el campo id_usuario de la tabla dispositivos 

UPDATE DISPOSITIVOS
SET ID_USUARIO = :NEW.ID_USUARIO_DESTINO
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;

-- Si el usuario origen no tiene asignado ningn otro dispositivo
-- Se incrementa el nmero de usuarios sin dispositivos de la tabla Usuarios_Dispositivos



-- Se mira si el departamento de los dos usuarios es distinto

SELECT ID_DEPARTAMENTO
INTO DEPART_OLD
FROM USUARIOS 
WHERE ID_USUARIO = :NEW.ID_USUARIO_ORIGEN;

-- Si son distintos se actualizan los datos para los dos departamentos

IF (DEPART <> DEPART_OLD) THEN

BEGIN

-- Se actualiza la tabla Dispositivos_Activos_Departamento para el departamento destino

UPDATE DISP_ACTIVOS_DEPARTAMENTO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS +1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP
AND ID_DEPARTAMENTO = DEPART;

-- Se cuentan los tipos diferentes de dispositivos que hay en el departamento destino y se actualiza la tabla Inventario_Departamento

SELECT  COUNT(DISTINCT (TIPO))
INTO NUM_DISP_DEPART
FROM DISPOSITIVOS
WHERE TIPO IN
(SELECT D.TIPO 
FROM DISPOSITIVOS D, USUARIOS U 
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART
GROUP BY  D.TIPO);

-- Se calcula el valor del inventario del departamento destino

SELECT NVL(SUM(D.PRECIO_COMPRA),0)
INTO VALOR_INV
FROM DISPOSITIVOS D, USUARIOS U
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART
AND D.FECHA_BAJA IS NULL;

-- Se actualiza la tabla Inventario_Departamento para el departamento destino

UPDATE INVENTARIO_DEPARTAMENTO
SET NUM_TIPOS_DISPOSITIVOS = NUM_DISP_DEPART,
VALOR_INVENTARIO = VALOR_INV
WHERE ID_DEPARTAMENTO = DEPART;

-- Se decrementa el nmero de dispositivos de la tabla Dispositivos_Activos_Departamento
-- para el departamento origen

UPDATE DISP_ACTIVOS_DEPARTAMENTO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS -1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP
AND ID_DEPARTAMENTO = DEPART_OLD;


-- Se recalcula el valor del inventario y el nmero de tipos de dispositivo para el departamento del usuario anterior

SELECT  COUNT(DISTINCT (TIPO))
INTO NUM_DISP_DEPART_OLD
FROM DISPOSITIVOS
WHERE TIPO IN
(SELECT D.TIPO 
FROM DISPOSITIVOS D, USUARIOS U 
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART_OLD
GROUP BY  D.TIPO);

SELECT NVL(SUM(D.PRECIO_COMPRA),0)
INTO VALOR_INV_OLD
FROM DISPOSITIVOS D, USUARIOS U
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART_OLD
AND D.FECHA_BAJA IS NULL;

-- Se actualiza la tabla Inventario_Departamento para el departamento del usuario anterior

UPDATE INVENTARIO_DEPARTAMENTO
SET NUM_TIPOS_DISPOSITIVOS = NUM_DISP_DEPART_OLD,
VALOR_INVENTARIO = VALOR_INV_OLD
WHERE ID_DEPARTAMENTO = DEPART_OLD;

-- Se actualiza la tabla Depto_Mas_TiposDisp con el departamento que tiene mas tipos de dispositivos

DELETE DEPTO_MAS_TIPOSDISP;

INSERT INTO DEPTO_MAS_TIPOSDISP
SELECT ID_DEPARTAMENTO, NOMBRE_DEPARTAMENTO, NUM_TIPOS_DISPOSITIVOS
FROM INVENTARIO_DEPARTAMENTO
WHERE NUM_TIPOS_DISPOSITIVOS IN
(SELECT MAX(NUM_TIPOS_DISPOSITIVOS) FROM INVENTARIO_DEPARTAMENTO);


END;
END IF;

-- Se mira si hay registros en la tabla Depto_Mas_Reasignaciones

BEGIN

SELECT 'S'
INTO EXISTE
FROM DEPTO_MAS_REASIGNACIONES
WHERE AO = TO_CHAR(:NEW.FECHA_MOVIMIENTO,'YYYY')
AND ROWNUM = 1;


-- Se recoge el numero de reasignaciones que hay en la tabla para ese ao

SELECT DISTINCT( NUM_REASIGNACIONES)
INTO  TOTAL_REASIG
FROM DEPTO_MAS_REASIGNACIONES
WHERE AO = TO_CHAR(:NEW.FECHA_MOVIMIENTO,'YYYY');


-- Se cuentan el nmero de reasignaciones para el departamento del usuario destino

SELECT COUNT(*)
INTO REASIGNACIONES
FROM MOVIMIENTOS_AUX M, USUARIOS U
WHERE M.ID_USUARIO_DESTINO = U.ID_USUARIO
AND M.ID_TIPO_MOVIMIENTO IN
(SELECT ID_TIPO_MOVIMIENTO FROM TIPO_MOVIMIENTO
WHERE DESCRIPCION ='REASIGNACION')
AND U.ID_DEPARTAMENTO = DEPART
AND TO_CHAR(M.FECHA_MOVIMIENTO,'YYYY') = TO_CHAR(:NEW.FECHA_MOVIMIENTO,'YYYY')
GROUP BY U.ID_DEPARTAMENTO;

IF (REASIGNACIONES > TOTAL_REASIG) THEN

-- Si el nmero de reasignaciones de este departamento es mayor, se borra la tabla para ese ao

DELETE DEPTO_MAS_REASIGNACIONES
WHERE AO = TO_CHAR(:NEW.FECHA_MOVIMIENTO,'YYYY');

-- Se insertan en la tabla los datos del departamento

INSERT INTO DEPTO_MAS_REASIGNACIONES (ID_DEPARTAMENTO, AO, NOMBRE_DEPARTAMENTO, 
NUM_REASIGNACIONES)
VALUES (DEPART, TO_CHAR(:NEW.FECHA_MOVIMIENTO,'YYYY'),
NOMBRE_DEPART, REASIGNACIONES);

END IF;


-- Si el nmero de reasignaciones es igual, insertamos el departamento

IF (REASIGNACIONES = TOTAL_REASIG ) THEN

INSERT INTO DEPTO_MAS_REASIGNACIONES (ID_DEPARTAMENTO, AO, NOMBRE_DEPARTAMENTO, 
NUM_REASIGNACIONES)
VALUES (DEPART, TO_CHAR(:NEW.FECHA_MOVIMIENTO,'YYYY'),
NOMBRE_DEPART, REASIGNACIONES);

END IF;

EXCEPTION 
WHEN NO_DATA_FOUND THEN 
-- Si no hay ningn departamento en la tabla reasignaciones, insertamos los datos
-- del departamento destino
INSERT INTO DEPTO_MAS_REASIGNACIONES (ID_DEPARTAMENTO, AO, NOMBRE_DEPARTAMENTO, 
NUM_REASIGNACIONES)
VALUES (DEPART, TO_CHAR(:NEW.FECHA_MOVIMIENTO,'YYYY'),
NOMBRE_DEPART, 1);


END;

-- Se cuentan los usuarios que tienen asignados mas de 5 dispositivos y se actualiza la tabla Usuarios_Dispositivos

SELECT COUNT(DISTINCT (ID_USUARIO))
INTO NUM_USUARIOS
FROM DISPOSITIVOS 
WHERE ID_USUARIO IN
(SELECT ID_USUARIO FROM DISPOSITIVOS
GROUP BY ID_USUARIO
HAVING COUNT(ID_USUARIO) > 5);

UPDATE USUARIOS_DISPOSITIVOS
SET USUARIOS_MAS5_DISPOSITIVOS = NUM_USUARIOS;



END;

END IF; 

END;


/

/*****************************************************************************
NOMBRE: TRG_MOVIMIENTO_DELETE
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_MOVIMIENTO_DELETE AFTER DELETE ON MOVIMIENTOS 
FOR EACH ROW 
DECLARE 

-- Se declaran las variables que son necesarias para el funcionamiento del disparador
EXISTE VARCHAR2(1);
-- Nmero de averas del dispositivo
AVERIA NUMBER;
-- Descripcin del tipo de movimiento
DESCRIP_MOVI TIPO_MOVIMIENTO.DESCRIPCION%TYPE;
-- Descripcin del tipo de dispositivo
DESCRIP_DISP TIPO_DISPOSITIVO.DESCRIPCION%TYPE;
-- Identificador del tipo de dispositivo
TIPODISP TIPO_DISPOSITIVO.ID_TIPO_DISPOSITIVO%TYPE;
-- Departamento al que pertenece el usuario destino
DEPART DEPARTAMENTOS.ID_DEPARTAMENTO%TYPE;
-- Nmero de tipos de dispositivos que tiene el departamento
NUM_DISP_DEPART NUMBER;
-- Nmero de usuarios con mas de 5 dispositivos asignados
NUM_USUARIOS NUMBER;
-- Valor del inventario del departamento del usuario destino
VALOR_INV NUMBER;
-- Nmero de dispositivos que tiene asignados el usuario destino
NUM_DISP_USUARIO NUMBER;

BEGIN

/*****************************************************************************
PROPSITO: Cuando se da de baja un movimiento de un dispositivo, dependiendo del
tipo de movimiento de que se trate, se actualizan diferentes tablas del 
repositorio estadstico.
********************************************************************************/

-- Se obtiene la descripcin del tipo de movimiento

SELECT DESCRIPCION
INTO DESCRIP_MOVI
FROM TIPO_MOVIMIENTO
WHERE ID_TIPO_MOVIMIENTO = :OLD.ID_TIPO_MOVIMIENTO;

-- Se obtiene el identificador del tipo de dispositivo
SELECT TIPO
INTO TIPODISP
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = :OLD.ID_DISPOSITIVO;

-- Se obtiene la descripcin del tipo de dispositivo
SELECT DESCRIPCION
INTO DESCRIP_DISP
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = TIPODISP;

-- Si el usuario_destino no es nulo
IF (:OLD.ID_USUARIO_DESTINO IS NOT NULL)

THEN
-- Se obtiene el departamento del usuario destino de la tabla usuarios
SELECT ID_DEPARTAMENTO
INTO DEPART
FROM USUARIOS 
WHERE ID_USUARIO = :OLD.ID_USUARIO_DESTINO;

END IF;


-- Si el movimiento es un alta

IF (DESCRIP_MOVI = 'ALTA') THEN

BEGIN

-- Se decrementan el nmero de unidades en el almacn para el tipo de dispositivo
UPDATE UNIDADES_DISPOSITIVO_ALMACEN
SET NUM_UNIDADES = NUM_UNIDADES - 1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP;

-- Se asigna la fecha de alta del dispositivo a la fecha del ltimo inventario
UPDATE DISPOSITIVOS
SET FECHA_ULTIMO_INVENTARIO = FECHA_ALTA
WHERE ID_DISPOSITIVO = :OLD.ID_DISPOSITIVO;

END;

END IF;

-- Si el movimiento corresponde a una baja

IF (DESCRIP_MOVI = 'BAJA') THEN

BEGIN

-- Se actualiza la tabla dispositivos, poniendo a null la fecha de baja para el 
--dispositivo que se ha dado de baja

UPDATE DISPOSITIVOS
SET FECHA_BAJA = NULL
WHERE ID_DISPOSITIVO = :OLD.ID_DISPOSITIVO;


-- Si el dispositivo es un portatil se actualiza la tabla Portatiles_Sin_Averia

IF (DESCRIP_DISP = 'PORTATIL') THEN

-- Se cuentan las averas que ha tenido el dispositivo
SELECT COUNT(*) INTO AVERIA
FROM MOVIMIENTOS_AUX M, DISPOSITIVOS D
WHERE M.ID_DISPOSITIVO = D.ID_DISPOSITIVO
AND M.ID_DISPOSITIVO = :OLD.ID_DISPOSITIVO 
AND ID_TIPO_MOVIMIENTO IN
(SELECT ID_TIPO_MOVIMIENTO FROM TIPO_MOVIMIENTO
WHERE DESCRIPCION='REPARACION DE AVERIAS');

-- Si ha tenido alguna avera, se incrementa el nmero de porttiles y los porttiles
-- sin avera de la tabla Portatiles_Sin_Averia.

IF (AVERIA > 0) THEN

UPDATE PORTATILES_SIN_AVERIA
SET NUM_PORTATILES = NUM_PORTATILES + 1,
PORTATILES_SIN_AVERIA = PORTATILES_SIN_AVERIA + 1,
POR_SIN_AVERIA = ((PORTATILES_SIN_AVERIA + 1)/ (NUM_PORTATILES + 1))*100;

ELSE

-- Se incremena el nmero de porttiles de la tabla Portatiles_Sn_Averia
UPDATE PORTATILES_SIN_AVERIA
SET NUM_PORTATILES = NUM_PORTATILES + 1 ,
POR_SIN_AVERIA = ( PORTATILES_SIN_AVERIA / (NUM_PORTATILES + 1)) * 100;

END IF;
END IF;

END;

END IF;


-- Si el movimiento es una actualizacion

IF (DESCRIP_MOVI = 'ACTUALIZACION') THEN

BEGIN

-- Se incrementa en 1 el nmero de dispositivos con fecha de inventario mayor de un ao

UPDATE DISPOSITIVOS_FECHA_INVENTARIO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS + 1;

-- Se actualiza la fecha_ultimo_inventario de la tabla dispositivos con la fecha del alta del dispositivo

UPDATE DISPOSITIVOS
SET FECHA_ULTIMO_INVENTARIO = FECHA_ALTA
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;

END;
END IF;

-- Si el movimiento correponde a una avera

IF (DESCRIP_MOVI = 'REPARACION DE AVERIAS') THEN

BEGIN

-- Se decrementa el nmero de averas de ese ao en la tabla Estadisticas_Ao

UPDATE ESTADISTICAS_AO
SET NUM_AVERIAS = NUM_AVERIAS - 1
WHERE AO = TO_CHAR(:NEW.FECHA_MOVIMIENTO, 'YYYY');

--  Se decrementa el nmero de averias del usuario que tiene asignado el dispositivo

UPDATE USUARIOS_AVERIAS
SET NUM_AVERIAS = NUM_AVERIAS -1
WHERE ID_USUARIO = :NEW.ID_USUARIO_DESTINO;

-- Si el dispositivo es un portatil, se incrementa el campo portatiles_sin_averia de 
--la misma tabla

IF (DESCRIP_DISP = 'PORTATIL') THEN

-- Se cuentan las averas que ha tenido el dispositivo
SELECT COUNT(*) INTO AVERIA
FROM MOVIMIENTOS_AUX M, DISPOSITIVOS D
WHERE M.ID_DISPOSITIVO = D.ID_DISPOSITIVO
AND M.ID_DISPOSITIVO = :OLD.ID_DISPOSITIVO 
AND ID_TIPO_MOVIMIENTO IN
(SELECT ID_TIPO_MOVIMIENTO FROM TIPO_MOVIMIENTO
WHERE DESCRIPCION='REPARACION DE AVERIAS');

-- Si no ha tenido ninguna avera
IF (AVERIA = 0) THEN

UPDATE PORTATILES_SIN_AVERIA
SET PORTATILES_SIN_AVERIA = PORTATILES_SIN_AVERIA + 1,
POR_SIN_AVERIA = ((PORTATILES_SIN_AVERIA + 1) / NUM_PORTATILES) * 100;

END IF;
END IF;

-- Calcula el ranking de los 10 usuarios que tienen ms averas y los introduce en la tabla Usuarios_Top10_Averias

INSERT INTO USUARIOS_TOP10_AVERIAS 
SELECT ID_USUARIO, NOMBRE_USUARIO, NUM_AVERIAS,
DENSE_RANK()
OVER (ORDER BY NUM_AVERIAS DESC) DRNK
FROM USUARIOS_AVERIAS
ORDER BY NUM_AVERIAS DESC;

END;

END IF;


-- Si el movimiento corresponde a una asignacin

IF (DESCRIP_MOVI = 'ASIGNACION') THEN

BEGIN

-- Se actualiza el campo id_usuario y la fecha de instalacion, ponindolos a null en la tabla dispositivos 

UPDATE DISPOSITIVOS
SET ID_USUARIO = NULL,
FECHA_INSTALACION = NULL
WHERE ID_DISPOSITIVO = :OLD.ID_DISPOSITIVO;

-- Se mira si el usuario tena asignado algn otro dispositivo. Si no es as
-- Se incrementa el nmero de usuarios sin dispositivos de la tabla Usuarios_Dispositivos


SELECT COUNT(*)
INTO NUM_DISP_USUARIO
FROM DISPOSITIVOS
WHERE ID_USUARIO = :OLD.ID_USUARIO_DESTINO;

IF (NUM_DISP_USUARIO = 0) THEN

UPDATE USUARIOS_DISPOSITIVOS
SET USUARIOS_SIN_DISPOSITIVOS = USUARIOS_SIN_DISPOSITIVOS + 1,
POR_SIN_DISPOSITIVOS = ((USUARIOS_SIN_DISPOSITIVOS + 1) / TOTAL_USUARIOS ) * 100;
END IF;


-- Se incrementa el nmero de dispositivos en el almacn.

UPDATE UNIDADES_DISPOSITIVO_ALMACEN
SET NUM_UNIDADES = NUM_UNIDADES + 1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP;

-- Se decrementa en 1 el nmero de dispositivos para ese tipo y ese departamento

UPDATE DISP_ACTIVOS_DEPARTAMENTO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS -1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP
AND ID_DEPARTAMENTO = DEPART;

-- Se mira si el departamento es el que tiene ms tipos de dispositivos
SELECT 'S'
INTO EXISTE
FROM DEPTO_MAS_TIPOSDISP
WHERE ID_DEPARTAMENTO = DEPART
AND ROWNUM = 1;

-- Si es as, se borra la tabla y se actualiza con los datos del departamento
-- que tiene ahora mas tipos de dispositivos

DELETE DEPTO_MAS_TIPOSDISP;

INSERT INTO DEPTO_MAS_TIPOSDISP
SELECT ID_DEPARTAMENTO, NOMBRE_DEPARTAMENTO, NUM_TIPOS_DISPOSITIVOS
FROM INVENTARIO_DEPARTAMENTO
WHERE NUM_TIPOS_DISPOSITIVOS IN
(SELECT MAX(NUM_TIPOS_DISPOSITIVOS) FROM INVENTARIO_DEPARTAMENTO);

-- Se cuentan los tipos diferentes de dispositivos que hay en el departamento y se 
-- actualiza la tabla Inventario_Departamento

SELECT  COUNT(DISTINCT (TIPO))
INTO NUM_DISP_DEPART
FROM DISPOSITIVOS
WHERE TIPO IN
(SELECT D.TIPO 
FROM DISPOSITIVOS D, USUARIOS U 
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART
GROUP BY  D.TIPO);

SELECT NVL(SUM(D.PRECIO_COMPRA),0)
INTO VALOR_INV
FROM DISPOSITIVOS D, USUARIOS U
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART
AND D.FECHA_BAJA IS NULL;


UPDATE INVENTARIO_DEPARTAMENTO
SET NUM_TIPOS_DISPOSITIVOS = NUM_DISP_DEPART,
VALOR_INVENTARIO = VALOR_INV
WHERE ID_DEPARTAMENTO = DEPART;


-- Se cuentan los usuarios que tienen asignados mas de 5 dispositivos y se actualiza 
-- la tabla Usuarios_Dispositivos

SELECT COUNT(DISTINCT (ID_USUARIO))
INTO NUM_USUARIOS
FROM DISPOSITIVOS 
WHERE ID_USUARIO IN
(SELECT ID_USUARIO FROM DISPOSITIVOS
GROUP BY ID_USUARIO
HAVING COUNT(ID_USUARIO) > 5);


UPDATE USUARIOS_DISPOSITIVOS
SET USUARIOS_MAS5_DISPOSITIVOS = NUM_USUARIOS;

END;
END IF;

END;


/

/*****************************************************************************
NOMBRE: TRG_MOVIMIENTO_UPDATE
********************************************************************************/


CREATE OR REPLACE
TRIGGER TRG_MOVIMIENTO_UPDATE AFTER UPDATE ON MOVIMIENTOS 
FOR EACH ROW 
DECLARE 

-- Se declaran las variables que van a ser necesarias para el funcionamiento del disparador
EXISTE VARCHAR2(1);
-- Descripcin del tipo de movimiento
DESCRIP_MOVI TIPO_MOVIMIENTO.DESCRIPCION%TYPE;
-- Descripcin del tipo de dispositivo
DESCRIP_DISP TIPO_DISPOSITIVO.DESCRIPCION%TYPE;
-- Identificador del tipo de dispositivo
TIPODISP TIPO_DISPOSITIVO.ID_TIPO_DISPOSITIVO%TYPE;
-- Departamento del usuario actual
DEPART DEPARTAMENTOS.ID_DEPARTAMENTO%TYPE;
-- Departamento del usuario anterior
DEPART_OLD DEPARTAMENTOS.ID_DEPARTAMENTO%TYPE;
-- Departamento con mas reasignaciones
DEPART_REASIG DEPTO_MAS_REASIGNACIONES.ID_DEPARTAMENTO%TYPE;
-- Nombre del departamento del usuario destino
NOMBRE_DEPART DEPARTAMENTOS.NOMBRE_DEPARTAMENTO%TYPE;
-- Nmero de tipos de dispositivos del departamento del usuario destino
NUM_DISP_DEPART NUMBER;
-- Nmero de tipos de dispositivos del departamento del usuario anterior
NUM_DISP_DEPART_OLD NUMBER;
-- Nmero de usuarios con mas de 5 dispositivos asignados
NUM_USUARIOS NUMBER;
-- Valor del inventario del departamento del usuario actual
VALOR_INV NUMBER;
-- Valor del inventario del departamento del usuario anterior
VALOR_INV_OLD NUMBER;
-- Nmero total de reasignaciones

BEGIN

/*****************************************************************************
PROPSITO: Cuando se modifica el campo usuario_destino de un movimiento de un 
dispositivo, dependiendo del tipo de movimiento de que se trate, se actualizan 
diferentes tablas del repositorio estadstico.
********************************************************************************/

-- Se obtiene la descripcin del tipo de movimiento
SELECT DESCRIPCION
INTO DESCRIP_MOVI
FROM TIPO_MOVIMIENTO
WHERE ID_TIPO_MOVIMIENTO = :NEW.ID_TIPO_MOVIMIENTO;

-- Se obtiene el tipo de dispositivo
SELECT TIPO
INTO TIPODISP
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;

-- Se obtiene la descripcin del tipo de dispositivo
SELECT DESCRIPCION
INTO DESCRIP_DISP
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = TIPODISP;

-- Si el usuario destino no es nulo
IF (:NEW.ID_USUARIO_DESTINO IS NOT NULL)

THEN

-- Se obtiene el departamento del usuario destino
SELECT ID_DEPARTAMENTO
INTO DEPART
FROM USUARIOS 
WHERE ID_USUARIO = :NEW.ID_USUARIO_DESTINO;


END IF;

-- Si el movimiento corresponde a una avera y se modifica el usuario_destino

IF (DESCRIP_MOVI = 'REPARACION DE AVERIAS') THEN

BEGIN


IF (:NEW.ID_USUARIO_DESTINO IS NOT NULL) THEN

-- Se actualiza la tabla Usuarios_Averias para el usuario nuevo y el anterior
UPDATE USUARIOS_AVERIAS
SET NUM_AVERIAS = NUM_AVERIAS + 1
WHERE ID_USUARIO = :NEW.ID_USUARIO_DESTINO;

UPDATE USUARIOS_AVERIAS
SET NUM_AVERIAS = NUM_AVERIAS - 1
WHERE ID_USUARIO = :OLD.ID_USUARIO_DESTINO;


END IF;

END;

END IF;

-- Si el movimiento corresponde a una asignacin

IF (DESCRIP_MOVI = 'ASIGNACION') THEN

BEGIN

-- Se actualiza el campo id_usuario de la tabla dispositivos 

UPDATE DISPOSITIVOS
SET ID_USUARIO = :NEW.ID_USUARIO_DESTINO
WHERE ID_DISPOSITIVO = :NEW.ID_DISPOSITIVO;


-- Se calcula el departamento al que pertenece el usuario que tena asignado anteriormente el dispositivo

SELECT ID_DEPARTAMENTO
INTO DEPART_OLD
FROM USUARIOS
WHERE ID_USUARIO = :OLD.ID_USUARIO_DESTINO;

-- Si el departamento es distinto

IF (DEPART <> DEPART_OLD) THEN

-- Se incrementa en 1 el nmero de dispositivos para ese tipo y el departammento destino

UPDATE DISP_ACTIVOS_DEPARTAMENTO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS + 1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP
AND ID_DEPARTAMENTO = DEPART;

-- Se cuentan los tipos diferentes de dispositivos que hay en el departamento destino  

SELECT  COUNT(DISTINCT (TIPO))
INTO NUM_DISP_DEPART
FROM DISPOSITIVOS
WHERE TIPO IN
(SELECT D.TIPO 
FROM DISPOSITIVOS D, USUARIOS U 
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART
GROUP BY  D.TIPO);


-- Se calcula el valor del inventario del departamento destino


SELECT NVL(SUM(D.PRECIO_COMPRA),0)
INTO VALOR_INV
FROM DISPOSITIVOS D, USUARIOS U
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART
AND D.FECHA_BAJA IS NULL;

-- Se actualiza la tabla Inventario_Departamento

UPDATE INVENTARIO_DEPARTAMENTO
SET NUM_TIPOS_DISPOSITIVOS = NUM_DISP_DEPART,
VALOR_INVENTARIO = VALOR_INV
WHERE ID_DEPARTAMENTO = DEPART;

-- Se actualiza la tabla Disp_Activos_Departamento para el departamento del usuario anterior

UPDATE DISP_ACTIVOS_DEPARTAMENTO
SET NUM_DISPOSITIVOS = NUM_DISPOSITIVOS -1
WHERE ID_TIPO_DISPOSITIVO = TIPODISP
AND ID_DEPARTAMENTO = DEPART_OLD;

-- Se calcula el nmero de tipos de dispositivo para el departamento del usuario anterior

SELECT  COUNT(DISTINCT (TIPO))
INTO NUM_DISP_DEPART_OLD
FROM DISPOSITIVOS
WHERE TIPO IN
(SELECT D.TIPO 
FROM DISPOSITIVOS D, USUARIOS U 
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART_OLD
GROUP BY  D.TIPO);

-- Se calcula el valor del inventario del departamento del usuario anterior

SELECT NVL(SUM(D.PRECIO_COMPRA),0)
INTO VALOR_INV_OLD
FROM DISPOSITIVOS D, USUARIOS U
WHERE D.ID_USUARIO = U.ID_USUARIO
AND U.ID_DEPARTAMENTO = DEPART_OLD
AND D.FECHA_BAJA IS NULL;

-- Se modifica la tabla Inventario_Departamento

UPDATE INVENTARIO_DEPARTAMENTO
SET NUM_TIPOS_DISPOSITIVOS = NUM_DISP_DEPART_OLD,
VALOR_INVENTARIO = VALOR_INV_OLD
WHERE ID_DEPARTAMENTO = DEPART_OLD;

--Se actualiza la tabla Depto_Mas_TiposDisp con el departamento que tiene mas tipos de dispositivos

DELETE DEPTO_MAS_TIPOSDISP;

INSERT INTO DEPTO_MAS_TIPOSDISP
SELECT ID_DEPARTAMENTO, NOMBRE_DEPARTAMENTO, NUM_TIPOS_DISPOSITIVOS
FROM INVENTARIO_DEPARTAMENTO
WHERE NUM_TIPOS_DISPOSITIVOS IN
(SELECT MAX(NUM_TIPOS_DISPOSITIVOS) FROM INVENTARIO_DEPARTAMENTO);


END IF;

-- Se cuentan los usuarios que tienen asignados mas de 5 dispositivos y se actualiza 
-- la tabla Usuarios_Dispositivos

SELECT COUNT(U.ID_USUARIO)
INTO NUM_USUARIOS
FROM DISPOSITIVOS D, USUARIOS U
WHERE D.ID_USUARIO = U.ID_USUARIO
GROUP BY D.ID_DISPOSITIVO
HAVING COUNT(D.ID_DISPOSITIVO) > 5;

END;

END IF;

END;

/

/*****************************************************************************
NOMBRE: TRG_INVENTARIO_INSERT
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_INVENTARIO_INSERT AFTER INSERT ON INVENTARIOS 
FOR EACH ROW 
DECLARE 

EXISTE VARCHAR2(1);
-- Nmero de dispositivos con fecha de inventario mayor de un ao
DISP NUMBER;

BEGIN

/*****************************************************************************
PROPSITO: Cuando se da de alta un nuevo inventario, se calculan los dispositivos 
que hay en la empresa con una fecha de inventario mayor de un ao y se actualiza
el campo nmero de dispositivos de la tabla Dispositivos_Fecha_Inventario.
********************************************************************************/

-- Se cuentan los dispositivos cuyta fecha de inventario sea mayor de un ao
SELECT COUNT(*) 
INTO DISP
FROM DISPOSITIVOS
WHERE FECHA_ULTIMO_INVENTARIO < ADD_MONTHS (SYSDATE,'-12')
AND FECHA_BAJA IS NULL;

-- Se mira si existe algn registro en la tabla Dispositivos_Fecha_Inventario
SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS_FECHA_INVENTARIO
WHERE ROWNUM = 1;

-- Si hay registros, modifica el nmero de dispositivos de la tabla
UPDATE DISPOSITIVOS_FECHA_INVENTARIO
SET NUM_DISPOSITIVOS = DISP;

-- Si no hay ningn registro en la tabla, inserta el nmero de dispositivos
EXCEPTION WHEN NO_DATA_FOUND THEN

INSERT INTO DISPOSITIVOS_FECHA_INVENTARIO VALUES (DISP); 

END;


/

/*****************************************************************************
NOMBRE: TRG_INVENTARIO_DELETE
********************************************************************************/

CREATE OR REPLACE
TRIGGER TRG_INVENTARIO_DELETE AFTER DELETE ON INVENTARIOS 
FOR EACH ROW 
DECLARE

-- Nmero de dispositivos con fecha de inventario mayor de un ao
DISP  NUMBER;

BEGIN

/*****************************************************************************
PROPSITO: Cuando se da de baja un inventario, se calculan los dispositivos que
hay en la empresa con una fecha de inventario mayor de un ao y se actualiza el
campo nmero de dispositivos de la tabla Dispositivos_Fecha_Inventario.
********************************************************************************/


-- Se cuentan los dispositivos que hay con fecha de ltimo inventario mayor de un ao
-- y se actualiza la tabla Dispositivos_Fecha_Inventario
SELECT COUNT(*) 
INTO DISP
FROM DISPOSITIVOS
WHERE FECHA_ULTIMO_INVENTARIO < ADD_MONTHS (SYSDATE,'-12')
AND FECHA_BAJA IS NULL;

UPDATE DISPOSITIVOS_FECHA_INVENTARIO
SET NUM_DISPOSITIVOS = DISP;


END;

/

spool off;