spool procedimientos.log

-- ALTA TIPO_MOVIMIENTO

CREATE OR REPLACE
PROCEDURE PR_ALTA_TIPO_MOVIMIENTO 
/***************************************************************
NOMBRE:   PR_ALTA_TIPO_MOVIMIENTO
PROPSITO:  Gestiona las altas de los tipos de movimientos
PARMETROS:
Entrada:  p_descripcin  descripcin del tipo de movimiento
Salida:  RSP
Pre-condicin: No existe el tipo de movimiento a dar de alta
Post-condicin: Se da de alta el tipo de movimiento
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el tipo de movimiento ya existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_descripcion IN TIPO_MOVIMIENTO.DESCRIPCION%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_TIPO_MOVIMIENTO_EXISTE	EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN
param_entrada := 'P_DESCRIPCION: '||p_descripcion;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_descripcion) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_MOVIMIENTO
WHERE DESCRIPCION = p_descripcion
AND ROWNUM = 1;

RAISE E_TIPO_MOVIMIENTO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;

-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO TIPO_MOVIMIENTO (DESCRIPCION, FECHA_ALTA, FECHA_BAJA)
VALUES (p_descripcion, SYSDATE, NULL);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_TIPO_MOVIMIENTO_EXISTE THEN
		RSP := 'ERROR: El tipo de movimiento ya ha sido dado de alta';
                
INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);
	COMMIT;
        
	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);
		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;        
END PR_ALTA_TIPO_MOVIMIENTO;


/

-- BAJA TIPO_MOVIMIENTO

CREATE OR REPLACE
PROCEDURE PR_BAJA_TIPO_MOVIMIENTO
/**********************************************************************
NOMBRE:   PR_BAJA_TIPO_MOVIMIENTO
PROPSITO:  Gestiona las bajas de los tipos de movimientos
PARMETROS:
Entrada:  p_id_tipo_movimiento identificador del tipo de movimiento
Salida:  RSP
Pre-condicin: Existe el tipo de movimiento a dar de baja
Post-condicin: Se da de baja el tipo de movimiento
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el tipo de movimiento no existe en la base de datos
-	ERROR: el tipo de movimiento ya haba sido dado de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_tipo_movimiento IN TIPO_MOVIMIENTO.ID_TIPO_MOVIMIENTO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_TIPOMOVIMIENTO_NOEXISTE	EXCEPTION;
E_TIPOMOVIMIENTO_YABAJA		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_TIPO_MOVIMIENTO: '||p_id_tipo_movimiento;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_tipo_movimiento) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_MOVIMIENTO
WHERE ID_TIPO_MOVIMIENTO = p_id_tipo_movimiento
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_TIPOMOVIMIENTO_NOEXISTE;
END;

-- Verifica que el tipo de movimiento no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_MOVIMIENTO
WHERE ID_TIPO_MOVIMIENTO = p_id_tipo_movimiento
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_TIPOMOVIMIENTO_YABAJA;
END;

-- Si ha pasado todas las validaciones, realiza el borrado

UPDATE TIPO_MOVIMIENTO
SET FECHA_BAJA=SYSDATE
WHERE ID_TIPO_MOVIMIENTO = p_id_tipo_movimiento;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_TIPOMOVIMIENTO_NOEXISTE THEN
		RSP := 'ERROR: El tipo de movimiento no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_TIPOMOVIMIENTO_YABAJA THEN
		RSP := 'ERROR: El tipo de movimiento ya haba sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);
	
	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_TIPO_MOVIMIENTO;


/

-- MODIFICACIN TIPO_MOVIMIENTO

CREATE OR REPLACE
PROCEDURE PR_MODIF_TIPO_MOVIMIENTO 
/***********************************************************************
NOMBRE:   PR_MODIF_TIPO_MOVIMIENTO
PROPSITO:  Gestiona las modificaciones de los tipos de movimientos
PARMETROS:
Entrada:  p_id_tipo_movimiento  identificador del tipo de movimiento
p_descripcin  descripcin del tipo de movimiento
Salida:  RSP
Pre-condicin: Existe el tipo de movimiento a modificar
Post-condicin: Se modifica el tipo de movimiento
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el identificador del tipo de movimiento a modificar no puede ser nulo
-	ERROR:  el identificador del tipo de movimiento a modificar no existe
-	ERROR: el tipo de movimiento que se va a modificar est de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_tipo_movimiento IN TIPO_MOVIMIENTO.ID_TIPO_MOVIMIENTO%TYPE, 
p_descripcion IN TIPO_MOVIMIENTO.DESCRIPCION%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_TIPO_MOVIMIENTO_NOEXISTE	EXCEPTION;
E_TIPO_MOVIMIENTO_BAJA	EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_TIPO_MOVIMIENTO: '||p_id_tipo_movimiento||', P_DESCRIPCION: '||p_descripcion;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_tipo_movimiento) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_MOVIMIENTO
WHERE ID_TIPO_MOVIMIENTO = p_id_tipo_movimiento
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_TIPO_MOVIMIENTO_NOEXISTE;
END;

-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE TIPO_MOVIMIENTO
SET DESCRIPCION = nvl(p_descripcion, DESCRIPCION)
WHERE ID_TIPO_MOVIMIENTO = p_id_tipo_movimiento;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_TIPO_MOVIMIENTO_NOEXISTE THEN
		RSP := 'ERROR: El tipo de movimiento a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_TIPO_MOVIMIENTO_BAJA THEN
		RSP := 'ERROR: El registro que quiere modificar ha sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_TIPO_MOVIMIENTO;

/

-- ALTA TIPO_DISPOSITIVO

CREATE OR REPLACE
PROCEDURE PR_ALTA_TIPO_DISPOSITIVO 
/***************************************************************
NOMBRE:   PR_ALTA_TIPO_DISPOSITIVO
PROPSITO:  Gestiona las altas de los tipos de dispositivos
PARMETROS:
Entrada:  p_descripcin  descripcin del tipo de dispositivo
Salida:  RSP
Pre-condicin: No existe el tipo de dispositivo a dar de alta
Post-condicin: Se da de alta el tipo de dispositivo
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el tipo de dispositivo ya existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_descripcion IN TIPO_DISPOSITIVO.DESCRIPCION%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_TIPO_DISPOSITIVO_EXISTE	EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_DESCRIPCION: '||p_descripcion;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_descripcion) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_DISPOSITIVO
WHERE DESCRIPCION = p_descripcion
AND ROWNUM = 1;

RAISE E_TIPO_DISPOSITIVO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;

-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO TIPO_DISPOSITIVO (DESCRIPCION, FECHA_ALTA, FECHA_BAJA)
VALUES (p_descripcion, SYSDATE, NULL);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_TIPO_DISPOSITIVO_EXISTE THEN
		RSP := 'ERROR: El tipo de dispositivo ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_TIPO_DISPOSITIVO;
 

 
/

-- BAJA TIPO_DISPOSITIVO

CREATE OR REPLACE
PROCEDURE PR_BAJA_TIPO_DISPOSITIVO
/**********************************************************************
NOMBRE:   PR_BAJA_TIPO_DISPOSITIVO
PROPSITO:  Gestiona las bajas de los tipos de dispositivos
PARMETROS:
Entrada:  p_id_tipo_dispositivo identificador del tipo de dispositivo
Salida:  RSP
Pre-condicin: Existe el tipo de dispositivo a dar de baja
Post-condicin: Se da de baja el tipo de dispositivo
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el tipo de dispositivo no existe en la base de datos
-	ERROR: el tipo de dispositivo ya haba sido dado de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_tipo_dispositivo IN TIPO_DISPOSITIVO.ID_TIPO_DISPOSITIVO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_TIPODISPOSITIVO_NOEXISTE	EXCEPTION;
E_TIPODISPOSITIVO_YABAJA	EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_TIPO_DISPOSITIVO: '||p_id_tipo_dispositivo;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_tipo_dispositivo) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = p_id_tipo_dispositivo
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_TIPODISPOSITIVO_NOEXISTE;
END;

-- Verifica que el tipo de dispositivo no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = p_id_tipo_dispositivo
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_TIPODISPOSITIVO_YABAJA;
END;

-- Si ha pasado todas las validaciones, realiza  el borrado

UPDATE TIPO_DISPOSITIVO
SET FECHA_BAJA = SYSDATE
WHERE ID_TIPO_DISPOSITIVO = p_id_tipo_dispositivo;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_TIPODISPOSITIVO_NOEXISTE THEN
		RSP := 'ERROR: El tipo de dispositivo no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_TIPODISPOSITIVO_YABAJA THEN
		RSP := 'ERROR: El tipo de dispositivo ya haba sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);
	
	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_TIPO_DISPOSITIVO;

/

-- MODIFICACIN TIPO_DISPOSITIVO

CREATE OR REPLACE
PROCEDURE PR_MODIF_TIPO_DISPOSITIVO 
/**************************************************************************
NOMBRE:   PR_MODIF_TIPO_DISPOSITIVO
PROPSITO:  Gestiona las modificaciones de los tipos de dispositivos
PARMETROS:
Entrada:  p_id_tipo_dispositivo  identificador del tipo de dispositivo
          p_descripcin  descripcin del tipo de dispositivo
Salida:  RSP
Pre-condicin: Existe el tipo de dispositivo a modificar
Post-condicin: Se modifica el tipo de dispositivo
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el identificador del tipo de dispositivo a modificar no puede ser nulo
-	ERROR:  el identificador del tipo de dispositivo a modificar no existe
-	ERROR: el tipo de dispositivo que se va a modificar est de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_tipo_dispositivo IN TIPO_DISPOSITIVO.ID_TIPO_DISPOSITIVO%TYPE, 
p_descripcion IN TIPO_DISPOSITIVO.DESCRIPCION%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_TIPO_DISPOSITIVO_NOEXISTE	EXCEPTION;
E_TIPO_DISPOSITIVO_BAJA		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_TIPO_DISPOSITIVO: '||p_id_tipo_dispositivo||', 
P_DESCRIPCION: '||p_descripcion;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_tipo_dispositivo) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = p_id_tipo_dispositivo
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_TIPO_DISPOSITIVO_NOEXISTE;
END;

-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE TIPO_DISPOSITIVO
SET DESCRIPCION = nvl(p_descripcion, DESCRIPCION)
WHERE ID_TIPO_DISPOSITIVO = p_id_tipo_dispositivo;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_TIPO_DISPOSITIVO_NOEXISTE THEN
		RSP := 'ERROR: El tipo de dispositivo a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_TIPO_DISPOSITIVO_BAJA THEN
		RSP := 'ERROR: El registro que quiere modificar ha sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_TIPO_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_TIPO_DISPOSITIVO;

/

-- ALTA PROVEEDORES

CREATE OR REPLACE
PROCEDURE PR_ALTA_PROVEEDORES 
/***************************************************************
NOMBRE:   PR_ALTA_PROVEEDORES
PROPSITO:  Gestiona las altas de los proveedores
PARMETROS:
Entrada:  p_nombre: nombre del proveedor
p_periodo_entrega : periodo medio de entrega del proveedor
p_direccin:  direccin del proveedor
p_localidad:  localidad del proveedor
p_provincia:  provincia del proveedor
p_pas:  pas del proveedor
p_telfono: telfono del proveedor
p_email:  correo electrnico del proveedor
Salida:  RSP
Pre-condicin: No existe el tipo proveedor a dar de alta
Post-condicin: Se da de alta el proveedor
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el proveedor ya existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_nombre IN PROVEEDORES.NOMBRE%TYPE,p_direccion IN PROVEEDORES.DIRECCION%TYPE, 
p_localidad IN PROVEEDORES.LOCALIDAD%TYPE, p_provincia IN PROVEEDORES.PROVINCIA%TYPE,
p_pais IN PROVEEDORES.PAIS%TYPE, p_telefono IN PROVEEDORES.TELEFONO%TYPE, 
p_email IN PROVEEDORES.EMAIL%TYPE, p_periodo_entrega IN PROVEEDORES.PERIODO_ENTREGA%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_PROVEEDOR_EXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_NOMBRE: '||p_nombre||', P_DIRECCION:  '||	p_direccion||', P_LOCALIDAD: 
'||p_localidad||', P_PROVINCIA: '||p_provincia||', PAIS: '||p_pais||', TELEFONO: '||p_telefono||
', EMAIL: '||p_email||', PERIODO_ENTREGA:  '||p_periodo_entrega;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_nombre) IS NULL OR
TRIM(p_periodo_entrega) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM PROVEEDORES
WHERE NOMBRE =p_nombre
AND ROWNUM = 1;

RAISE E_PROVEEDOR_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;

-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO PROVEEDORES_AUX (NOMBRE,DIRECCION, LOCALIDAD, PROVINCIA, PAIS, TELEFONO, EMAIL, PERIODO_ENTREGA, FECHA_ALTA, FECHA_BAJA)
VALUES (p_nombre, p_direccion, p_localidad, p_provincia, p_pais, p_telefono, p_email, p_periodo_entrega, SYSDATE, NULL);

INSERT INTO PROVEEDORES (NOMBRE,DIRECCION, LOCALIDAD, PROVINCIA, PAIS, TELEFONO, EMAIL, PERIODO_ENTREGA, FECHA_ALTA, FECHA_BAJA)
VALUES (p_nombre, p_direccion, p_localidad, p_provincia, p_pais, p_telefono, p_email, p_periodo_entrega, SYSDATE, NULL);


	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PROVEEDORES', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_PROVEEDOR_EXISTE THEN
		RSP := 'ERROR: El proveedor ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PROVEEDORES', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PROVEEDORES', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PROVEEDORES', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_PROVEEDORES;


/

-- BAJA PROVEEDORES

CREATE OR REPLACE
PROCEDURE PR_BAJA_PROVEEDOR
/**********************************************************************
NOMBRE:   PR_BAJA_PROVEEDOR
PROPSITO:  Gestiona las bajas de los proveedores
PARMETROS:
Entrada:  p_id_proveedor identificador del proveedor
Salida:  RSP
Pre-condicin: Existe el proveedor a dar de baja
Post-condicin: Se da de baja el proveedor
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el proveedor no existe en la base de datos
-	ERROR: el proveedor ya haba sido dado de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_proveedor IN PROVEEDORES.ID_PROVEEDOR%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_PROVEEDOR_NOEXISTE		EXCEPTION;
E_PROVEEDOR_YABAJA		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_PROVEEDOR: '||p_id_proveedor;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_proveedor) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM PROVEEDORES
WHERE ID_PROVEEDOR = p_id_proveedor
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_PROVEEDOR_NOEXISTE;
END;

-- Verifica que el proveedor no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM PROVEEDORES
WHERE ID_PROVEEDOR = p_id_proveedor
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_PROVEEDOR_YABAJA;
END;

-- Si ha pasado todas las validaciones, realiza el borrado

UPDATE PROVEEDORES_AUX
SET FECHA_BAJA = SYSDATE
WHERE ID_PROVEEDOR = p_id_proveedor;

UPDATE PROVEEDORES
SET FECHA_BAJA = SYSDATE
WHERE ID_PROVEEDOR = p_id_proveedor;

--UPDATE PROVEEDORES_AUX
--SET FECHA_BAJA = SYSDATE
--WHERE ID_PROVEEDOR = p_id_proveedor;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PROVEEDOR', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_PROVEEDOR_NOEXISTE THEN
		RSP := 'ERROR: El proveedor no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PROVEEDOR', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_PROVEEDOR_YABAJA THEN
		RSP := 'ERROR: El proveedor ya haba sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PROVEEDOR', param_entrada, 'RSP:  '||RSP);
	
	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PROVEEDOR', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PROVEEDOR', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_PROVEEDOR;

/

-- MODIFICACIN PROVEEDORES

CREATE OR REPLACE
PROCEDURE PR_MODIF_PROVEEDOR 
/***************************************************************
NOMBRE:   PR_MODIF_PROVEEDOR
PROPSITO:  Gestiona las modificaciones de los proveedores
PARMETROS:
Entrada:  p_id_proveedor:  identificador del proveedor
p_nombre:  nombre del proveedor
p_direccin:  direccin del proveedor
p_localidad:  localidad del proveedor
p_provincia:  provincia del proveedor
p_pas:  pas del proveedor
p_telfono:  telfono del proveedor
p_email:  correo electrnico del proveedor
p_periodo_entrega: periodo medio de entrega en das

Salida:  RSP
Pre-condicin: Existe el proveedor a modificar
Post-condicin: Se modifica el proveedor
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el identificador del proveedor a modificar no puede ser nulo
-	ERROR:  el identificador del proveedor a modificar no existe
-	ERROR: el proveedor que se va a modificar est de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_proveedor IN PROVEEDORES.ID_PROVEEDOR%TYPE, p_nombre IN PROVEEDORES.NOMBRE%TYPE,
p_direccion IN PROVEEDORES.DIRECCION%TYPE, p_localidad IN PROVEEDORES.LOCALIDAD%TYPE, 
p_provincia IN PROVEEDORES.PROVINCIA%TYPE, p_pais IN PROVEEDORES.PAIS%TYPE, 
p_telefono IN PROVEEDORES.TELEFONO%TYPE, p_email IN PROVEEDORES.EMAIL%TYPE, p_periodo_entrega IN PROVEEDORES.PERIODO_ENTREGA%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_PROVEEDOR_NOEXISTE		EXCEPTION;
E_PROVEEDOR_BAJA		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_PROVEEDOR: '||p_id_proveedor||', P_NOMBRE: '||p_nombre||', 
P_DIRECCION:  '||p_direccion||', P_LOCALIDAD: '||p_localidad||', 
P_PROVINCIA: '||p_provincia||', P_PAIS: '||p_pais||', P_TELEFONO: '||p_telefono||', P_EMAIL: '||p_email||', PERIODO_ENTREGA:  '||p_periodo_entrega;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_proveedor) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM PROVEEDORES
WHERE ID_PROVEEDOR = p_id_proveedor
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_PROVEEDOR_NOEXISTE;
END;

-- Verifica que el proveedor no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM PROVEEDORES
WHERE ID_PROVEEDOR = p_id_proveedor
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_PROVEEDOR_BAJA;
END;

-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE PROVEEDORES_AUX
SET NOMBRE = nvl(p_nombre, NOMBRE),
DIRECCION = nvl(p_direccion, DIRECCION),
LOCALIDAD = nvl(p_localidad, LOCALIDAD),
PROVINCIA = nvl(p_provincia, PROVINCIA),
PAIS = nvl(p_pais, PAIS),
TELEFONO = nvl (p_telefono, TELEFONO), 
EMAIL = nvl (p_email, EMAIL),
PERIODO_ENTREGA = nvl (p_periodo_entrega, PERIODO_ENTREGA)
WHERE ID_PROVEEDOR = p_id_proveedor;

UPDATE PROVEEDORES
SET NOMBRE = nvl(p_nombre, NOMBRE),
DIRECCION = nvl(p_direccion, DIRECCION),
LOCALIDAD = nvl(p_localidad, LOCALIDAD),
PROVINCIA = nvl(p_provincia, PROVINCIA),
PAIS = nvl(p_pais, PAIS),
TELEFONO = nvl (p_telefono, TELEFONO), 
EMAIL = nvl (p_email, EMAIL),
PERIODO_ENTREGA = nvl (p_periodo_entrega, PERIODO_ENTREGA)
WHERE ID_PROVEEDOR = p_id_proveedor;



	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PROVEEDOR', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_PROVEEDOR_NOEXISTE THEN
		RSP := 'ERROR: El proveedor a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PROVEEDOR', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PROVEEDOR', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_PROVEEDOR_BAJA THEN
		RSP := 'ERROR: El registro que quiere modificar ha sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PROVEEDOR', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PROVEEDOR', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_PROVEEDOR;


/

-- ALTA DEPARTAMENTOS

CREATE OR REPLACE
PROCEDURE PR_ALTA_DEPARTAMENTOS 
/***********************************************************************
NOMBRE:   PR_ALTA_DEPARTAMENTOS
PROPSITO:  Gestiona las altas de los departamentos de la empresa
PARMETROS:
Entrada:  p_nombre_departamento:  nombre del departamento
p_id_responsable:  identificador del responsable del departamento

Salida:  RSP
Pre-condicin: No existe el departamento a dar de alta
Post-condicin: Se da de alta el departamento
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  la departamento ya existe
-	ERROR: el identificador del usuario responsable no existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_nombre_departamento IN DEPARTAMENTOS.NOMBRE_DEPARTAMENTO%TYPE, 
p_id_responsable IN DEPARTAMENTOS.ID_RESPONSABLE%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_DEPARTAMENTO_EXISTE		EXCEPTION;
E_RESPONSABLE_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_NOMBRE_DEPARTAMENTO: '||p_nombre_departamento||
', P_ID_RESPONSABLE:  '||p_id_responsable;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_nombre_departamento) IS NULL) 

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM DEPARTAMENTOS
WHERE NOMBRE_DEPARTAMENTO =p_nombre_departamento
AND ROWNUM = 1;

RAISE E_DEPARTAMENTO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;

-- Verifica si el usuario responsable del departamento existe

IF (p_id_responsable is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_responsable
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	RAISE E_RESPONSABLE_NOEXISTE;
        
 END;    
 END IF;

-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO DEPARTAMENTOS (NOMBRE_DEPARTAMENTO, ID_RESPONSABLE, FECHA_ALTA, FECHA_BAJA)
VALUES (p_nombre_departamento, p_id_responsable, SYSDATE, NULL);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DEPARTAMENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_DEPARTAMENTO_EXISTE THEN
		RSP := 'ERROR: El departamento ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DEPARTAMENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DEPARTAMENTOS', param_entrada, 'RSP:  '||RSP);

	WHEN E_RESPONSABLE_NOEXISTE THEN
		RSP := 'ERROR: El usuario responsable del departamento no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DEPARTAMENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DEPARTAMENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_DEPARTAMENTOS;

/

-- BAJA DEPARTAMENTOS

CREATE OR REPLACE
PROCEDURE PR_BAJA_DEPARTAMENTO
/**********************************************************************
NOMBRE:   PR_BAJA_DEPARTAMENTO
PROPSITO:  Gestiona las bajas de los departamentos
PARMETROS:
Entrada:  p_id_departamento identificador del departamento
Salida:  RSP
Pre-condicin: Existe el departamento a dar de baja
Post-condicin: Se da de baja el departamento
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el departamento no existe en la base de datos
-	ERROR: el departamento ya haba sido dada de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_departamento IN DEPARTAMENTOS.ID_DEPARTAMENTO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_DEPARTAMENTO_NOEXISTE		EXCEPTION;
E_DEPARTAMENTO_YABAJA		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_DEPARTAMENTO: '||p_id_departamento;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_departamento) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM DEPARTAMENTOS
WHERE ID_DEPARTAMENTO = p_id_departamento
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_DEPARTAMENTO_NOEXISTE;
END;

-- Verifica que el departamento no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM DEPARTAMENTOS
WHERE ID_DEPARTAMENTO = p_id_departamento
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_DEPARTAMENTO_YABAJA;
END;

-- Si ha pasado todas las validaciones, realiza el borrado

UPDATE DEPARTAMENTOS
SET FECHA_BAJA = SYSDATE
WHERE ID_DEPARTAMENTO = p_id_departamento;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_DEPARTAMENTO_NOEXISTE THEN
		RSP := 'ERROR: El departamento no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_DEPARTAMENTO_YABAJA THEN
		RSP := 'ERROR: El departamento ya haba sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);
	
	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_DEPARTAMENTO;


/

-- MODIFICACIN DEPARTAMENTOS

CREATE OR REPLACE
PROCEDURE PR_MODIF_DEPARTAMENTO 
/***********************************************************************
NOMBRE:   PR_MODIF_DEPARTAMENTO
PROPSITO:  Gestiona las modificaciones de los departamentos
PARMETROS:
Entrada:  p_id_departamento:  identificador del departamento
p_nombre_departamento:  nombre del departamento
p_id_responsable:  identificador del responsable del departamento

Salida:  RSP
Pre-condicin: Existe el departamento a modificar
Post-condicin: Se modifica el departamento
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador del departamento a modificar no existe
-	ERROR:  el identificador del departamento a modificar no puede ser nulo
-	ERROR: el departamento que se va a modificar est de baja
-	ERROR: el identificador del usuario responsable no existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_departamento IN DEPARTAMENTOS.ID_DEPARTAMENTO%TYPE, p_nombre_departamento IN DEPARTAMENTOS.NOMBRE_DEPARTAMENTO%TYPE, p_id_responsable IN DEPARTAMENTOS.ID_RESPONSABLE%TYPE,  RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_DEPARTAMENTO_NOEXISTE		EXCEPTION;
E_DEPARTAMENTO_BAJA		EXCEPTION;
E_RESPONSABLE_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_DEPARTAMENTO: '||p_id_departamento||', P_NOMBRE_DEPARTAMENTO: '||p_nombre_departamento||', P_ID_RESPONSABLE:  '||	p_id_responsable;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_departamento) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM DEPARTAMENTOS
WHERE ID_DEPARTAMENTO = p_id_departamento
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_DEPARTAMENTO_NOEXISTE;
END;

-- Verifica que el departamento no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM DEPARTAMENTOS
WHERE ID_DEPARTAMENTO = p_id_departamento
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_DEPARTAMENTO_BAJA;
END;

-- Si se va a modificar el usuario responsable se verifica que exista en la base de datos
if (p_id_responsable is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_responsable
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_RESPONSABLE_NOEXISTE;
END;
END IF;


-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE DEPARTAMENTOS
SET NOMBRE_DEPARTAMENTO = nvl(p_nombre_departamento, NOMBRE_DEPARTAMENTO),
ID_RESPONSABLE = nvl(p_id_responsable, ID_RESPONSABLE)
WHERE ID_DEPARTAMENTO = p_id_departamento;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_DEPARTAMENTO_NOEXISTE THEN
		RSP := 'ERROR: El departamento a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_DEPARTAMENTO_BAJA THEN
		RSP := 'ERROR: El registro que quiere modificar ha sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_RESPONSABLE_NOEXISTE THEN
		RSP := 'ERROR: El usuario responsable del departamento no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DEPARTAMENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_DEPARTAMENTO;

/

-- ALTA SEDES

CREATE OR REPLACE
PROCEDURE PR_ALTA_SEDES 
/***********************************************************************
NOMBRE:   PR_ALTA_SEDES
PROPSITO:  Gestiona las altas de las sedes de la empresa
PARMETROS:
Entrada:  p_nombre_sede:  nombre de la sede
p_direccin_sede:  direccin de la sede
p_localidad_sede : localidad de la sede
p_provincia_sede : provincia de la sede
p_pas_sede:  pas de la sede

Salida:  RSP
Pre-condicin: No existe la sede a dar de alta
Post-condicin: Se da de alta la sede
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  la sede ya existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_nombre_sede IN SEDES.NOMBRE_SEDE%TYPE, p_direccion_sede IN SEDES.DIRECCION_SEDE%TYPE, p_localidad_sede IN SEDES.LOCALIDAD_SEDE%TYPE, p_provincia_sede IN SEDES.PROVINCIA_SEDE%TYPE, p_pais_sede IN SEDES.PAIS_SEDE%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_SEDE_EXISTE			EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_NOMBRE_SEDE: '||p_nombre_sede||', P_DIRECCION_SEDE:  '||	p_direccion_sede||', P_LOCALIDAD_SEDE: '||p_localidad_sede||', P_PROVINCIA_SEDE: '||p_provincia_sede||', PAIS_SEDE: '||p_pais_sede;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_nombre_sede) IS NULL) 

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM SEDES
WHERE NOMBRE_SEDE = p_nombre_sede
AND DIRECCION_SEDE = p_direccion_sede
AND LOCALIDAD_SEDE = p_localidad_sede
AND ROWNUM = 1;

RAISE E_SEDE_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;

-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO SEDES (NOMBRE_SEDE,DIRECCION_SEDE, LOCALIDAD_SEDE, PROVINCIA_SEDE, PAIS_SEDE, FECHA_ALTA, FECHA_BAJA)
VALUES (p_nombre_sede, p_direccion_sede, p_localidad_sede, p_provincia_sede, p_pais_sede, SYSDATE, NULL);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_SEDES', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_SEDE_EXISTE THEN
		RSP := 'ERROR: La sede ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_SEDES', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_SEDES', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_SEDES', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_SEDES;


/

-- BAJA SEDES

CREATE OR REPLACE
PROCEDURE PR_BAJA_SEDE
/**********************************************************************
NOMBRE:   PR_BAJA_SEDE
PROPSITO:  Gestiona las bajas de Las sedes
PARMETROS:
Entrada:  p_id_sede identificador de la sede
Salida:  RSP
Pre-condicin: Existe la sede a dar de baja
Post-condicin: Se da de baja la sede
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  la sede no existe en la base de datos
-	ERROR: la sede ya haba sido dada de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_sede IN SEDES.ID_SEDE%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_SEDE_NOEXISTE			EXCEPTION;
E_SEDE_YABAJA			EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_SEDE: '||p_id_sede;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_sede) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM SEDES
WHERE ID_SEDE = p_id_sede
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_SEDE_NOEXISTE;
END;

-- Verifica que la sede no est dada de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM SEDES
WHERE ID_SEDE = p_id_sede
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_SEDE_YABAJA;
END;

-- Si ha pasado todas las validaciones, realiza el borrado

UPDATE SEDES
SET FECHA_BAJA = SYSDATE
WHERE ID_SEDE = p_id_sede;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_SEDE', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_SEDE_NOEXISTE THEN
		RSP := 'ERROR: La sede no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_SEDE', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_SEDE_YABAJA THEN
		RSP := 'ERROR: La sede ya haba sido dada de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_SEDE', param_entrada, 'RSP:  '||RSP);
	
	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_SEDE', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_SEDE', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_SEDE;


/

-- MODIFICACIN SEDES

CREATE OR REPLACE
PROCEDURE PR_MODIF_SEDE 
/***************************************************************
NOMBRE:   PR_MODIF_SEDE
PROPSITO:  Gestiona las modificaciones de las sedes
PARMETROS:
Entrada:  p_id_sede:  identificador de la sede
p_nombre_sede:  nombre de la sede
p_direccin_sede:  direccin de la sede
p_localidad_sede:  localidad de la sede
p_provincia_sede:  provincia de la sede
p_pas_sede:  pas de la sede
Salida:  RSP
Pre-condicin: Existe la sede a modificar
Post-condicin: Se modifica la sede
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador de la sede a modificar no existe
-	ERROR:  el identificador de la sede a modificar no puede ser nulo
-	ERROR: la sede que se va a modificar est de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_sede IN SEDES.ID_SEDE%TYPE, p_nombre_sede IN SEDES.NOMBRE_SEDE%TYPE,
p_direccion_sede IN SEDES.DIRECCION_SEDE%TYPE, 
p_localidad_sede IN SEDES.LOCALIDAD_SEDE%TYPE, 
p_provincia_sede IN SEDES.PROVINCIA_SEDE%TYPE, 
p_pais_sede IN SEDES.PAIS_SEDE%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO		EXCEPTION;
E_SEDE_NOEXISTE		EXCEPTION;
E_SEDE_YABAJA		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_SEDE: '||p_id_sede||', P_NOMBRE_SEDE: '||p_nombre_sede||', 
P_DIRECCION_SEDE:  '||p_direccion_sede||', P_LOCALIDAD_SEDE: '||p_localidad_sede||', 
P_PROVINCIA_SEDE: '||p_provincia_sede||', PAIS_SEDE: '||p_pais_sede;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_sede) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM SEDES
WHERE ID_SEDE = p_id_sede
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_SEDE_NOEXISTE;
END;

-- Verifica que la sede no est dada de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM SEDES
WHERE ID_SEDE = p_id_sede
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_SEDE_YABAJA;
END;

-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE SEDES
SET NOMBRE_SEDE = nvl(p_nombre_sede, NOMBRE_SEDE),
DIRECCION_SEDE = nvl(p_direccion_sede, DIRECCION_SEDE),
LOCALIDAD_SEDE = nvl(p_localidad_sede, LOCALIDAD_SEDE),
PROVINCIA_SEDE = nvl(p_provincia_sede, PROVINCIA_SEDE),
PAIS_SEDE = nvl(p_pais_sede, PAIS_SEDE)
WHERE ID_SEDE = p_id_sede;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_SEDE', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_SEDE_NOEXISTE THEN
		RSP := 'ERROR: La sede a modificar no ha sido dada de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_SEDE', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_SEDE', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_SEDE_YABAJA THEN
		RSP := 'ERROR: El registro que quiere modificar ha sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_SEDE', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_SEDE', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_SEDE;


/

-- ALTA PEDIDOS

CREATE OR REPLACE
PROCEDURE PR_ALTA_PEDIDOS
/***********************************************************************
NOMBRE:   PR_ALTA_PEDIDOS
PROPSITO:  Gestiona las altas de los pedidos de la empresa
PARMETROS:
Entrada: p_id_proveedor:  identificador del proveedor del pedido
p_id_departamento:  identificador del departamento al que pertenece el pedido
p_fecha_pedido:  fecha del pedido
p_importe_pedido: importe del pedido
p_orden_inversion: orden de inversin a la que va ligada el pedido

Salida:  RSP
Pre-condicin: No existe el pedido a dar de alta
Post-condicin: Se da de alta el pedido
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el pedido ya existe
-	ERROR: el proveedor que sirve el pedido no existe
-	ERROR: el departamento que ha solicitado el pedido no existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_proveedor IN PEDIDOS.ID_PROVEEDOR%TYPE, p_id_departamento IN PEDIDOS.ID_DEPARTAMENTO%TYPE, p_fecha_pedido IN PEDIDOS.FECHA_PEDIDO%TYPE, p_importe_pedido IN PEDIDOS.IMPORTE_PEDIDO%TYPE, p_orden_inversion IN PEDIDOS.ORDEN_INVERSION%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_PEDIDO_EXISTE			EXCEPTION;
E_PROVEEDOR_NOEXISTE		EXCEPTION;
E_DEPARTAMENTO_NOEXISTE	        EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_PROVEEDOR: '||p_id_proveedor||', P_ID_DEPARTAMENTO: '||p_id_departamento||', P_FECHA_PEDIDO:  '||p_fecha_pedido||', P_IMPORTE_PEDIDO:  '||p_importe_pedido||', P_ORDEN_INVERSION:  '||p_orden_inversion;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_proveedor) IS NULL OR
TRIM(p_id_departamento) IS NULL OR
TRIM(p_fecha_pedido) IS NULL OR
TRIM(p_importe_pedido) IS NULL) 

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM PEDIDOS
WHERE ID_PROVEEDOR =p_id_proveedor AND
ID_DEPARTAMENTO = p_id_departamento AND
FECHA_PEDIDO = p_fecha_pedido AND
IMPORTE_PEDIDO = p_importe_pedido
AND ROWNUM = 1;

RAISE E_PEDIDO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;


-- Verifica que el proveedor del pedido existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM PROVEEDORES
WHERE ID_PROVEEDOR =p_id_proveedor
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_PROVEEDOR_NOEXISTE;
END;

-- Verifica que el departamento asignado al usuario existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM DEPARTAMENTOS
WHERE ID_DEPARTAMENTO =p_id_departamento
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_DEPARTAMENTO_NOEXISTE;
END;


-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO PEDIDOS (ID_PROVEEDOR,ID_DEPARTAMENTO, FECHA_PEDIDO, IMPORTE_PEDIDO, ORDEN_INVERSION)
VALUES (p_id_proveedor, p_id_departamento, p_fecha_pedido, p_importe_pedido, p_orden_inversion);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PEDIDOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_PEDIDO_EXISTE THEN
		RSP := 'ERROR: El pedido ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PEDIDOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PEDIDOS', param_entrada, 'RSP:  '||RSP);

	WHEN E_PROVEEDOR_NOEXISTE THEN
		RSP := 'ERROR: El proveedor que sirve el pedido no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PEDIDOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_DEPARTAMENTO_NOEXISTE THEN
		RSP := 'ERROR: El departamento que ha solicitado el pedido no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PEDIDOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PEDIDOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_PEDIDOS;

/

-- BAJA PEDIDOS

CREATE OR REPLACE
PROCEDURE PR_BAJA_PEDIDO
/**********************************************************************
NOMBRE:   PR_BAJA_PEDIDO
PROPSITO:  Gestiona las bajas de los pedidos de la empresa
PARMETROS:
Entrada:  p_id_pedido identificador del pedido
Salida:  RSP
Pre-condicin: Existe el pedido a dar de baja
Post-condicin: Se da de baja el pedido
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el pedido no existe en la base de datos
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_pedido IN PEDIDOS.ID_PEDIDO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO				EXCEPTION;
E_PEDIDO_NOEXISTE			EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_PEDIDO: '||p_id_pedido;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_pedido) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM PEDIDOS
WHERE ID_PEDIDO = p_id_pedido
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_PEDIDO_NOEXISTE;
END;


-- Si ha pasado todas las validaciones, realiza el borrado

DELETE FROM PEDIDOS
WHERE ID_PEDIDO = p_id_pedido;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_PEDIDO_NOEXISTE THEN
		RSP := 'ERROR: El pedido no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_PEDIDO;


/

-- MODIFICACIN PEDIDOS

CREATE OR REPLACE
PROCEDURE PR_MODIF_PEDIDO 
/*********************************************************************
NOMBRE:   PR_MODIF_PEDIDO
PROPSITO:  Gestiona las modificaciones de los pedidos de la empresa
PARMETROS:
Entrada:  p_id_pedido:  identificador del pedido
p_id_proveedor:  identificador del proveedor que servir el pedido
p_id_departamento:  identificador del departamento al que pertenece el pedido
p_fecha_pedido:  fecha del pedido
p_importe_pedido:  importe del pedido
p_orden_inversion:  orden de inversin a la que va ligada el pedido

Salida:  RSP
Pre-condicin: Existe el pedido a modificar
Post-condicin: Se modifica el pedido
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador del pedido a modificar no existe
-	ERROR:  el identificador del pedido a modificar no puede ser nulo
-	ERROR: el identificador del proveedor del pedido no existe
-	ERROR: el identificador del departamento al que pertenece el pedido no existe
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_pedido IN PEDIDOS.ID_PEDIDO%TYPE, p_id_proveedor IN PEDIDOS.ID_PROVEEDOR%TYPE, p_id_departamento IN PEDIDOS.ID_DEPARTAMENTO%TYPE, p_fecha_pedido IN PEDIDOS.FECHA_PEDIDO%TYPE, p_importe_pedido IN PEDIDOS.IMPORTE_PEDIDO%TYPE, p_orden_inversion IN PEDIDOS.ORDEN_INVERSION%TYPE, RSP OUT VARCHAR2)


IS
E_CAMPO_NULO			EXCEPTION;
E_PEDIDO_NOEXISTE		EXCEPTION;
E_PROVEEDOR_NOEXISTE		EXCEPTION;
E_DEPARTAMENTO_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN
param_entrada := 'P_ID_PEDIDO: '||p_id_pedido||', P_ID_PROVEEDOR: '||p_id_proveedor||', P_ID_DEPARTAMENTO: '||p_id_departamento||', P_FECHA_PEDIDO:  '||p_fecha_pedido||', P_IMPORTE_PEDIDO:  '||p_importe_pedido||', P_ORDEN_INVERSION:  '||p_orden_inversion;


-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_pedido) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM PEDIDOS
WHERE ID_PEDIDO = p_id_pedido
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_PEDIDO_NOEXISTE;
END;


-- Si se va a modificar el proveedor se verifica que exista en la base de datos

IF (p_id_proveedor IS NOT NULL) THEN
BEGIN
SELECT 'S'
INTO EXISTE
FROM PROVEEDORES
WHERE ID_PROVEEDOR = p_id_proveedor
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_PROVEEDOR_NOEXISTE;
END;
END IF;

-- Si se va a modificar el departamento al que pertenece el pedido, se verifica que exista en la base de datos

IF (p_id_departamento IS NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM DEPARTAMENTOS
WHERE ID_DEPARTAMENTO = p_id_departamento
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_DEPARTAMENTO_NOEXISTE;
END;
END IF;

-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE PEDIDOS
SET ID_PROVEEDOR = nvl(p_id_proveedor, ID_PROVEEDOR),
ID_DEPARTAMENTO = nvl(p_id_departamento, ID_DEPARTAMENTO),
FECHA_PEDIDO = nvl(p_fecha_pedido, FECHA_PEDIDO),
IMPORTE_PEDIDO = nvl(p_importe_pedido, IMPORTE_PEDIDO),
ORDEN_INVERSION = nvl(p_orden_inversion, ORDEN_INVERSION)
WHERE ID_PEDIDO = p_id_pedido;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_PEDIDO_NOEXISTE THEN
		RSP := 'ERROR: El pedido a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;


WHEN E_PROVEEDOR_NOEXISTE THEN
		RSP := 'ERROR: El proveedor del pedido no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_DEPARTAMENTO_NOEXISTE THEN
		RSP := 'ERROR: El departamento al que corresponde el pedido no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_PEDIDO;


/

-- ALTA LINEAS_PEDIDO

CREATE OR REPLACE
PROCEDURE PR_ALTA_LINEASPEDIDO
/***********************************************************************
NOMBRE:   PR_ALTA_LINEASPEDIDO
PROPSITO:  Gestiona las altas de las lneas de pedido
PARMETROS:
Entrada: p_id_linea:  identificador de la lnea del pedido
p_id_pedido:  identificador del  pedido
p_id_material:  identificador del tipo de dispositivo
p_unidades:  unidades de cada tipo de dispositivo
p_precio_unitario:  precio unitario de cada tipo de dispositivo

Salida:  RSP
Pre-condicin: No existe la lnea de  pedido a dar de alta. Tienen que existir 
el pedido y el tipo de dispositivo
Post-condicin: Se da de alta la lnea del pedido
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  la lnea de pedido ya existe
-	ERROR: el pedido al que corresponde la lnea no existe
-	ERROR: el tipo de dispositivo solicitado en el pedido no existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_linea IN LINEAS_PEDIDO.ID_LINEA%TYPE, p_id_pedido IN LINEAS_PEDIDO.ID_PEDIDO%TYPE, 
p_id_material IN LINEAS_PEDIDO.ID_MATERIAL%TYPE, p_unidades IN LINEAS_PEDIDO.UNIDADES%TYPE, 
p_precio_unitario IN LINEAS_PEDIDO.PRECIO_UNITARIO%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_LINEAPEDIDO_EXISTE		EXCEPTION;
E_PEDIDO_NOEXISTE		EXCEPTION;
E_TIPODISPOSITIVO_NOEXISTE	EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_LINEA: '||p_id_linea||', P_ID_PEDIDO: '||p_id_pedido||', 
P_ID_MATERIAL:  '||p_id_material||', P_UNIDADES:  '||p_unidades||', 
P_PRECIO_UNITARIO:  '||p_precio_unitario;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_linea) IS NULL OR
TRIM(p_id_pedido) IS NULL OR
TRIM(p_id_material) IS NULL OR
TRIM(p_unidades) IS NULL OR
TRIM(p_precio_unitario) IS NULL) 

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM LINEAS_PEDIDO
WHERE ID_LINEA =p_id_linea AND
ID_PEDIDO = p_id_pedido 
AND ROWNUM = 1;

RAISE E_LINEAPEDIDO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;


-- Verifica que el pedido al que corresponde la lnea existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM PEDIDOS
WHERE ID_PEDIDO =p_id_pedido
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_PEDIDO_NOEXISTE;
END;

-- Verifica que el tipo de dispositivo (material) del pedido existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = p_id_material
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_TIPODISPOSITIVO_NOEXISTE;
END;


-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO LINEAS_PEDIDO (ID_LINEA,ID_PEDIDO, ID_MATERIAL, UNIDADES, PRECIO_UNITARIO)
VALUES (p_id_linea, p_id_pedido, p_id_material, p_unidades, p_precio_unitario);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_LINEASPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_LINEAPEDIDO_EXISTE THEN
		RSP := 'ERROR: La lnea del pedido ya ha sido dada de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_LINEASPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_LINEASPEDIDO', param_entrada, 'RSP:  '||RSP);

	WHEN E_PEDIDO_NOEXISTE THEN
		RSP := 'ERROR: El pedido al que corresponde la linea no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_LINEASPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_TIPODISPOSITIVO_NOEXISTE THEN
		RSP := 'ERROR: El tipo de dispositivo solicitado en el pedido no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_LINEASPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_LINEASPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_LINEASPEDIDO;

/

-- BAJA LINEAS_PEDIDO

CREATE OR REPLACE
PROCEDURE PR_BAJA_LINEAPEDIDO
/**********************************************************************
NOMBRE:   PR_BAJA_LINEAPEDIDO
PROPSITO:  Gestiona las bajas de las lneas de pedido
PARMETROS:
Entrada:  p_id_linea: identificador de la lnea
p_id_pedido identificador del pedido

Salida:  RSP
Pre-condicin: Existe la lnea de pedido a dar de baja y existe el pedido
Post-condicin: Se da de baja la lnea del pedido
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  la lnea de pedido no existe en la base de datos
-	ERROR: el pedido no existe en la base de datos
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_linea IN LINEAS_PEDIDO.ID_LINEA%TYPE, p_id_pedido IN LINEAS_PEDIDO.ID_PEDIDO%
TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO				EXCEPTION;
E_LINEAPEDIDO_NOEXISTE			EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_LINEA: '||p_id_linea||', P_ID_PEDIDO: '||p_id_pedido;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_linea) IS NULL OR
TRIM(p_id_pedido) IS NULL)

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM LINEAS_PEDIDO
WHERE ID_LINEA = p_id_linea
AND ID_PEDIDO = p_id_pedido
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_LINEAPEDIDO_NOEXISTE;
END;


-- Si ha pasado todas las validaciones, realiza el borrado

DELETE FROM LINEAS_PEDIDO
WHERE ID_LINEA = p_id_linea AND ID_PEDIDO = p_id_pedido;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_LINEAPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_LINEAPEDIDO_NOEXISTE THEN
		RSP := 'ERROR: La lnea no existe para ese pedido';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_LINEAPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_LINEAPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_LINEAPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_LINEAPEDIDO;


/

-- MODIFICACIN LINEAS_PEDIDO

CREATE OR REPLACE
PROCEDURE PR_MODIF_LINEAPEDIDO 
/*********************************************************************
NOMBRE:   PR_MODIF_LINEAPEDIDO
PROPSITO:  Gestiona las modificaciones de las lneas de pedido
PARMETROS:
Entrada:  p_id_linea:  identificador de la lnea
p_id_pedido: identificador del pedido al que pertenece la lnea que se quiere modificar
p_id_material:  identificador del tipo de dispositivo
p_unidades:  unidades que se piden del tipo de dispositivo
p_precio_unitario:  precio unitario de cada tipo de dispositivo

Salida:  RSP
Pre-condicin: Existe el pedido y la lnea a modificar. Si se modifica el material, se verifica que el
Tipo de dispositivo exista.
Post-condicin: Se modifica la lnea de pedido
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador de la lnea no puede ser nulo
-	ERROR:  el identificador del pedido  no puede ser nulo
-	ERROR: el identificador de la lnea no existe para ese pedido
-	ERROR: el identificador del material no existe
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_linea IN LINEAS_PEDIDO.ID_LINEA%TYPE, p_id_pedido IN LINEAS_PEDIDO.ID_PEDIDO%TYPE, p_id_material IN LINEAS_PEDIDO.ID_MATERIAL%TYPE, p_unidades IN LINEAS_PEDIDO.UNIDADES%TYPE, p_precio_unitario IN LINEAS_PEDIDO.PRECIO_UNITARIO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_LINEAPEDIDO_NOEXISTE		EXCEPTION;
E_PEDIDO_NOEXISTE		EXCEPTION;
E_MATERIAL_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_LINEA: '||p_id_linea||', P_ID_PEDIDO: '||p_id_pedido||', P_ID_MATERIAL:  '||p_id_material||', P_UNIDADES:  '||p_unidades||', P_PRECIO_UNITARIO:  '||p_precio_unitario;


-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_linea) IS NULL
OR TRIM(p_id_pedido) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM LINEAS_PEDIDO
WHERE ID_LINEA = p_id_linea
AND ID_PEDIDO = p_id_pedido
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_PEDIDO_NOEXISTE;
END;


-- Si se va a modificar el tipo de dispositivo de la lnea de pedido, se verifica que exista en la base de datos

if (p_id_material is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = p_id_material
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_MATERIAL_NOEXISTE;
END;
END IF;


-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE LINEAS_PEDIDO
SET ID_MATERIAL = nvl(p_id_material, ID_MATERIAL),
UNIDADES = nvl(p_unidades, UNIDADES),
PRECIO_UNITARIO = nvl(p_precio_unitario, PRECIO_UNITARIO)
WHERE ID_LINEA = p_id_linea AND
ID_PEDIDO = p_id_pedido;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_LINEAPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_LINEAPEDIDO_NOEXISTE THEN
		RSP := 'ERROR: La lnea a modificar no existe para el pedido';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_LINEAPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_LINEAPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;


WHEN E_MATERIAL_NOEXISTE THEN
		RSP := 'ERROR: El tipo de dispositivo de la lnea del pedido no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_LINEAPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_LINEAPEDIDO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_LINEAPEDIDO;


/

-- ALTA DISPOSITIVOS

CREATE OR REPLACE
PROCEDURE PR_ALTA_DISPOSITIVOS
/***********************************************************************
NOMBRE:   PR_ALTA_DISPOSITIVOS
PROPSITO:  Gestiona las altas de los dispositivos informticos de la empresa
PARMETROS:
Entrada:  p_tipo:  tipo de dispositivo
p_numero_serie:  nmero de serie del dispositivo
p_fecha_compra:   fecha de compra del dispositivo
p_precio_compra:  precio de compra del dispositivo
p_fecha_instalacion:  fecha de instalacin del dispositivo
p_id_usuario:  identificador del usuario al que se asigna el dispositivo
p_fecha_ultimo_inventario:  fecha del ltimo inventario del dispositivo

Salida:  RSP
Pre-condicin: No existe el dispositivo a dar de alta
Post-condicin: Se da de alta el dispositivo
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  la dispositivo ya existe
-	ERROR: el tipo de dispositivo no existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_tipo IN DISPOSITIVOS.TIPO%TYPE, p_numero_serie IN DISPOSITIVOS.NUMERO_SERIE%TYPE, 
p_fecha_instalacion IN DISPOSITIVOS.FECHA_INSTALACION%TYPE, 
p_id_usuario IN DISPOSITIVOS.ID_USUARIO%TYPE, 
p_fecha_compra IN DISPOSITIVOS.FECHA_COMPRA%TYPE, 
p_precio_compra IN DISPOSITIVOS.PRECIO_COMPRA%TYPE, 
p_fecha_ultimo_inventario IN DISPOSITIVOS.FECHA_ULTIMO_INVENTARIO%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_DISPOSITIVO_EXISTE		EXCEPTION;
E_TIPODISPOSITIVO_NOEXISTE	EXCEPTION;
E_USUARIO_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_TIPO:  '||p_tipo||',  P_NUMERO_SERIE:  '||p_numero_serie||',  
P_FECHA_INSTALACION: '||p_fecha_instalacion||',  P_ID_USUARIO: '||p_id_usuario||', 
P_FECHA_COMPRA:  '||p_fecha_compra||',  P_PRECIO_COMPRA:  '||p_precio_compra||',  
P_FECHA_ULTIMO_INVENTARIO:  '||p_fecha_ultimo_inventario;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_tipo) IS NULL OR
TRIM(p_numero_serie) IS NULL OR
TRIM(p_fecha_compra) IS NULL OR
TRIM(p_precio_compra) IS NULL) 

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS
WHERE NUMERO_SERIE =p_numero_serie
AND ROWNUM = 1;

RAISE E_DISPOSITIVO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;

-- Verifica que el tipo de dispositivo exista

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO =p_tipo
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_TIPODISPOSITIVO_NOEXISTE;
END;

-- Verifica que el usuario al que se asigna el dispositivo exista

BEGIN
IF (p_id_usuario is NOT NULL) THEN

SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO =p_id_usuario
AND ROWNUM = 1;

END IF;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_USUARIO_NOEXISTE;
        


END;

-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO DISPOSITIVOS (TIPO, NUMERO_SERIE, FECHA_INSTALACION, ID_USUARIO, FECHA_COMPRA, PRECIO_COMPRA, FECHA_ULTIMO_INVENTARIO, FECHA_ALTA, FECHA_BAJA)
VALUES (p_tipo, p_numero_serie, p_fecha_instalacion, p_id_usuario, p_fecha_compra, p_precio_compra, p_fecha_ultimo_inventario, SYSDATE, NULL);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DISPOSITIVOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_DISPOSITIVO_EXISTE THEN
		RSP := 'ERROR: El dispositivo ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DISPOSITIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DISPOSITIVOS', param_entrada, 'RSP:  '||RSP);

	WHEN E_TIPODISPOSITIVO_NOEXISTE THEN
		RSP := 'ERROR: El tipo de dispositivo no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DISPOSITIVOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_USUARIO_NOEXISTE THEN
		RSP := 'ERROR: El usuario responsable del dispositivo no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DISPOSITIVOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_DISPOSITIVOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_DISPOSITIVOS;

/

-- BAJA DISPOSITIVOS

CREATE OR REPLACE
PROCEDURE PR_BAJA_DISPOSITIVO
/**********************************************************************
NOMBRE:   PR_BAJA_DISPOSITIVO
PROPSITO:  Gestiona las bajas de los dispositivos informticos de la empresa
PARMETROS:
Entrada:  p_id_dispositivo identificador del dispositivo
Salida:  RSP
Pre-condicin: Existe el dispositivo a dar de baja
Post-condicin: Se da de baja el dispositivo
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el dispositivo no existe en la base de datos
-	ERROR: el dispositivo ya haba sido dada de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_dispositivo IN DISPOSITIVOS.ID_DISPOSITIVO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			        EXCEPTION;
E_DISPOSITIVO_NOEXISTE			EXCEPTION;
E_DISPOSITIVO_YABAJA			EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER; 
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_DISPOSITIVO: '||p_id_dispositivo;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_dispositivo) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = p_id_dispositivo
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_DISPOSITIVO_NOEXISTE;
END;

-- Verifica que el dispositivo no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = p_id_dispositivo
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_DISPOSITIVO_YABAJA;
END;

-- Si ha pasado todas las validaciones, realiza el borrado

UPDATE DISPOSITIVOS
SET FECHA_BAJA = SYSDATE
WHERE ID_DISPOSITIVO = p_id_dispositivo;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_DISPOSITIVO_NOEXISTE THEN
		RSP := 'ERROR: El dispositivo no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_DISPOSITIVO_YABAJA THEN
		RSP := 'ERROR: El dispositivo ya haba sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);
	
	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_DISPOSITIVO;

/

-- MODIFICACIN DISPOSITIVOS

CREATE OR REPLACE
PROCEDURE PR_MODIF_DISPOSITIVO 
/*********************************************************************
NOMBRE:   PR_MODIF_DISPOSITIVO
PROPSITO:  Gestiona las modificaciones de los dispositivos informticos de la empresa
PARMETROS:
Entrada:  p_id_dispositivo:  identificador del dispositivo
p_tipo:  tipo del dispositivo
p_numero_serie:  nmero de serie del dispositivo
p_fecha_instalacion:  fecha de instalacin del dispositivo
p_id_usuario:  identificador del usuario al que se ha asignado el dispositivo
p_fecha_compra:  fecha de compra del dispositivo
p_precio_compra:  precio de compra del dispositivo
p_fecha_ultimo_inventario:  fecha del ltimo inventario del dispositivo

Salida:  RSP
Pre-condicin: Existe el dispositivo a modificar
Post-condicin: Se modifica el dispositivo
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador del dispositivo a modificar no existe
-	ERROR:  el identificador del dispositivo a modificar no puede ser nulo
-	ERROR: el dispositivo que se va a modificar est de baja
-	ERROR: el identificador del tipo de dispositivo no existe
-	ERROR: el identificador del usuario al que se ha asignado no existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_dispositivo IN DISPOSITIVOS.ID_DISPOSITIVO%TYPE, p_tipo IN DISPOSITIVOS.TIPO%TYPE, p_numero_serie IN DISPOSITIVOS.NUMERO_SERIE%TYPE, p_fecha_instalacion IN DISPOSITIVOS.FECHA_INSTALACION%TYPE, p_id_usuario IN DISPOSITIVOS.ID_USUARIO%TYPE, p_fecha_compra IN DISPOSITIVOS.FECHA_COMPRA%TYPE, p_precio_compra IN DISPOSITIVOS.PRECIO_COMPRA%TYPE, p_fecha_ultimo_inventario IN DISPOSITIVOS.FECHA_ULTIMO_INVENTARIO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_DISPOSITIVO_NOEXISTE		EXCEPTION;
E_DISPOSITIVO_BAJA		EXCEPTION;
E_TIPODISPOSITIVO_NOEXISTE	EXCEPTION;
E_USUARIO_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_DISPOSITIVO: '||p_id_dispositivo||', P_TIPO:  '||p_tipo||',  P_NUMERO_SERIE:  '||p_numero_serie||',  P_FECHA_INSTALACION: '||p_fecha_instalacion||',  P_ID_USUARIO: '||p_id_usuario||',  P_FECHA_COMPRA:  '||p_fecha_compra||',  P_PRECIO_COMPRA:  '||p_precio_compra||',  P_FECHA_ULTIMO_INVENTARIO:  '||p_fecha_ultimo_inventario;


-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_dispositivo) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = p_id_dispositivo
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_DISPOSITIVO_NOEXISTE;
END;

-- Verifica que el dispositivo no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = p_id_dispositivo
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_DISPOSITIVO_BAJA;
END;

-- Si se va a modificar el tipo de dispositivo se verifica que exista en la base de datos
if (p_tipo is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_DISPOSITIVO
WHERE ID_TIPO_DISPOSITIVO = p_tipo
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_TIPODISPOSITIVO_NOEXISTE;
END;
END IF;

-- Si se va a modificar el usuario al que se ha asignado el dispositivo, se verifica que exista en la base de datos

if (p_id_usuario is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_usuario
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_USUARIO_NOEXISTE;
END;
END IF;

-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE DISPOSITIVOS
SET TIPO = nvl(p_tipo, TIPO),
NUMERO_SERIE = nvl(p_numero_serie, NUMERO_SERIE),
FECHA_INSTALACION = nvl(p_fecha_instalacion, FECHA_INSTALACION),
ID_USUARIO = nvl(p_id_usuario, ID_USUARIO),
FECHA_COMPRA = nvl(p_fecha_compra, FECHA_COMPRA),
PRECIO_COMPRA  = nvl(p_precio_compra, PRECIO_COMPRA),
FECHA_ULTIMO_INVENTARIO = nvl(p_fecha_ultimo_inventario, FECHA_ULTIMO_INVENTARIO)
WHERE ID_DISPOSITIVO = p_id_dispositivo;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_DISPOSITIVO_NOEXISTE THEN
		RSP := 'ERROR: El dispositivo a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_DISPOSITIVO_BAJA THEN
		RSP := 'ERROR: El registro que quiere modificar ha sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_TIPODISPOSITIVO_NOEXISTE THEN
		RSP := 'ERROR: El tipo de dispositivo no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_USUARIO_NOEXISTE THEN
		RSP := 'ERROR: El usuario al que se ha asignado el dispositivo no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_DISPOSITIVO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_DISPOSITIVO;

/

-- ALTA PRESUPUESTOS

CREATE OR REPLACE
PROCEDURE PR_ALTA_PRESUPUESTOS
/***********************************************************************
NOMBRE:   PR_ALTA_PRESUPUESTOS
PROPSITO:  Gestiona las altas de los presupuestos de los pedidos de la empresa
PARMETROS:
Entrada: p_id_proveedor:  identificador del proveedor que da el presupuesto
p_id_pedido:  identificador del  pedido sobre el que se hace el presupuesto
p_fecha_presupuesto:  fecha en la que se da el presupuesto
p_importe:  importe total del presupuesto
p_aceptado: indica si el presupuesto ha sido aceptado

Salida:  RSP
Pre-condicin: No existe el presupuesto a dar de alta. Tienen que existir el proveedor y el pedido. No pueden existir dos presupuestos para el mismo proveedor y el mismo pedido. Si aceptado = Y, se comprobar que no exista otro presupuesto 
aceptado para ese mismo pedido

Post-condicin: Se da de alta el presupuesto
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el presupuesto ya existe
-	ERROR: el proveedor que da el presupuesto no existe
-	ERROR: el pedido asociado al presupuesto no existe
-	ERROR: ya existe un presupuesto para ese proveedor y ese pedido
-	ERROR: ya existe otro presupuesto para ese mismo pedido que ha sido aceptado
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_proveedor IN PRESUPUESTOS.ID_PROVEEDOR%TYPE, p_id_pedido IN 
PRESUPUESTOS.ID_PEDIDO%TYPE, p_fecha_presupuesto IN PRESUPUESTOS.FECHA_PRESUPUESTO%
TYPE, p_importe IN PRESUPUESTOS.IMPORTE%TYPE, p_aceptado IN PRESUPUESTOS.ACEPTADO%
TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_PRESUPUESTO_EXISTE		EXCEPTION;
E_PROVEEDOR_NOEXISTE		EXCEPTION;
E_PEDIDO_NOEXISTE		EXCEPTION;
E_PRESUACEPTADO_EXISTE  	EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_PROVEEDOR: '||p_id_proveedor||', P_ID_PEDIDO: 
'||p_id_pedido||', P_FECHA_PRESUPUESTO:  '||p_fecha_presupuesto||', P_IMPORTE:  
'||p_importe||', P_ACEPTADO:  '||p_aceptado;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_proveedor) IS NULL OR
TRIM(p_id_pedido) IS NULL OR
TRIM(p_fecha_presupuesto) IS NULL OR
TRIM(p_importe) IS NULL) 

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM PRESUPUESTOS
WHERE ID_PROVEEDOR =p_id_proveedor AND
ID_PEDIDO = p_id_pedido AND
FECHA_PRESUPUESTO = p_fecha_presupuesto AND
IMPORTE = p_importe
AND ROWNUM = 1;

RAISE E_PRESUPUESTO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;


-- Verifica que el proveedor del pedido existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM PROVEEDORES
WHERE ID_PROVEEDOR =p_id_proveedor
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_PROVEEDOR_NOEXISTE;
END;

-- Verifica que el pedido sobre el que se realiza el presupuesto existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM PEDIDOS
WHERE ID_PEDIDO =p_id_pedido
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_PEDIDO_NOEXISTE;
END;

-- Verifica que no haya otro presupuesto para ese pedido de otro proveedor que tambin haya sido aceptado.

BEGIN
SELECT 'S'
INTO EXISTE
FROM PRESUPUESTOS
WHERE ID_PEDIDO =p_id_pedido
AND ACEPTADO = p_aceptado
AND ACEPTADO = 'S'
AND ROWNUM = 1;

RAISE E_PRESUACEPTADO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;


-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO PRESUPUESTOS (ID_PROVEEDOR,ID_PEDIDO, FECHA_PRESUPUESTO, IMPORTE, 
ACEPTADO)
VALUES (p_id_proveedor, p_id_pedido, p_fecha_presupuesto, p_importe, p_aceptado);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PRESUPUESTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_PRESUPUESTO_EXISTE THEN
		RSP := 'ERROR: El presupuesto ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PRESUPUESTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PRESUPUESTOS', param_entrada, 'RSP:  '||RSP);

	WHEN E_PROVEEDOR_NOEXISTE THEN
		RSP := 'ERROR: El proveedor que da el presupuesto no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PRESUPUESTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_PEDIDO_NOEXISTE THEN
		RSP := 'ERROR: El pedido asociado al presupuesto no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PRESUPUESTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_PRESUACEPTADO_EXISTE THEN
		RSP := 'ERROR: Ya existe un presupuesto aceptado para ese mismo 
pedido y no puede haber ms de uno';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PRESUPUESTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_PRESUPUESTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_PRESUPUESTOS;


/

-- BAJA PRESUPUESTOS

CREATE OR REPLACE
PROCEDURE PR_BAJA_PRESUPUESTO
/**********************************************************************
NOMBRE:   PR_BAJA_PRESUPUESTO
PROPSITO:  Gestiona las bajas de los presupuestos
PARMETROS:
Entrada:  p_id_presupuesto: identificador del presupuesto

Salida:  RSP
Pre-condicin: Existe el presupuesto a dar de baja
Post-condicin: Se da de baja el presupuesto
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el presupuesto no existe en la base de datos
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_presupuesto IN PRESUPUESTOS.ID_PRESUPUESTO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_PRESUPUESTO_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_PRESUPUESTO: '||p_id_presupuesto;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_presupuesto) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM PRESUPUESTOS
WHERE ID_PRESUPUESTO = p_id_presupuesto
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_PRESUPUESTO_NOEXISTE;
END;


-- Si ha pasado todas las validaciones, realiza el borrado

DELETE FROM PRESUPUESTOS
WHERE ID_PRESUPUESTO = p_id_presupuesto;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_PRESUPUESTO_NOEXISTE THEN
		RSP := 'ERROR: El presupuesto no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_PRESUPUESTO;


/

-- MODIFICACIN PRESUPUESTOS

CREATE OR REPLACE
PROCEDURE PR_MODIF_PRESUPUESTO
/*********************************************************************
NOMBRE:   PR_MODIF_PRESUPUESTO
PROPSITO:  Gestiona las modificaciones de los presupuestos
PARMETROS:
Entrada:  p_id_presupuesto:  identificador del presupuesto
p_id_proveedor:  identificador del proveedor que dar el presupuesto
p_id_pedido:  identificador del pedido para el que se ha dado el presupuesto
p_fecha_presupuesto:  fecha del presupuesto
p_importe:  importe total del presupuesto
p_aceptado:  indica si el presupuesto ha sido aceptado

Salida:  RSP
Pre-condicin: Existe el presupuesto a modificar. Si se va a modificar el proveedor, se verifica que el nuevo proveedor exista en la BD. Si se va a modificar el pedido, se verifica que el nuevo pedido exista en la BD. Si se va a modificar el campo aceptado ponindolo a Y, se comprobar que no exista otro presupuesto aceptado para ese mismo pedido.

Post-condicin: Se modifica el presupuesto
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador del presupuesto a modificar no existe
-	ERROR:  el identificador del presupuesto  no puede ser nulo
-	ERROR: el identificador del proveedor que da el presupuesto no existe
-	ERROR: el identificador del pedido para el que se da el presupuesto no existe
-	ERROR: ya existe otro presupuesto para ese mismo pedido que ha sido aceptado
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_presupuesto IN PRESUPUESTOS.ID_PRESUPUESTO%TYPE, p_id_proveedor IN PRESUPUESTOS.ID_PROVEEDOR%TYPE,
p_id_pedido IN PRESUPUESTOS.ID_PEDIDO%TYPE, p_fecha_presupuesto IN PRESUPUESTOS.FECHA_PRESUPUESTO%TYPE,
p_importe IN PRESUPUESTOS.IMPORTE%TYPE, p_aceptado IN PRESUPUESTOS.ACEPTADO%TYPE, RSP OUT VARCHAR2)


IS
E_CAMPO_NULO			EXCEPTION;
E_PRESUPUESTO_NOEXISTE		EXCEPTION;
E_PROVEEDOR_NOEXISTE		EXCEPTION;
E_PEDIDO_NOEXISTE		EXCEPTION;
E_PRESUACEPTADO_EXISTE          EXCEPTION;

existe		VARCHAR2(1);
pedido          VARCHAR2(6);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN
param_entrada := 'P_ID_PROVEEDOR: '||p_id_proveedor||', P_ID_PEDIDO: '||p_id_pedido||',
P_FECHA_PRESUPUESTO:  '||p_fecha_presupuesto||', P_IMPORTE:  '||p_importe||',
P_ACEPTADO:  '||p_aceptado;


-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_presupuesto) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM PRESUPUESTOS
WHERE ID_PRESUPUESTO = p_id_presupuesto
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_PRESUPUESTO_NOEXISTE;
END;


-- Si se va a modificar el proveedor se verifica que exista en la base de datos

IF (p_id_proveedor IS NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM PROVEEDORES
WHERE ID_PROVEEDOR = p_id_proveedor
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_PROVEEDOR_NOEXISTE;
END;
END IF;

-- Si se va a modificar el pedido al que pertenece el presupuesto, se verifica que exista en la base de datos

IF (p_id_pedido IS NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM PEDIDOS
WHERE ID_PEDIDO = p_id_pedido
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_PEDIDO_NOEXISTE;
END;
END IF;

-- Si se va a modificar el campo aceptado
-- Verifica que no haya otro presupuesto para ese pedido de otro proveedor que tambin haya sido aceptado.

IF (p_aceptado IS NOT NULL) THEN

BEGIN
SELECT ID_PEDIDO
INTO PEDIDO
FROM PRESUPUESTOS
WHERE ID_PRESUPUESTO = p_id_presupuesto;

SELECT 'S'
INTO EXISTE
FROM PRESUPUESTOS
WHERE ID_PEDIDO =PEDIDO
AND ACEPTADO = p_aceptado
AND ACEPTADO = 'S'
AND ROWNUM = 1;

RAISE E_PRESUACEPTADO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;

END;
END IF;
-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE PRESUPUESTOS
SET ID_PROVEEDOR = nvl(p_id_proveedor, ID_PROVEEDOR),
ID_PEDIDO = nvl(p_id_pedido, ID_PEDIDO),
FECHA_PRESUPUESTO = nvl(p_fecha_presupuesto, FECHA_PRESUPUESTO),
IMPORTE = nvl(p_importe, IMPORTE),
ACEPTADO = nvl(p_aceptado, ACEPTADO)
WHERE ID_PRESUPUESTO = p_id_presupuesto;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_PRESUPUESTO_NOEXISTE THEN
		RSP := 'ERROR: El presupuesto a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;


WHEN E_PROVEEDOR_NOEXISTE THEN
		RSP := 'ERROR: El proveedor no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_PEDIDO_NOEXISTE THEN
		RSP := 'ERROR: El pedido al que corresponde el presupuesto no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;
 WHEN E_PRESUACEPTADO_EXISTE THEN
		RSP := 'ERROR: Ya existe un presupuesto aceptado para ese mismo pedido y no puede haber mas de uno';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;       
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_PRESUPUESTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_PRESUPUESTO;

/

-- ALTA RECEPCIONES

CREATE OR REPLACE
PROCEDURE PR_ALTA_RECEPCIONES
/***********************************************************************
NOMBRE:   PR_ALTA_RECEPCIONES
PROPSITO:  Gestiona las altas de las recepciones de los  pedidos de la empresa
PARMETROS:
Entrada: p_id_pedido:  identificador del pedido del que se hace la recepcin
p_id_responsable:  identificador del  usuario responsable de la recepcin del pedido
p_fecha_recepcion:  fecha en la que se recibe el material del pedido.
p_observaciones:  recoge las incidencias que se hayan producido en la recepcin del material

Salida:  RSP
Pre-condicin: No existe la recepcin a dar de alta.  Tienen  que existir el pedido y el usuario responsable. El usuario responsable debe pertenecer al departamento de Informtica

Post-condicin: Se da de alta la recepcin
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  la recepcin ya existe
-	ERROR: el pedido que se ha recepcionado no existe
-	ERROR: el usuario responsable de la recepcin del pedido no existe
-	ERROR: el usuario responsable no pertenece al departamento de informtica
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_pedido IN RECEPCIONES.ID_PEDIDO%TYPE, p_id_responsable IN RECEPCIONES.ID_RESPONSABLE%TYPE,
p_fecha_recepcion IN RECEPCIONES.FECHA_RECEPCION%TYPE, p_observaciones IN RECEPCIONES.OBSERVACIONES%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_RECEPCION_EXISTE		EXCEPTION;
E_PEDIDO_NOEXISTE		EXCEPTION;
E_RESPONSABLE_NOEXISTE		EXCEPTION;
E_RESPONSABLE_NOINFORMATICO     EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_PEDIDO: '||p_id_pedido||', P_ID_RESPONSABLE: '||p_id_responsable||
', P_FECHA_RECEPCION:  '||p_fecha_recepcion||', P_OBSERVACIONES:  '||p_observaciones;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_pedido) IS NULL OR
TRIM(p_id_responsable) IS NULL OR
TRIM(p_fecha_recepcion) IS NULL)
 

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM RECEPCIONES
WHERE ID_PEDIDO =p_id_pedido AND
ID_RESPONSABLE = p_id_responsable AND
FECHA_RECEPCION = p_fecha_recepcion
AND ROWNUM = 1;

RAISE E_RECEPCION_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;


-- Verifica que el pedido recepcionado existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM PEDIDOS
WHERE ID_PEDIDO =p_id_pedido
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_PEDIDO_NOEXISTE;
END;

-- Verifica que el usuario responsable de recepcionar el pedido existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO =p_id_responsable
AND ROWNUM = 1;
EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_RESPONSABLE_NOEXISTE;
END;

-- Verifica que el usuario responsable de recepcionar el pedido sea del departamento de Informtica

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO =p_id_responsable
AND ID_DEPARTAMENTO IN
(SELECT ID_DEPARTAMENTO FROM DEPARTAMENTOS
WHERE NOMBRE_DEPARTAMENTO = 'INFORMATICA')
AND ROWNUM = 1;
EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_RESPONSABLE_NOINFORMATICO;
END;
-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO RECEPCIONES (ID_PEDIDO, ID_RESPONSABLE, FECHA_RECEPCION, OBSERVACIONES)
VALUES (p_id_pedido, p_id_responsable, p_fecha_recepcion, p_observaciones);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_RECEPCIONES', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_RECEPCION_EXISTE THEN
		RSP := 'ERROR: La recepcin ya ha sido dada de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_RECEPCIONES', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_RECEPCIONES', param_entrada, 'RSP:  '||RSP);

	WHEN E_PEDIDO_NOEXISTE THEN
		RSP := 'ERROR: El pedido que se ha recepcionado no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_RECEPCIONES', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_RESPONSABLE_NOEXISTE THEN
		RSP := 'ERROR: El usuario responsable de la recepcin del pedido no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_RECEPCIONES', param_entrada, 'RSP:  '||RSP);

	COMMIT;
        
    WHEN E_RESPONSABLE_NOINFORMATICO THEN
		RSP := 'ERROR: El usuario responsable no pertenece al departamento de Informtica';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_RECEPCIONES', param_entrada, 'RSP:  '||RSP);

	COMMIT;    
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_RECEPCIONES', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_RECEPCIONES;


/

-- BAJA RECEPCIONES

CREATE OR REPLACE
PROCEDURE PR_BAJA_RECEPCION
/**********************************************************************
NOMBRE:   PR_BAJA_RECEPCION
PROPSITO:  Gestiona las bajas de las recepciones de los pedidos de la empresa
PARMETROS:
Entrada:  p_id_recepcion: identificador de la recepcin

Salida:  RSP
Pre-condicin: Existe la recepcin a dar de baja
Post-condicin: Se da de baja la recepcin
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  la recepcin no existe en la base de datos
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_recepcion IN RECEPCIONES.ID_RECEPCION%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_RECEPCION_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_RECEPCION: '||p_id_recepcion;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_recepcion) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM RECEPCIONES
WHERE ID_RECEPCION = p_id_recepcion
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_RECEPCION_NOEXISTE;
END;



-- Si ha pasado todas las validaciones, realiza el borrado

DELETE FROM RECEPCIONES
WHERE ID_RECEPCION = p_id_recepcion;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_RECEPCION_NOEXISTE THEN
		RSP := 'ERROR: La recepcin no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_RECEPCION;


/

-- MODIFICACIN RECEPCIONES

CREATE OR REPLACE
PROCEDURE PR_MODIF_RECEPCION 
/*********************************************************************
NOMBRE:   PR_MODIF_RECEPCION
PROPSITO:  Gestiona las modificaciones de las recepciones de los pedidos de la empresa
PARMETROS:
Entrada: p_id_recepcion:  identificador de la recepcin de un pedido
p_id_pedido:  identificador del pedido que se recepciona
p_id_responsable:  identificador del usuario que recoge el pedido
p_fecha_recepcin:  fecha en la que se recepciona el pedido
p_observaciones:  refleja las incidencias que puedan producirse en la recepcin del material
Salida:  RSP
Pre-condicin: Existe la recepcin a modificar. Si se va a modificar el pedido, se verifica 
que el nuevo pedido exista en la BD. Si se va a modificar el responsable, se verifica que el 
nuevo responsable exista en la BD y que pertenezca al departamento de informtica.
Post-condicin: Se modifica la recepcin
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador de la recepcin a modificar no existe
-	ERROR:  el identificador de la recepcin  no puede ser nulo
-	ERROR: el identificador del pedido que se recepciona no existe
-	ERROR: el identificador del usuario responsable de recoger el pedido no existe
-	ERROR: el usuario responsable no pertenece al departamento de informtica
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_recepcion IN RECEPCIONES.ID_RECEPCION%TYPE, p_id_pedido IN RECEPCIONES.ID_PEDIDO%TYPE,
p_id_responsable IN RECEPCIONES.ID_RESPONSABLE%TYPE, 
p_fecha_recepcion IN RECEPCIONES.FECHA_RECEPCION%TYPE,
p_observaciones IN RECEPCIONES.OBSERVACIONES%TYPE, RSP OUT VARCHAR2)


IS
E_CAMPO_NULO			EXCEPTION;
E_RECEPCION_NOEXISTE		EXCEPTION;
E_PEDIDO_NOEXISTE		EXCEPTION;
E_RESPONSABLE_NOEXISTE		EXCEPTION;
E_RESPONSABLE_NOINFORMATICO     EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN
param_entrada := 'P_ID_RECEPCION: '||p_id_recepcion||', P_ID_PEDIDO: '||p_id_pedido||', 
P_ID_RESPONSABLE: '||p_id_responsable||', P_FECHA_RECEPCION:  '||p_fecha_recepcion||', 
P_OBSERVACIONES:  '||p_observaciones;


-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_recepcion) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM RECEPCIONES
WHERE ID_RECEPCION = p_id_recepcion
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_RECEPCION_NOEXISTE;
END;


-- Si se va a modificar el pedido se verifica que exista en la base de datos

IF (p_id_pedido IS NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM PEDIDOS
WHERE ID_PEDIDO = p_id_pedido
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_PEDIDO_NOEXISTE;
END;
END IF;

-- Si se va a modificar el usuario responsable de recoger el pedido, se verifica que exista en la base de datos

IF (p_id_responsable IS NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_responsable
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_RESPONSABLE_NOEXISTE;
END;
END IF;

-- Si se va a modificar el usuario responsable de recoger el pedido, se verifica que pertenezca al departamento deInformatica

IF (p_id_responsable IS NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_responsable
AND ID_DEPARTAMENTO IN
(SELECT ID_DEPARTAMENTO FROM DEPARTAMENTOS
WHERE NOMBRE_DEPARTAMENTO = 'INFORMATICA')
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_RESPONSABLE_NOINFORMATICO;
END;
END IF;

-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE RECEPCIONES
SET ID_PEDIDO = nvl(p_id_pedido, ID_PEDIDO),
ID_RESPONSABLE = nvl(p_id_responsable, ID_RESPONSABLE),
FECHA_RECEPCION = nvl(p_fecha_recepcion, FECHA_RECEPCION),
OBSERVACIONES = nvl(p_observaciones, OBSERVACIONES)
WHERE ID_RECEPCION = p_id_recepcion;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_RECEPCION_NOEXISTE THEN
		RSP := 'ERROR: La recepcin a modificar no ha sido dada de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;


WHEN E_PEDIDO_NOEXISTE THEN
		RSP := 'ERROR: El pedido a recepcionar no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_RESPONSABLE_NOEXISTE THEN
		RSP := 'ERROR: El usuario responsable de recoger el pedido no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;
        
WHEN E_RESPONSABLE_NOINFORMATICO THEN
		RSP := 'ERROR: El usuario responsable de recoger el pedido no pertenece al departamento de Informtica';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;        
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_RECEPCION', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_RECEPCION;

/

-- ALTA USUARIOS

CREATE OR REPLACE
PROCEDURE PR_ALTA_USUARIOS
/***********************************************************************
NOMBRE:   PR_ALTA_USUARIOS
PROPSITO:  Gestiona las altas de los usuarios de la empresa
PARMETROS:
Entradap_id_departamento:  identificador del departamento al que pertenece el usuario
p_nombre:  nombre del usuario
p_id_sede:  identificador de la sede donde trabaja el usuario
p_telfono:  telfono del usuario
p_email:  correo electrnico del usuario

Salida:  RSP
Pre-condicin: No existe el usuario a dar de alta
Post-condicin: Se da de alta el usuario
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR: el usuario ya existe
-	ERROR: el departamento asignado al usuario no existe
-	ERROR: la sede donde trabaja el usuario no existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_departamento IN USUARIOS.ID_DEPARTAMENTO%TYPE, p_nombre IN USUARIOS.NOMBRE%TYPE, p_telefono IN USUARIOS.TELEFONO%TYPE, p_email IN USUARIOS.EMAIL%TYPE, p_id_sede IN USUARIOS.ID_SEDE%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_USUARIO_EXISTE		EXCEPTION;
E_DEPARTAMENTO_NOEXISTE		EXCEPTION;
E_SEDE_NOEXISTE			EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_DEPARTAMENTO: '||p_id_departamento||', P_NOMBRE:  '||p_nombre||', P_TELEFONO:  '||p_telefono||', P_EMAIL:  '||p_email||', P_ID_SEDE:  '||p_id_sede;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_nombre) IS NULL OR
TRIM(p_id_departamento) IS NULL OR
TRIM(p_id_sede) IS NULL) 

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE NOMBRE =p_nombre AND
ID_DEPARTAMENTO = p_id_departamento AND
ID_SEDE = p_id_sede AND
TELEFONO = p_telefono
AND ROWNUM = 1;

RAISE E_USUARIO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;

-- Verifica que el departamento asignado al usuario existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM DEPARTAMENTOS
WHERE ID_DEPARTAMENTO =p_id_departamento
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_DEPARTAMENTO_NOEXISTE;
END;

-- Verifica que la sede donde trabaja el usuario existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM SEDES
WHERE ID_SEDE =p_id_sede
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_SEDE_NOEXISTE;
END;

-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO USUARIOS(ID_DEPARTAMENTO, NOMBRE, TELEFONO, EMAIL, ID_SEDE, FECHA_ALTA, FECHA_BAJA)
VALUES (p_id_departamento, p_nombre, p_telefono, p_email, p_id_sede, SYSDATE, NULL);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_USUARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_USUARIO_EXISTE THEN
		RSP := 'ERROR: El usuario ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_USUARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_USUARIOS', param_entrada, 'RSP:  '||RSP);

	WHEN E_DEPARTAMENTO_NOEXISTE THEN
		RSP := 'ERROR: El departamento asignado al usuario no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_USUARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_SEDE_NOEXISTE THEN
		RSP := 'ERROR: La sede donde trabaja el usuario no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_USUARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_USUARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_USUARIOS;


/

-- BAJA USUARIOS

CREATE OR REPLACE
PROCEDURE PR_BAJA_USUARIO
/**********************************************************************
NOMBRE:   PR_BAJA_USUARIO
PROPSITO:  Gestiona las bajas de los usuarios de la empresa
PARMETROS:
Entrada:  p_id_usuario identificador del usuario
Salida:  RSP
Pre-condicin: Existe el usuario a dar de baja
Post-condicin: Se da de baja el usuario
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el usuario no existe en la base de datos
-	ERROR: el usuario ya haba sido dada de baja
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_usuario IN USUARIOS.ID_USUARIO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_USUARIO_NOEXISTE		EXCEPTION;
E_USUARIO_YABAJA		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_USUARIO: '||p_id_usuario;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_usuario) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_usuario
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_USUARIO_NOEXISTE;
END;

-- Verifica que el usuario no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_usuario
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_USUARIO_YABAJA;
END;

-- Si ha pasado todas las validaciones, realiza el borrado

UPDATE USUARIOS
SET FECHA_BAJA = SYSDATE
WHERE ID_USUARIO = p_id_usuario;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_USUARIO_NOEXISTE THEN
		RSP := 'ERROR: El usuario no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_USUARIO_YABAJA THEN
		RSP := 'ERROR: El usuario ya haba sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_USUARIO', param_entrada, 'RSP:  '||RSP);
	
	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_USUARIO;


/

-- MODIFICACIN USUARIOS

CREATE OR REPLACE
PROCEDURE PR_MODIF_USUARIO 
/*********************************************************************
NOMBRE:   PR_MODIF_USUARIO
PROPSITO:  Gestiona las modificaciones de los usuarios de la empresa
PARMETROS:
Entrada:  p_id_usuario:  identificador del usuario
p_id_departamento:  identificador del departamento al que pertenece el usuario
p_nombre:  nombre del usuario
p_telfono:  telfono del usuario
p_email:  correo electrnico del usuario
p_id_sede:  identificador de la sede en la que trabaja el usuario

Salida:  RSP
Pre-condicin: Existe el usuario a modificar
Post-condicin: Se modifica el usuario
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador del usuario a modificar no existe
-	ERROR:  el identificador del usuario a modificar no puede ser nulo
-	ERROR: el usuario que se va a modificar est de baja
-	ERROR: el identificador del departamento no existe
-	ERROR: el identificador de la sede no existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_usuario IN USUARIOS.ID_USUARIO%TYPE, p_id_departamento IN USUARIOS.ID_DEPARTAMENTO%TYPE,
p_nombre IN USUARIOS.NOMBRE%TYPE, p_telefono IN USUARIOS.TELEFONO%TYPE, p_email IN USUARIOS.EMAIL%TYPE, 
p_id_sede IN USUARIOS.ID_SEDE%TYPE,  RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_USUARIO_NOEXISTE		EXCEPTION;
E_USUARIO_BAJA			EXCEPTION;
E_DEPARTAMENTO_NOEXISTE		EXCEPTION;
E_SEDE_NOEXISTE			EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_USUARIO: '||p_id_usuario||', P_ID_DEPARTAMENTO: '||p_id_departamento||', P_NOMBRE:  '||p_nombre||', P_TELEFONO: '||p_telefono||', P_EMAIL: '||p_email||', P_ID_SEDE: '||p_id_sede;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_usuario) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_usuario
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_USUARIO_NOEXISTE;
END;

-- Verifica que el usuario no est dado de baja

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_usuario
AND FECHA_BAJA IS NULL
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
	RAISE E_USUARIO_BAJA;
END;

-- Si se va a modificar el departamento al que pertenece el usuario se verifica que exista en la base de datos

IF (p_id_departamento IS NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM DEPARTAMENTOS
WHERE ID_DEPARTAMENTO = p_id_departamento
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_DEPARTAMENTO_NOEXISTE;
END;
END IF;

-- Si se va a modificar la sede en la que trabaja el usuario se verifica que exista en la base de datos

IF (p_id_sede IS NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM SEDES
WHERE ID_SEDE = p_id_sede
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_SEDE_NOEXISTE;
END;
END IF;

-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE USUARIOS
SET ID_DEPARTAMENTO = nvl(p_id_departamento, ID_DEPARTAMENTO),
NOMBRE = nvl(p_nombre, NOMBRE),
TELEFONO = nvl(p_telefono, TELEFONO),
EMAIL = nvl(p_email, EMAIL),
ID_SEDE = nvl(p_id_sede, ID_SEDE)
WHERE ID_USUARIO = p_id_usuario;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_USUARIO_NOEXISTE THEN
		RSP := 'ERROR: El usuario a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_USUARIO_BAJA THEN
		RSP := 'ERROR: El registro que quiere modificar ha sido dado de baja';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_DEPARTAMENTO_NOEXISTE THEN
		RSP := 'ERROR: El departamento al que pertenece el usuario no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_SEDE_NOEXISTE THEN
		RSP := 'ERROR: La sede en la que est destinado el usuario no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_USUARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_USUARIO;

/

-- ALTA MOVIMIENTOS

CREATE OR REPLACE
PROCEDURE PR_ALTA_MOVIMIENTOS
/***********************************************************************
NOMBRE:   PR_ALTA_MOVIMIENTOS
PROPSITO:  Gestiona los movimientos de los dispsitivos de la empresa
PARMETROS:
Entrada: p_id_tipo_movimientor:  identificador del tipo de movimiento
p_id_dispositivo:  identificador del  dispositivo sobre el que se realiza el movimiento
p_fecha_movimiento:  fecha en la que se realiza el movimiento
p_id_responsable:  identificador del usuario responsable del movimiento
p_id_usuario_origen:  identificador del usuario que tiene el dispositivo
p_id_usuario_destino:  identificador del nuevo usuario al que se ha asignado el dispositivo
Salida:  RSP
Pre-condicin: No existe el movimiento a dar de alta.  Tienen  que existir el
tipo de movimiento, el dispositivo y el usuario responsable de llevar a cabo el movimiento.
El usuario responsable debe pertenecer al departamento de Informtica.
Post-condicin: Se da de alta el movimiento
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el movimiento ya existe
-	ERROR: el tipo de movimiento no existe
-	ERROR: el dispositivo sobre el que se realiza el movimiento no existe
-	ERROR: el usuario que tiene asignado el dispositivo no existe
-	ERROR: el usuario al que se le asigna el dispositivo no existe
-	ERROR: el usuario responsable no existe
-	ERROR: el usuario responsable no pertenece al departamento de informtica
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_tipo_movimiento IN MOVIMIENTOS.ID_TIPO_MOVIMIENTO%TYPE, p_id_dispositivo IN 
MOVIMIENTOS.ID_DISPOSITIVO%TYPE, p_id_usuario_origen IN MOVIMIENTOS.ID_USUARIO_ORIGEN%
TYPE, p_id_usuario_destino IN MOVIMIENTOS.ID_USUARIO_DESTINO%TYPE, 
p_fecha_movimiento IN MOVIMIENTOS.FECHA_MOVIMIENTO%TYPE, p_id_responsable IN 
MOVIMIENTOS.ID_RESPONSABLE%TYPE, RSP OUT VARCHAR2)
IS
E_CAMPO_NULO			     EXCEPTION;
E_MOVIMIENTO_EXISTE		     EXCEPTION;
E_TIPOMOVIMIENTO_NOEXISTE	     EXCEPTION;
E_DISPOSITIVO_NOEXISTE		     EXCEPTION;
E_USUARIOORIGEN_NOEXISTE	     EXCEPTION;
E_USUARIODESTINO_NOEXISTE	     EXCEPTION;
E_USUARIORESPONSABLE_NOEXISTE	     EXCEPTION;
E_RESPONSABLE_NOINFORMATICO          EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_TIPO_MOVIMIENTO: '||p_id_tipo_movimiento||', 
P_ID_DISPOSITIVO: '||p_id_dispositivo||', P_ID_USUARIO_ORIGEN:  
'||p_id_usuario_origen||', P_ID_USUARIO_DESTINO:  '||p_id_usuario_destino||', 
P_FECHA_MOVIMIENTO:  '||p_fecha_movimiento||', P_ID_RESPONSABLE: 
'||p_id_responsable;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_tipo_movimiento) IS NULL OR
TRIM(p_id_dispositivo) IS NULL OR
TRIM(p_fecha_movimiento) IS NULL OR
TRIM(p_id_responsable) IS NULL) 

THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM MOVIMIENTOS	
WHERE ID_TIPO_MOVIMIENTO = p_id_tipo_movimiento AND
ID_DISPOSITIVO = p_id_dispositivo AND
FECHA_MOVIMIENTO = p_fecha_movimiento AND
ID_RESPONSABLE = p_id_responsable
AND ROWNUM = 1;

RAISE E_MOVIMIENTO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;


-- Verifica que el tipo de movimiento existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_MOVIMIENTO
WHERE ID_TIPO_MOVIMIENTO =p_id_tipo_movimiento
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_TIPOMOVIMIENTO_NOEXISTE;
END;

-- Verifica que el dispositivo sobre el que se realiza el movimiento existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO =p_id_dispositivo
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_DISPOSITIVO_NOEXISTE;
END;


-- Verifica que el usuario que tiene asignado el dispositivo existe

IF (p_id_usuario_origen IS NOT NULL ) THEN
BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO =p_id_usuario_origen
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_USUARIOORIGEN_NOEXISTE;
END;

END IF;
-- Verifica que el usuario al que se asigna el dispositivo existe

IF (p_id_usuario_destino IS NOT NULL) THEN
BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO =p_id_usuario_destino
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_USUARIODESTINO_NOEXISTE;
END;
END IF;

-- Verifica que el usuario responsable del movimiento existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO =p_id_responsable
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_USUARIORESPONSABLE_NOEXISTE;
END;

-- Verifica que el usuario responsable sea del departamento de Informtica

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO =p_id_responsable
AND ID_DEPARTAMENTO IN
(SELECT ID_DEPARTAMENTO FROM DEPARTAMENTOS
WHERE NOMBRE_DEPARTAMENTO = 'INFORMATICA')
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_RESPONSABLE_NOINFORMATICO;
END;
-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO MOVIMIENTOS_AUX (ID_TIPO_MOVIMIENTO,ID_DISPOSITIVO, 
ID_USUARIO_ORIGEN, ID_USUARIO_DESTINO, FECHA_MOVIMIENTO, ID_RESPONSABLE)
VALUES (p_id_tipo_movimiento, p_id_dispositivo, p_id_usuario_origen, 
p_id_usuario_destino, p_fecha_movimiento, p_id_responsable);

INSERT INTO MOVIMIENTOS (ID_TIPO_MOVIMIENTO,ID_DISPOSITIVO, 
ID_USUARIO_ORIGEN, ID_USUARIO_DESTINO, FECHA_MOVIMIENTO, ID_RESPONSABLE)
VALUES (p_id_tipo_movimiento, p_id_dispositivo, p_id_usuario_origen, 
p_id_usuario_destino, p_fecha_movimiento, p_id_responsable);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_MOVIMIENTO_EXISTE THEN
		RSP := 'ERROR: El movimiento ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	WHEN E_TIPOMOVIMIENTO_NOEXISTE THEN
		RSP := 'ERROR: El tipo de movimiento no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_DISPOSITIVO_NOEXISTE THEN
		RSP := 'ERROR: El dispositivo sobre el que se realiza el movimiento 
no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_USUARIOORIGEN_NOEXISTE THEN
		RSP := 'ERROR: El usuario que tiene asignado el dispositivo no 
existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_USUARIODESTINO_NOEXISTE THEN
		RSP := 'ERROR: El usuario al que se asigna el dispositivo no 
existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;
WHEN E_USUARIORESPONSABLE_NOEXISTE THEN
		RSP := 'ERROR: El usuario responsable del movimiento no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;
        
WHEN E_RESPONSABLE_NOINFORMATICO THEN
		RSP := 'ERROR: El usuario responsable del movimiento no es del departamento de Informtica';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;        

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_MOVIMIENTOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_MOVIMIENTOS;

/

-- BAJA MOVIMIENTOS

CREATE OR REPLACE
PROCEDURE PR_BAJA_MOVIMIENTO
/**********************************************************************
NOMBRE:   PR_BAJA_MOVIMIENTO
PROPSITO:  Gestiona las bajas de los movimientos de los dispositivos de la empresa
PARMETROS:
Entrada:  p_id_movimiento: identificador del movimiento

Salida:  RSP
Pre-condicin: Existe el movimiento a dar de baja
Post-condicin: Se da de baja el movimiento
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el movimiento no existe en la base de datos
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_movimiento IN MOVIMIENTOS.ID_MOVIMIENTO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO				EXCEPTION;
E_MOVIMIENTO_NOEXISTE			EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_MOVIMIENTO: '||p_id_movimiento;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_movimiento) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM MOVIMIENTOS
WHERE ID_MOVIMIENTO = p_id_movimiento
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_MOVIMIENTO_NOEXISTE;
END;


-- Si ha pasado todas las validaciones, realiza el borrado

DELETE FROM MOVIMIENTOS_AUX
WHERE ID_MOVIMIENTO	 = p_id_movimiento;

DELETE FROM MOVIMIENTOS
WHERE ID_MOVIMIENTO	 = p_id_movimiento;


	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_MOVIMIENTO_NOEXISTE THEN
		RSP := 'ERROR: El movimiento no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_MOVIMIENTO;

/

-- MODIFICACIN MOVIMIENTOS
CREATE OR REPLACE
PROCEDURE PR_MODIF_MOVIMIENTO
/*********************************************************************
NOMBRE:   PR_MODIF_MOVIMIENTO
PROPSITO:  Gestiona las modificaciones de los movimientos de los dispositivos de la empresa
PARMETROS:
Entrada: p_id_movimiento:  identificador del movimiento
p_id_tipo_movimiento:  identificador del tipo de movimiento
p_id_dispositivo:  identificador del dispositivo sobre el que se ha realizado el movimiento
p_id_usuario_origen: identificador del usuario que tiene el dispositivo
p_id_usuario_destino: identificador del nuevo usuario al que se ha asignado el dispositivo
p_fecha_movimiento:  fecha del movimiento
p_id_responsable:  identificador del usuario responsable del movimiento	

Salida:  RSP
Pre-condicin: Existe el movimiento a modificar. Si se va a modificar el tipo de movimiento, 
se verifica que el nuevo tipo de movimiento exista en la BD. Si se va a modificar el 
dispositivo, se verifica que el nuevo dispositivo exista en la BD. Si se van a modificar 
los campos usuario_origen, usuario_destino o id_responsable, se comprobar que el usuario
exista en la BD y en el caso del responsable que pertenezca al departamento de informtica.
Post-condicin: Se modifica el movimiento
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador del movimiento a modificar no existe
-	ERROR:  el identificador del movimiento  no puede ser nulo
-	ERROR:  el identificador del tipo de movimiento  no existe
-	ERROR:  el identificador del dispositivo no existe
-	ERROR:  el usuario que tiene asignado el dispositivo no existe
-	ERROR:  el usuario al que se asigna el dispositivo no existe
-	ERROR:  el usuario responsable del movimiento no existe
-	ERROR:  el usuario responsable no pertenece al departamento de informtica
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_movimiento IN MOVIMIENTOS.ID_MOVIMIENTO%TYPE, p_id_tipo_movimiento IN MOVIMIENTOS.ID_TIPO_MOVIMIENTO%TYPE, 
p_id_dispositivo IN MOVIMIENTOS.ID_DISPOSITIVO%TYPE, 
p_id_usuario_origen IN MOVIMIENTOS.ID_USUARIO_ORIGEN%TYPE, 
p_id_usuario_destino IN MOVIMIENTOS.ID_USUARIO_DESTINO%TYPE, 
p_fecha_movimiento IN MOVIMIENTOS.FECHA_MOVIMIENTO%TYPE, 
p_id_responsable IN MOVIMIENTOS.ID_RESPONSABLE%TYPE, RSP OUT VARCHAR2)

IS

E_CAMPO_NULO			EXCEPTION;
E_MOVIMIENTO_NOEXISTE		EXCEPTION;
E_TIPOMOVIMIENTO_NOEXISTE	EXCEPTION;
E_DISPOSITIVO_NOEXISTE		EXCEPTION;
E_USUARIOORIGEN_NOEXISTE	EXCEPTION;
E_USUARIODESTINO_NOEXISTE	EXCEPTION;
E_RESPONSABLE_NOEXISTE	        EXCEPTION;
E_RESPONSABLE_NOINFORMATICO     EXCEPTION;


existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_MOVIMIENTO: '||p_id_movimiento||', P_ID_TIPO_MOVIMIENTO: '||p_id_tipo_movimiento||', 
P_ID_DISPOSITIVO: '||p_id_dispositivo||', P_ID_USUARIO_ORIGEN:  '||p_id_usuario_origen||', 
P_ID_USUARIO_DESTINO:  '||p_id_usuario_destino||', 
P_FECHA_MOVIMIENTO:  '||p_fecha_movimiento||', P_ID_RESPONSABLE: '||p_id_responsable;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_movimiento) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM MOVIMIENTOS
WHERE ID_MOVIMIENTO = p_id_movimiento
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_MOVIMIENTO_NOEXISTE;
END;


-- Si se va a modificar el tipo de movimiento, se verifica que exista en la base de datos

IF (p_id_tipo_movimiento is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM TIPO_MOVIMIENTO
WHERE ID_TIPO_MOVIMIENTO = p_id_tipo_movimiento
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_TIPOMOVIMIENTO_NOEXISTE;
END;
END IF;

-- Si se va a modificar el dispositivo al que corresponde el movimiento, se verifica que exista en la base de datos

IF (p_id_dispositivo is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM DISPOSITIVOS
WHERE ID_DISPOSITIVO = p_id_dispositivo
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_DISPOSITIVO_NOEXISTE;
END;
END IF;

-- Si se va a modificar el usuario origen, se verifica que exista en la base de datos

IF (p_id_usuario_origen is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_usuario_origen
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_USUARIOORIGEN_NOEXISTE;
END;
END IF;

-- Si se va a modificar el usuario destino, se verifica que exista en la base de datos

IF (p_id_usuario_destino is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_usuario_destino
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_USUARIODESTINO_NOEXISTE;
END;
END IF;


-- Si se va a modificar el usuario responsable, se verifica que exista en la base de datos

IF (p_id_responsable is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_responsable
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_RESPONSABLE_NOEXISTE;
END;
END IF;

-- Si se modifica el usuario responsable y este no pertenece al departamento de Informtica

IF (p_id_responsable is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_responsable
AND ID_DEPARTAMENTO IN
(SELECT ID_DEPARTAMENTO FROM DEPARTAMENTOS
WHERE NOMBRE_DEPARTAMENTO = 'INFORMATICA')
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_RESPONSABLE_NOINFORMATICO;
END;
END IF;


-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE MOVIMIENTOS_AUX
SET ID_TIPO_MOVIMIENTO = nvl(p_id_tipo_movimiento, ID_TIPO_MOVIMIENTO),
ID_DISPOSITIVO = nvl(p_id_dispositivo, ID_DISPOSITIVO),
ID_USUARIO_ORIGEN = nvl(p_id_usuario_origen, ID_USUARIO_ORIGEN),
ID_USUARIO_DESTINO = nvl(p_id_usuario_destino, ID_USUARIO_DESTINO),
FECHA_MOVIMIENTO = nvl(p_fecha_movimiento, FECHA_MOVIMIENTO),
ID_RESPONSABLE = nvl (p_id_responsable, ID_RESPONSABLE)
WHERE ID_MOVIMIENTO = p_id_movimiento;

UPDATE MOVIMIENTOS
SET ID_TIPO_MOVIMIENTO = nvl(p_id_tipo_movimiento, ID_TIPO_MOVIMIENTO),
ID_DISPOSITIVO = nvl(p_id_dispositivo, ID_DISPOSITIVO),
ID_USUARIO_ORIGEN = nvl(p_id_usuario_origen, ID_USUARIO_ORIGEN),
ID_USUARIO_DESTINO = nvl(p_id_usuario_destino, ID_USUARIO_DESTINO),
FECHA_MOVIMIENTO = nvl(p_fecha_movimiento, FECHA_MOVIMIENTO),
ID_RESPONSABLE = nvl (p_id_responsable, ID_RESPONSABLE)
WHERE ID_MOVIMIENTO = p_id_movimiento;


	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_MOVIMIENTO_NOEXISTE THEN
		RSP := 'ERROR: El movimiento a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;


WHEN E_TIPOMOVIMIENTO_NOEXISTE THEN
		RSP := 'ERROR: El tipo de movimiento no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_DISPOSITIVO_NOEXISTE THEN
		RSP := 'ERROR: El dispositivo al que corresponde el movimiento no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_USUARIOORIGEN_NOEXISTE THEN
		RSP := 'ERROR: El usuario que tiene asignado el dispositivo no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_USUARIODESTINO_NOEXISTE THEN
		RSP := 'ERROR: El usuario al que se asigna el dispositivo no existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_RESPONSABLE_NOEXISTE THEN
		RSP := 'ERROR: El usuario responsable no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;
        
  WHEN E_RESPONSABLE_NOINFORMATICO THEN
		RSP := 'ERROR: El usuario responsable no pertenece al Departamento de Informtica';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;      
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_MOVIMIENTO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_MOVIMIENTO;

/

-- ALTA INVENTARIOS

CREATE OR REPLACE
PROCEDURE PR_ALTA_INVENTARIOS
/***********************************************************************
NOMBRE:   PR_ALTA_INVENTARIOS
PROPSITO:  Gestiona los inventarios de material que se hacen en la empresa
PARMETROS:
Entrada: p_id_sede:  identificador de la sede donde se realiza el inventario
p_id_coordinador:  identificador del  usuario responsable de hacer el inventario
p_fecha_inventario:  fecha en la que se realiza el inventario

Salida:  RSP
Pre-condicin: No existe el inventario a dar de alta.  Tienen  que existir la sede 
donde se realiza el inventario y el usuario responsable.

Post-condicin: Se da de alta el inventario
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el inventario ya existe
-	ERROR:  la sede donde se ha realizado el inventario no existe
-	ERROR: el usuario responsable de realizar el inventario no existe
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_fecha_inventario IN INVENTARIOS.FECHA_INVENTARIO%TYPE, p_id_sede IN 
INVENTARIOS.ID_SEDE%TYPE, p_id_coordinador IN INVENTARIOS.ID_COORDINADOR%TYPE, RSP 
OUT VARCHAR2)
IS
E_CAMPO_NULO			EXCEPTION;
E_INVENTARIO_EXISTE		EXCEPTION;
E_SEDE_NOEXISTE			EXCEPTION;
E_COORDINADOR_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'FECHA_INVENTARIO: '||p_fecha_inventario||', P_ID_SEDE: 
'||p_id_sede||', P_ID_COORDINADOR:  '||p_id_coordinador;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_fecha_inventario) IS NULL OR
TRIM(p_id_sede) IS NULL OR
TRIM(p_id_coordinador) IS NULL)
 
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro no se haya insertado previamente con otro identificador

BEGIN
SELECT 'S'
INTO EXISTE
FROM INVENTARIOS
WHERE FECHA_INVENTARIO =p_fecha_inventario AND
ID_SEDE = p_id_sede AND
ID_COORDINADOR = p_id_coordinador
AND ROWNUM = 1;

RAISE E_INVENTARIO_EXISTE;

EXCEPTION
	WHEN NO_DATA_FOUND THEN NULL;
END;


-- Verifica que la sede donde se realiza el inventario existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM SEDES
WHERE ID_SEDE =p_id_sede
AND ROWNUM = 1;

EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_SEDE_NOEXISTE;
END;

-- Verifica que el usuario responsable de realizar el inventario existe

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO =p_id_coordinador
AND ROWNUM = 1;
EXCEPTION
	WHEN NO_DATA_FOUND THEN 
	RAISE E_COORDINADOR_NOEXISTE;
END;


-- Si ha pasado todas las validaciones, realiza la insercin

INSERT INTO INVENTARIOS (FECHA_INVENTARIO, ID_SEDE, ID_COORDINADOR)
VALUES (p_fecha_inventario, p_id_sede, p_id_coordinador);

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_INVENTARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_INVENTARIO_EXISTE THEN
		RSP := 'ERROR: El inventario ya ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_INVENTARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;


	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_INVENTARIOS', param_entrada, 'RSP:  '||RSP);

	WHEN E_SEDE_NOEXISTE THEN
		RSP := 'ERROR: La sede donde se ha realizado el inventario no 
existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_INVENTARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_COORDINADOR_NOEXISTE THEN
		RSP := 'ERROR: El usuario responsable de hacer el inventario no 
existe';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_INVENTARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_ALTA_INVENTARIOS', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_ALTA_INVENTARIOS;

/

-- BAJA INVENTARIOS

CREATE OR REPLACE
PROCEDURE PR_BAJA_INVENTARIO
/**********************************************************************
NOMBRE:   PR_BAJA_INVENTARIO
PROPSITO:  Gestiona las bajas de los inventarios que se realizan en la empresa
PARMETROS:
Entrada:  p_id_inventario: identificador del inventario

Salida:  RSP
Pre-condicin: Existe el inventario a dar de baja
Post-condicin: Se da de baja el inventario
RETORNA:
-	OK si la operacin se ha realizado con xito
-	ERROR:  el campo no puede ser nulo
-	ERROR:  el inventario no existe en la base de datos
-	 ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_inventario IN INVENTARIOS.ID_INVENTARIO%TYPE, RSP OUT VARCHAR2)

IS
E_CAMPO_NULO			EXCEPTION;
E_INVENTARIO_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN

param_entrada := 'P_ID_INVENTARIO: '||p_id_inventario;

-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_inventario) IS NULL)
THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM INVENTARIOS
WHERE ID_INVENTARIO = p_id_inventario
AND ROWNUM = 1;


EXCEPTION
	WHEN NO_DATA_FOUND THEN 

	RAISE E_INVENTARIO_NOEXISTE;
END;


-- Si ha pasado todas las validaciones, realiza el borrado

DELETE FROM INVENTARIOS
WHERE ID_INVENTARIO = p_id_inventario;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_INVENTARIO_NOEXISTE THEN
		RSP := 'ERROR: El inventario no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_BAJA_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_BAJA_INVENTARIO;

/

-- MODIFICACIN INVENTARIOS

CREATE OR REPLACE
PROCEDURE PR_MODIF_INVENTARIO 
/*********************************************************************
NOMBRE:   PR_MODIF_INVENTARIO
PROPSITO:  Gestiona las modificaciones de los inventarios que se realizan en la empresa
PARMETROS:
Entrada: p_id_inventario:  identificador del inventario
p_id_sede:  identificador de la sede donde se realiza el inventario
p_id_coordinador:  identificador del usuario responsable del inventario
p_fecha_inventario:  fecha en que se realiza el inventario

Salida:  RSP
Pre-condicin: Existe el inventario a modificar. Si se va a modificar la sede, se 
verifica que la nueva sede exista en la BD. Si se va a modificar el coordinador, 
se verifica que el nuevo coordinador exista en la BD.
Post-condicin: Se modifica el inventario
RETORNA:
-	OK si la operacin se ha realizado con xito
-	 ERROR:  el identificador del inventario a modificar no existe
-	ERROR:  el identificador del inventario  no puede ser nulo
-	ERROR:  el identificador de la sede no existe
-	ERROR:  el identificador del usuario responsable no existe
-	ERROR: tipo de error en caso de que el error no haya sido tratado
***************************************************************************/

(p_id_inventario IN INVENTARIOS.ID_INVENTARIO%TYPE,
p_fecha_inventario IN INVENTARIOS.FECHA_INVENTARIO%TYPE, 
p_id_sede IN INVENTARIOS.ID_SEDE%TYPE, p_id_coordinador IN INVENTARIOS.ID_COORDINADOR%TYPE, RSP OUT VARCHAR2)


IS
E_CAMPO_NULO			EXCEPTION;
E_INVENTARIO_NOEXISTE		EXCEPTION;
E_SEDE_NOEXISTE			EXCEPTION;
E_COORDINADOR_NOEXISTE		EXCEPTION;

existe		VARCHAR2(1);
param_entrada	VARCHAR2(500);
err_num		NUMBER;
err_msg		VARCHAR2(100);

BEGIN
param_entrada := 'P_ID_INVENTARIO: '||p_id_inventario||', P_FECHA_INVENTARIO: '||p_fecha_inventario||', P_ID_SEDE: '||p_id_sede||', P_ID_COORDINADOR:  '||p_id_coordinador;


-- Verifica que todos los campos obligatorios tengan valor

IF (TRIM(p_id_inventario) IS NULL) THEN
RAISE E_CAMPO_NULO;
END IF;

-- Verifica que el registro exista en la tabla

BEGIN
SELECT 'S'
INTO EXISTE
FROM INVENTARIOS
WHERE ID_INVENTARIO = p_id_inventario
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_INVENTARIO_NOEXISTE;
END;


-- Si se va a modificar la sede, se verifica que exista en la base de datos

if (p_id_sede is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM SEDES
WHERE ID_SEDE = p_id_sede
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_SEDE_NOEXISTE;
END;
END IF;

-- Si se va a modificar el usuario responsable del inventario, se verifica que exista en la base de datos

if (p_id_coordinador is NOT NULL) THEN

BEGIN
SELECT 'S'
INTO EXISTE
FROM USUARIOS
WHERE ID_USUARIO = p_id_coordinador
AND ROWNUM = 1;

EXCEPTION
WHEN NO_DATA_FOUND THEN
	RAISE E_COORDINADOR_NOEXISTE;
END;
END IF;

-- Si ha pasado todas las validaciones, actualiza el campo correspondiente

UPDATE INVENTARIOS
SET FECHA_INVENTARIO = nvl(p_fecha_inventario, FECHA_INVENTARIO),
ID_SEDE = nvl(p_id_sede, ID_SEDE),
ID_COORDINADOR = nvl(p_id_coordinador, ID_COORDINADOR)
WHERE ID_INVENTARIO = p_id_inventario;

	RSP := 'OK';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

EXCEPTION
	WHEN E_INVENTARIO_NOEXISTE THEN
		RSP := 'ERROR: El inventario a modificar no ha sido dado de alta';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

	WHEN E_CAMPO_NULO THEN
		RSP := 'ERROR: Un campo obligatorio no puede ser nulo';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;


WHEN E_SEDE_NOEXISTE THEN
		RSP := 'ERROR: La sede no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

WHEN E_COORDINADOR_NOEXISTE THEN
		RSP := 'ERROR: El usuario responsable del inventario no existe en la base de datos';

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SUBSTR(SQLERRM, 1, 200);

		RSP := 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

INSERT INTO LOGS (FECHA, PROCEDIMIENTO, PARAMSENTRADA, PARAMSSALIDA)
VALUES (SYSDATE, 'PR_MODIF_INVENTARIO', param_entrada, 'RSP:  '||RSP);

	COMMIT;

END PR_MODIF_INVENTARIO;

/


spool off;
