spool tablas.log

DROP TABLE TIPO_MOVIMIENTO CASCADE CONSTRAINTS;

CREATE TABLE TIPO_MOVIMIENTO
(
	ID_TIPO_MOVIMIENTO	NUMBER		NOT NULL,
	DESCRIPCION		VARCHAR2(40)	NOT NULL,
	FECHA_ALTA		DATE		NOT NULL,
	FECHA_BAJA		DATE
)
TABLESPACE TFG_DAT;

ALTER TABLE TIPO_MOVIMIENTO
ADD CONSTRAINT PK_TIPO_MOVIMIENTO PRIMARY KEY (ID_TIPO_MOVIMIENTO)
USING INDEX
TABLESPACE TFG_IND;


DROP TABLE TIPO_DISPOSITIVO CASCADE CONSTRAINTS;

CREATE TABLE TIPO_DISPOSITIVO
(
	ID_TIPO_DISPOSITIVO	NUMBER		NOT NULL,
	DESCRIPCION		VARCHAR2(30)	NOT NULL,
	FECHA_ALTA		DATE		NOT NULL,
	FECHA_BAJA		DATE		
)
TABLESPACE TFG_DAT;

ALTER TABLE TIPO_DISPOSITIVO
ADD CONSTRAINT PK_TIPO_DISPOSITIVO PRIMARY KEY (ID_TIPO_DISPOSITIVO)
USING INDEX
TABLESPACE TFG_IND;


DROP TABLE PROVEEDORES CASCADE CONSTRAINTS;

CREATE TABLE PROVEEDORES
(
	ID_PROVEEDOR		VARCHAR2(6)	NOT NULL,
	NOMBRE			VARCHAR2(60)	NOT NULL,
	DIRECCION		VARCHAR2(40),
	LOCALIDAD		VARCHAR2(30),
	PROVINCIA		VARCHAR2(30),
	PAIS			VARCHAR2(30),
	TELEFONO		NUMBER(9),
	EMAIL			VARCHAR2(30),
	PERIODO_ENTREGA		NUMBER		NOT NULL,
	FECHA_ALTA		DATE		NOT NULL,
	FECHA_BAJA		DATE
)
TABLESPACE TFG_DAT;

ALTER TABLE PROVEEDORES
ADD CONSTRAINT PK_PROVEEDORES PRIMARY KEY (ID_PROVEEDOR)
USING INDEX
TABLESPACE TFG_IND;


DROP TABLE SEDES CASCADE CONSTRAINTS;

CREATE TABLE SEDES
(
	ID_SEDE			VARCHAR2(3)	NOT NULL,
	NOMBRE_SEDE		VARCHAR2(60)	NOT NULL,
	DIRECCION_SEDE		VARCHAR2(40),
	LOCALIDAD_SEDE		VARCHAR2(30),
	PROVINCIA_SEDE		VARCHAR2(30),
	PAIS_SEDE		VARCHAR2(30),
	FECHA_ALTA		DATE		NOT NULL,
	FECHA_BAJA		DATE
)
TABLESPACE TFG_DAT;

ALTER TABLE SEDES 
ADD CONSTRAINT PK_SEDES PRIMARY KEY (ID_SEDE)
USING INDEX
TABLESPACE TFG_IND;

DROP TABLE DEPARTAMENTOS CASCADE CONSTRAINTS;

CREATE TABLE DEPARTAMENTOS
(
	ID_DEPARTAMENTO		VARCHAR(3)	NOT NULL,
	NOMBRE_DEPARTAMENTO	VARCHAR2(60)	NOT NULL,
	ID_RESPONSABLE		VARCHAR2(6),
	FECHA_ALTA		DATE		NOT NULL,
	FECHA_BAJA		DATE
)
TABLESPACE TFG_DAT;

ALTER TABLE DEPARTAMENTOS
ADD CONSTRAINT PK_DEPARTAMENTOS PRIMARY KEY (ID_DEPARTAMENTO)
USING INDEX
TABLESPACE TFG_IND;


DROP TABLE USUARIOS CASCADE CONSTRAINTS;

CREATE TABLE USUARIOS
(
	ID_USUARIO		VARCHAR2(6)	NOT NULL,
	ID_DEPARTAMENTO		VARCHAR2(30)	NOT NULL,
	NOMBRE			VARCHAR2(60)	NOT NULL,
	TELEFONO		NUMBER(9),	
	EMAIL			VARCHAR2(30),
	ID_SEDE			VARCHAR2(3)	NOT NULL,
	FECHA_ALTA		DATE		NOT NULL,
	FECHA_BAJA		DATE
)
TABLESPACE TFG_DAT;

ALTER TABLE USUARIOS
ADD CONSTRAINT PK_USUARIOS PRIMARY KEY (ID_USUARIO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE USUARIOS
ADD CONSTRAINT FK_USUARIO_DEPARTAMENTO FOREIGN KEY (ID_DEPARTAMENTO)
REFERENCES DEPARTAMENTOS (ID_DEPARTAMENTO);
ALTER TABLE USUARIOS
ADD CONSTRAINT FK_USUARIOS_SEDE FOREIGN KEY (ID_SEDE)
REFERENCES SEDES (ID_SEDE);

-- Esta clave fornea tiene que crearse despus de crear la tabla 
-- usuarios para evitar errores

ALTER TABLE DEPARTAMENTOS
ADD CONSTRAINT FK_DEPARTAMENTO_RESPONSABLE FOREIGN KEY (ID_RESPONSABLE)
REFERENCES USUARIOS (ID_USUARIO);


DROP TABLE DISPOSITIVOS CASCADE CONSTRAINTS;


CREATE TABLE DISPOSITIVOS
(
	ID_DISPOSITIVO		VARCHAR2(6)	NOT NULL,
	TIPO			NUMBER		NOT NULL,
	NUMERO_SERIE		VARCHAR2(15)	NOT NULL,
	FECHA_INSTALACION	DATE,
	ID_USUARIO		VARCHAR2(6),
	FECHA_COMPRA		DATE		NOT NULL,
	PRECIO_COMPRA		NUMBER(8,2)	NOT NULL,
	FECHA_ULTIMO_INVENTARIO	DATE,
	FECHA_ALTA		DATE		NOT NULL,
	FECHA_BAJA		DATE
)
TABLESPACE TFG_DAT;

ALTER TABLE DISPOSITIVOS
ADD CONSTRAINT PK_DISPOSITIVOS PRIMARY KEY (ID_DISPOSITIVO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE DISPOSITIVOS
ADD CONSTRAINT FK_DISPOSITIVO_TIPO FOREIGN KEY (TIPO)
REFERENCES TIPO_DISPOSITIVO (ID_TIPO_DISPOSITIVO);
ALTER TABLE DISPOSITIVOS
ADD CONSTRAINT  FK_DISPOSITIVO_USUARIO FOREIGN KEY (ID_USUARIO)
REFERENCES USUARIOS (ID_USUARIO);


DROP TABLE PEDIDOS CASCADE CONSTRAINTS;

CREATE TABLE PEDIDOS
(
	ID_PEDIDO		VARCHAR2(6)	NOT NULL,
	ID_PROVEEDOR		VARCHAR2(6)	NOT NULL,
	ID_DEPARTAMENTO		VARCHAR2(3)	NOT NULL,
	FECHA_PEDIDO		DATE		NOT NULL,
	IMPORTE_PEDIDO		NUMBER(10,2)	NOT NULL,
	ORDEN_INVERSION		VARCHAR2(20)	

)
TABLESPACE TFG_DAT;

ALTER TABLE PEDIDOS
ADD CONSTRAINT PK_PEDIDOS PRIMARY KEY (ID_PEDIDO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE PEDIDOS
ADD CONSTRAINT FK_PEDIDO_PROVEEDOR FOREIGN KEY (ID_PROVEEDOR)
REFERENCES PROVEEDORES (ID_PROVEEDOR);
ALTER TABLE PEDIDOS
ADD CONSTRAINT FK_PEDIDO_DEPARTAMENTO FOREIGN KEY (ID_DEPARTAMENTO)
REFERENCES DEPARTAMENTOS (ID_DEPARTAMENTO);



DROP TABLE LINEAS_PEDIDO CASCADE CONSTRAINTS;

CREATE TABLE LINEAS_PEDIDO
(
	ID_LINEA		NUMBER		NOT NULL,
	ID_PEDIDO		VARCHAR2(6)	NOT NULL,
	ID_MATERIAL		NUMBER		NOT NULL,
	UNIDADES		NUMBER		NOT NULL,
	PRECIO_UNITARIO		NUMBER(8,2)	NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE LINEAS_PEDIDO
ADD CONSTRAINT PK_LINEASPEDIDO PRIMARY KEY (ID_LINEA, ID_PEDIDO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE LINEAS_PEDIDO
ADD CONSTRAINT FK_LINEASPEDIDO_PEDIDO FOREIGN KEY (ID_PEDIDO)
REFERENCES PEDIDOS (ID_PEDIDO);
ALTER TABLE LINEAS_PEDIDO
ADD CONSTRAINT FK_LINEASPEDIDO_MATERIAL FOREIGN KEY (ID_MATERIAL)
REFERENCES TIPO_DISPOSITIVO (ID_TIPO_DISPOSITIVO);


DROP TABLE PRESUPUESTOS CASCADE CONSTRAINTS;

CREATE TABLE PRESUPUESTOS
(
	ID_PRESUPUESTO		VARCHAR2(6)	NOT NULL,
	ID_PROVEEDOR		VARCHAR2(6)	NOT NULL,
	ID_PEDIDO		VARCHAR2(6)	NOT NULL,
	FECHA_PRESUPUESTO	DATE		NOT NULL,
	IMPORTE			NUMBER(10,2)	NOT NULL,
	ACEPTADO		CHAR   CHECK (ACEPTADO IN ('N', 'S'))
)
TABLESPACE TFG_DAT;

ALTER TABLE PRESUPUESTOS
ADD CONSTRAINT PK_PRESUPUESTOS PRIMARY KEY (ID_PRESUPUESTO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE PRESUPUESTOS
ADD CONSTRAINT FK_PRESUPUESTO_PROVEEDOR FOREIGN KEY (ID_PROVEEDOR)
REFERENCES PROVEEDORES (ID_PROVEEDOR);
ALTER TABLE PRESUPUESTOS
ADD CONSTRAINT FK_PRESUPUESTO_PEDIDO FOREIGN KEY (ID_PEDIDO)
REFERENCES PEDIDOS (ID_PEDIDO);


DROP TABLE RECEPCIONES CASCADE CONSTRAINTS;

CREATE TABLE RECEPCIONES
(
	ID_RECEPCION		VARCHAR2(6)	NOT NULL,
	ID_PEDIDO		VARCHAR2(6)	NOT NULL,
	ID_RESPONSABLE		VARCHAR2(6)	NOT NULL,
	FECHA_RECEPCION		DATE		NOT NULL,
	OBSERVACIONES		VARCHAR2(100)
)
TABLESPACE TFG_DAT;

ALTER TABLE RECEPCIONES
ADD CONSTRAINT PK_RECEPCIONES PRIMARY KEY (ID_RECEPCION)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE RECEPCIONES
ADD CONSTRAINT FK_RECEPCION_PEDIDO FOREIGN KEY (ID_PEDIDO)
REFERENCES PEDIDOS (ID_PEDIDO);
ALTER TABLE RECEPCIONES
ADD CONSTRAINT FK_RECEPCION_RESPONSABLE FOREIGN KEY (ID_RESPONSABLE)
REFERENCES USUARIOS (ID_USUARIO);


DROP TABLE MOVIMIENTOS CASCADE CONSTRAINTS;

CREATE TABLE MOVIMIENTOS
(
	ID_MOVIMIENTO		VARCHAR2(6)	NOT NULL,
	ID_TIPO_MOVIMIENTO	NUMBER		NOT NULL,
	ID_DISPOSITIVO		VARCHAR2(6)	NOT NULL,
	ID_USUARIO_ORIGEN	VARCHAR2(6),
	ID_USUARIO_DESTINO	VARCHAR2(6),
	FECHA_MOVIMIENTO	DATE		NOT NULL,
	ID_RESPONSABLE		VARCHAR2(6)	NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE MOVIMIENTOS
ADD CONSTRAINT PK_MOVIMIENTOS PRIMARY KEY (ID_MOVIMIENTO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE MOVIMIENTOS
ADD CONSTRAINT FK_MOVIMIENTO_TIPO FOREIGN KEY (ID_TIPO_MOVIMIENTO)
REFERENCES TIPO_MOVIMIENTO (ID_TIPO_MOVIMIENTO);
ALTER TABLE MOVIMIENTOS
ADD CONSTRAINT FK_MOVIMIENTO_DISPOSITIVO FOREIGN KEY (ID_DISPOSITIVO)
REFERENCES DISPOSITIVOS (ID_DISPOSITIVO);
ALTER TABLE MOVIMIENTOS
ADD CONSTRAINT FK_MOVIMIENTO_USUARIOORIGEN FOREIGN KEY (ID_USUARIO_ORIGEN)
REFERENCES USUARIOS (ID_USUARIO);
ALTER TABLE MOVIMIENTOS
ADD CONSTRAINT FK_MOVIMIENTO_USUARIODESTINO FOREIGN KEY (ID_USUARIO_DESTINO)
REFERENCES USUARIOS (ID_USUARIO);
ALTER TABLE MOVIMIENTOS
ADD CONSTRAINT FK_MOVIMIENTO_USUARIORESP FOREIGN KEY (ID_RESPONSABLE)
REFERENCES USUARIOS (ID_USUARIO);


DROP TABLE INVENTARIOS CASCADE CONSTRAINTS;

CREATE TABLE INVENTARIOS
(
	ID_INVENTARIO		VARCHAR2(5)	NOT NULL,
	FECHA_INVENTARIO	DATE		NOT NULL,
	ID_SEDE			VARCHAR2(3)	NOT NULL,
	ID_COORDINADOR		VARCHAR2(6)	NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE INVENTARIOS
ADD CONSTRAINT PK_INVENTARIOS PRIMARY KEY (ID_INVENTARIO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE INVENTARIOS
ADD CONSTRAINT FK_INVENTARIO_SEDE FOREIGN KEY (ID_SEDE)
REFERENCES SEDES (ID_SEDE);
ALTER TABLE INVENTARIOS
ADD CONSTRAINT FK_INVENTARIO_USUARIO FOREIGN KEY (ID_COORDINADOR)
REFERENCES USUARIOS (ID_USUARIO);

-- REPOSITORIO ESTADISTICO

DROP TABLE ESTADISTICAS_AO CASCADE CONSTRAINTS;

CREATE TABLE ESTADISTICAS_AO
(
	AO			NUMBER		NOT NULL,
	NUM_DISPOSITIVOS	NUMBER		NOT NULL,
	IMPORTE_PEDIDOS		NUMBER(10,2)	NOT NULL,
	NUM_AVERIAS		NUMBER		NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE ESTADISTICAS_AO
ADD CONSTRAINT PK_ESTADISTICAS_AO PRIMARY KEY (AO)
USING INDEX
TABLESPACE TFG_IND;


DROP TABLE PORTATILES_SIN_AVERIA CASCADE CONSTRAINTS;

CREATE TABLE PORTATILES_SIN_AVERIA
(
	NUM_PORTATILES		NUMBER		NOT NULL,
	PORTATILES_SIN_AVERIA	NUMBER		NOT NULL,
	POR_SIN_AVERIA		NUMBER
)	

TABLESPACE TFG_DAT;


DROP TABLE PROVEEDOR_MAS_RAPIDO CASCADE CONSTRAINTS;

CREATE TABLE PROVEEDOR_MAS_RAPIDO
(
	ID_PROVEEDOR		VARCHAR2(6)	NOT NULL,
	NOMBRE			VARCHAR2(60)	NOT NULL,
	PERIODO_ENTREGA		NUMBER		NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE PROVEEDOR_MAS_RAPIDO
ADD CONSTRAINT PK_PROVEEDOR_MAS_RAPIDO PRIMARY KEY (ID_PROVEEDOR)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE PROVEEDOR_MAS_RAPIDO
ADD CONSTRAINT FK_PROVEEDOR_MAS_RAPIDO_PROV FOREIGN KEY (ID_PROVEEDOR)
REFERENCES PROVEEDORES (ID_PROVEEDOR);


DROP TABLE DISP_ACTIVOS_DEPARTAMENTO;

CREATE TABLE DISP_ACTIVOS_DEPARTAMENTO
(
	ID_DEPARTAMENTO		VARCHAR2(3)	NOT NULL,
	ID_TIPO_DISPOSITIVO	NUMBER		NOT NULL,
	NOMBRE_DEPARTAMENTO	VARCHAR2(60)	NOT NULL,
	DESCRIPCION_TIPO	VARCHAR2(30)	NOT NULL,
	NUM_DISPOSITIVOS	NUMBER
)
TABLESPACE TFG_DAT;

ALTER TABLE DISP_ACTIVOS_DEPARTAMENTO
ADD CONSTRAINT PK_DISP_ACT_DEPARTAMENTO PRIMARY KEY (ID_DEPARTAMENTO, ID_TIPO_DISPOSITIVO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE DISP_ACTIVOS_DEPARTAMENTO
ADD CONSTRAINT FK_DISP_ACT_DEP_DEPARTAMENTO FOREIGN KEY (ID_DEPARTAMENTO)
REFERENCES DEPARTAMENTOS (ID_DEPARTAMENTO);
ALTER TABLE DISP_ACTIVOS_DEPARTAMENTO 
ADD CONSTRAINT FK_DISP_ACT_DEP_TIPO_DISPOS FOREIGN KEY (ID_TIPO_DISPOSITIVO)
REFERENCES TIPO_DISPOSITIVO (ID_TIPO_DISPOSITIVO);


DROP TABLE INVENTARIO_DEPARTAMENTO CASCADE CONSTRAINTS;

CREATE TABLE INVENTARIO_DEPARTAMENTO
(
	ID_DEPARTAMENTO		VARCHAR2(3)	NOT NULL,
	NOMBRE_DEPARTAMENTO	VARCHAR2(60)	NOT NULL,
	NUM_TIPOS_DISPOSITIVOS	NUMBER		NOT NULL,
	VALOR_INVENTARIO	NUMBER		NOT NULL
)
TABLESPACE TFG_DAT;
ALTER TABLE INVENTARIO_DEPARTAMENTO
ADD CONSTRAINT PK_INVENTARIO_DEPARTAMENTO PRIMARY KEY (ID_DEPARTAMENTO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE INVENTARIO_DEPARTAMENTO
ADD CONSTRAINT FK_INVENTARIO_DEPT_DEPART FOREIGN KEY (ID_DEPARTAMENTO)
REFERENCES DEPARTAMENTOS (ID_DEPARTAMENTO);


DROP TABLE DEPTO_MAS_TIPOSDISP CASCADE CONSTRAINTS;

CREATE TABLE DEPTO_MAS_TIPOSDISP
(
	ID_DEPARTAMENTO		VARCHAR2(3)	NOT NULL,
	NOMBRE_DEPARTAMENTO	VARCHAR2(60)	NOT NULL,
	NUM_TIPOS_DISPOSITIVOS	NUMBER		NOT NULL
	
)
TABLESPACE TFG_DAT;
ALTER TABLE DEPTO_MAS_TIPOSDISP
ADD CONSTRAINT PK_DEPTO_MAS_TIPODISP_DEPART PRIMARY KEY (ID_DEPARTAMENTO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE DEPTO_MAS_TIPOSDISP
ADD CONSTRAINT FK_TIPODISP_DEPT_DEPART FOREIGN KEY (ID_DEPARTAMENTO)
REFERENCES DEPARTAMENTOS (ID_DEPARTAMENTO);


DROP TABLE DISPOSITIVOS_FECHA_INVENTARIO CASCADE CONSTRAINTS;

CREATE TABLE DISPOSITIVOS_FECHA_INVENTARIO
(
	NUM_DISPOSITIVOS	NUMBER		NOT NULL
)
TABLESPACE TFG_DAT;


DROP TABLE DEPTO_MAS_REASIGNACIONES CASCADE CONSTRAINTS;

CREATE TABLE DEPTO_MAS_REASIGNACIONES
(
	ID_DEPARTAMENTO		VARCHAR2(3)	NOT NULL,
	AO			NUMBER		NOT NULL,
	NOMBRE_DEPARTAMENTO	VARCHAR2(60)	NOT NULL,
	NUM_REASIGNACIONES	NUMBER		NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE DEPTO_MAS_REASIGNACIONES
ADD CONSTRAINT PK_DEP_MAS_REASIGNACIONES PRIMARY KEY (ID_DEPARTAMENTO, AO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE DEPTO_MAS_REASIGNACIONES
ADD CONSTRAINT FK_DEP_MAS_REASIG_DEPART FOREIGN KEY (ID_DEPARTAMENTO)
REFERENCES DEPARTAMENTOS (ID_DEPARTAMENTO);

DROP TABLE USUARIOS_DISPOSITIVOS CASCADE CONSTRAINTS;


CREATE TABLE USUARIOS_DISPOSITIVOS
(

	TOTAL_USUARIOS		NUMBER		NOT NULL,
	USUARIOS_SIN_DISPOSITIVOS  NUMBER	NOT NULL,
	POR_SIN_DISPOSITIVOS	NUMBER,
	USUARIOS_MAS5_DISPOSITIVOS NUMBER	NOT NULL
)
TABLESPACE TFG_DAT;


DROP TABLE UNIDADES_DISPOSITIVO_ALMACEN CASCADE CONSTRAINTS;

CREATE TABLE UNIDADES_DISPOSITIVO_ALMACEN
(
	ID_TIPO_DISPOSITIVO	NUMBER		NOT NULL,
	DESCRIPCION		VARCHAR2(30)	NOT NULL,
	NUM_UNIDADES		NUMBER		NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE UNIDADES_DISPOSITIVO_ALMACEN
ADD CONSTRAINT PK_UN_DISP_ALMACEN PRIMARY KEY (ID_TIPO_DISPOSITIVO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE UNIDADES_DISPOSITIVO_ALMACEN
ADD CONSTRAINT FK_UN_DISP_ALMACEN_TIPO_DISP FOREIGN KEY (ID_TIPO_DISPOSITIVO)
REFERENCES TIPO_DISPOSITIVO (ID_TIPO_DISPOSITIVO);


DROP TABLE USUARIOS_AVERIAS CASCADE CONSTRAINTS;

CREATE TABLE USUARIOS_AVERIAS
(
	ID_USUARIO		VARCHAR2(6)	NOT NULL,
	NOMBRE_USUARIO		VARCHAR2(60)	NOT NULL,
	NUM_AVERIAS		NUMBER		NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE USUARIOS_AVERIAS
ADD CONSTRAINT PK_USUARIOS_AVERIAS PRIMARY KEY (ID_USUARIO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE USUARIOS_AVERIAS
ADD CONSTRAINT FK_USUARIOS_AVERIAS_USUARIO FOREIGN KEY (ID_USUARIO)
REFERENCES USUARIOS (ID_USUARIO);


DROP TABLE USUARIOS_TOP10_AVERIAS CASCADE CONSTRAINTS;

CREATE TABLE USUARIOS_TOP10_AVERIAS
(
	ID_USUARIO		    VARCHAR2(6)	NOT NULL,
	NOMBRE_USUARIO		VARCHAR2(60)	NOT NULL,
	NUM_AVERIAS		NUMBER		NOT NULL,
	RANKING			NUMBER
)
TABLESPACE TFG_DAT;
ALTER TABLE USUARIOS_TOP10_AVERIAS
ADD CONSTRAINT PK_USUARIOS_TOP10_AVERIAS PRIMARY KEY (ID_USUARIO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE USUARIOS_TOP10_AVERIAS
ADD CONSTRAINT FK_USU_TOP10_AVERIAS_USUARIO FOREIGN KEY (ID_USUARIO)
REFERENCES USUARIOS (ID_USUARIO);


DROP TABLE LOGS CASCADE CONSTRAINTS;

CREATE TABLE LOGS
(	ID_LOG			NUMBER		NOT NULL,
	FECHA			DATE		NOT NULL,
	PROCEDIMIENTO		VARCHAR2(200)	NOT NULL,
	PARAMSENTRADA		VARCHAR2(2000)	NOT NULL,
	PARAMSSALIDA		VARCHAR2(2000) 	NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE LOGS
ADD CONSTRAINT PK_LOGS PRIMARY KEY (ID_LOG)
USING INDEX
TABLESPACE TFG_IND;

DROP TABLE PROVEEDORES_AUX CASCADE CONSTRAINTS;

CREATE TABLE PROVEEDORES_AUX
(
	ID_PROVEEDOR		VARCHAR2(6)	NOT NULL,
	NOMBRE			VARCHAR2(60)	NOT NULL,
	DIRECCION		VARCHAR2(40),
	LOCALIDAD		VARCHAR2(30),
	PROVINCIA		VARCHAR2(30),
	PAIS			VARCHAR2(30),
	TELEFONO		NUMBER(9),
	EMAIL			VARCHAR2(30),
	PERIODO_ENTREGA		NUMBER		NOT NULL,
	FECHA_ALTA		DATE		NOT NULL,
	FECHA_BAJA		DATE
)
TABLESPACE TFG_DAT;

ALTER TABLE PROVEEDORES_AUX
ADD CONSTRAINT PK_PROVEEDORES_AUX PRIMARY KEY (ID_PROVEEDOR)
USING INDEX
TABLESPACE TFG_IND;

DROP TABLE MOVIMIENTOS_AUX CASCADE CONSTRAINTS;

CREATE TABLE MOVIMIENTOS_AUX
(
	ID_MOVIMIENTO		VARCHAR2(6)	NOT NULL,
	ID_TIPO_MOVIMIENTO	NUMBER		NOT NULL,
	ID_DISPOSITIVO		VARCHAR2(6)	NOT NULL,
	ID_USUARIO_ORIGEN	VARCHAR2(6),
	ID_USUARIO_DESTINO	VARCHAR2(6),
	FECHA_MOVIMIENTO	DATE		NOT NULL,
	ID_RESPONSABLE		VARCHAR2(6)	NOT NULL
)
TABLESPACE TFG_DAT;

ALTER TABLE MOVIMIENTOS_AUX
ADD CONSTRAINT PK_MOVIMIENTOS_AUX PRIMARY KEY (ID_MOVIMIENTO)
USING INDEX
TABLESPACE TFG_IND;
ALTER TABLE MOVIMIENTOS_AUX
ADD CONSTRAINT FK_MOVIMIENTO_AUX_TIPO FOREIGN KEY (ID_TIPO_MOVIMIENTO)
REFERENCES TIPO_MOVIMIENTO (ID_TIPO_MOVIMIENTO);
ALTER TABLE MOVIMIENTOS_AUX
ADD CONSTRAINT FK_MOVIMIENTO_AUX_DISPOSITIVO FOREIGN KEY (ID_DISPOSITIVO)
REFERENCES DISPOSITIVOS (ID_DISPOSITIVO);
ALTER TABLE MOVIMIENTOS_AUX
ADD CONSTRAINT FK_MOV_AUX_USUARIOORIGEN FOREIGN KEY (ID_USUARIO_ORIGEN)
REFERENCES USUARIOS (ID_USUARIO);
ALTER TABLE MOVIMIENTOS_AUX
ADD CONSTRAINT FK_MOV_AUX_USUARIODESTINO FOREIGN KEY (ID_USUARIO_DESTINO)
REFERENCES USUARIOS (ID_USUARIO);
ALTER TABLE MOVIMIENTOS_AUX
ADD CONSTRAINT FK_MOV_AUX_USUARIORESP FOREIGN KEY (ID_RESPONSABLE)
REFERENCES USUARIOS (ID_USUARIO);

	
spool off;


